/* A recursive-descent parser generated by greg 0.4.3 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
struct _GREG;
#define YYRULECOUNT 266

/* PEG Markdown Highlight
 * Copyright 2011-2016 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_grammar.leg
 * 
 * This is a slightly adapted version of the PEG grammar from John MacFarlane's
 * peg-markdown compiler.
 */

/* PEG Markdown Highlight
 * Copyright 2011-2016 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_parser_head.c
 * 
 * Code to be inserted into the beginning of the parser code generated
 * from the PEG grammar.
 */

#include "pmh_parser.h"

#ifndef pmh_DEBUG_OUTPUT
#define pmh_DEBUG_OUTPUT 0
#endif

#if pmh_DEBUG_OUTPUT
#define pmh_IF(x)           if (x)
#define pmh_PRINTF(x, ...)  fprintf(stderr, x, ##__VA_ARGS__)
#define pmh_PUTCHAR(x)      putchar(x)
#else
#define pmh_IF(x)
#define pmh_PRINTF(x, ...)
#define pmh_PUTCHAR(x)
#endif



// Alias strdup to _strdup on MSVC:
#ifdef _MSC_VER
#define strdup _strdup
#endif


char *strdup_or_null(char *s)
{
    return (s == NULL) ? NULL : strdup(s);
}


// Internal language element occurrence structure, containing
// both public and private members:
struct pmh_RealElement
{
    // "Public" members:
    // (these must match what's defined in pmh_definitions.h)
    // -----------------------------------------------
    pmh_element_type type;
    unsigned long pos;
    unsigned long end;
    struct pmh_RealElement *next;
    char *label;
    char *address;
    
    // "Private" members for use by the parser itself:
    // -----------------------------------------------
    
    // next element in list of all elements:
    struct pmh_RealElement *all_elems_next;
    
    // offset to text (for elements of type pmh_EXTRA_TEXT, used when the
    // parser reads the value of 'text'):
    int text_offset;
    
    // text content (for elements of type pmh_EXTRA_TEXT):
    char *text;
    
    // children of element (for elements of type pmh_RAW_LIST)
    struct pmh_RealElement *children;
};
typedef struct pmh_RealElement pmh_realelement;




// Parser state data:
typedef struct
{
    /* The original, unmodified UTF-8 input: */
    char *original_input;
    
    /* The offsets of the bytes we have stripped from original_input: */
    unsigned long *strip_positions;
    size_t strip_positions_len;
    
    /* Buffer of characters to be parsed: */
    char *charbuf;
    
    /* Linked list of {start, end} offset pairs determining which parts */
    /* of charbuf to actually parse: */
    pmh_realelement *current_elem;
    pmh_realelement *elem_head;
    
    /* Current parsing offset within charbuf: */
    unsigned long offset;
    
    /* The extensions to use for parsing (bitfield */
    /* of enum pmh_extensions): */
    int extensions;
    
    /* Array of parsing result elements, indexed by type: */
    pmh_realelement **head_elems;
    
    /* Whether we are parsing only references: */
    bool parsing_only_references;
    
    /* List of reference elements: */
    pmh_realelement *references;
} parser_data;

static parser_data *mk_parser_data(char *original_input,
                                   unsigned long *strip_positions,
                                   size_t strip_positions_len,
                                   char *charbuf,
                                   pmh_realelement *parsing_elems,
                                   unsigned long offset,
                                   int extensions,
                                   pmh_realelement **head_elems,
                                   pmh_realelement *references)
{
    parser_data *p_data = (parser_data *)malloc(sizeof(parser_data));
    p_data->extensions = extensions;
    p_data->original_input = original_input;
    p_data->strip_positions = strip_positions;
    p_data->strip_positions_len = strip_positions_len;
    p_data->charbuf = charbuf;
    p_data->offset = offset;
    p_data->elem_head = p_data->current_elem = parsing_elems;
    p_data->references = references;
    p_data->parsing_only_references = false;
    if (head_elems != NULL)
        p_data->head_elems = head_elems;
    else {
        p_data->head_elems = (pmh_realelement **)
                             malloc(sizeof(pmh_realelement *) * pmh_NUM_TYPES);
        int i;
        for (i = 0; i < pmh_NUM_TYPES; i++)
            p_data->head_elems[i] = NULL;
    }
    return p_data;
}


// Forward declarations
static void parse_markdown(parser_data *p_data);
static void parse_references(parser_data *p_data);





static char **get_element_type_names()
{
    static char **elem_type_names = NULL;
    if (elem_type_names == NULL)
    {
        elem_type_names = (char **)malloc(sizeof(char*) * pmh_NUM_LANG_TYPES);
        int i;
        for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
            elem_type_names[i] = NULL;
        elem_type_names[pmh_LINK] = "LINK";
        elem_type_names[pmh_AUTO_LINK_URL] = "AUTO_LINK_URL";
        elem_type_names[pmh_AUTO_LINK_EMAIL] = "AUTO_LINK_EMAIL";
        elem_type_names[pmh_IMAGE] = "IMAGE";
        elem_type_names[pmh_CODE] = "CODE";
        elem_type_names[pmh_HTML] = "HTML";
        elem_type_names[pmh_HTML_ENTITY] = "HTML_ENTITY";
        elem_type_names[pmh_EMPH] = "EMPH";
        elem_type_names[pmh_STRONG] = "STRONG";
        elem_type_names[pmh_LIST_BULLET] = "LIST_BULLET";
        elem_type_names[pmh_LIST_ENUMERATOR] = "LIST_ENUMERATOR";
        elem_type_names[pmh_COMMENT] = "COMMENT";
        elem_type_names[pmh_H1] = "H1";
        elem_type_names[pmh_H2] = "H2";
        elem_type_names[pmh_H3] = "H3";
        elem_type_names[pmh_H4] = "H4";
        elem_type_names[pmh_H5] = "H5";
        elem_type_names[pmh_H6] = "H6";
        elem_type_names[pmh_BLOCKQUOTE] = "BLOCKQUOTE";
        elem_type_names[pmh_VERBATIM] = "VERBATIM";
        elem_type_names[pmh_HTMLBLOCK] = "HTMLBLOCK";
        elem_type_names[pmh_HRULE] = "HRULE";
        elem_type_names[pmh_REFERENCE] = "REFERENCE";
        elem_type_names[pmh_FENCEDCODEBLOCK] = "FENCEDCODEBLOCK";
        elem_type_names[pmh_NOTE] = "NOTE";
        elem_type_names[pmh_STRIKE] = "STRIKE";
        elem_type_names[pmh_FRONTMATTER] = "FRONTMATTER";
        elem_type_names[pmh_DISPLAYFORMULA] = "DISPLAYFORMULA";
        elem_type_names[pmh_INLINEEQUATION] = "INLINEEQUATION";
        elem_type_names[pmh_MARK] = "MARK";
        elem_type_names[pmh_TABLE] = "TABLE";
        elem_type_names[pmh_TABLEHEADER] = "TABLEHEADER";
        elem_type_names[pmh_TABLEBORDER] = "TABLEBORDER";
    }
    return elem_type_names;
}

pmh_element_type pmh_element_type_from_name(char *name)
{
    char **elem_type_names = get_element_type_names();
    
    int i;
    for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
    {
        char *i_name = elem_type_names[i];
        if (i_name == NULL)
            continue;
        if (strcmp(i_name, name) == 0)
            return i;
    }
    
    return pmh_NO_TYPE;
}

char *pmh_element_name_from_type(pmh_element_type type)
{
    char **elem_type_names = get_element_type_names();
    char* ret = elem_type_names[type];
    if (ret == NULL)
        return "unknown type";
    return ret;
}




/*
Remove pmh_RAW elements with zero length; return pointer
to new head.
*/
static pmh_realelement *remove_zero_length_raw_spans(pmh_realelement *elem)
{
    pmh_realelement *head = elem;
    pmh_realelement *parent = NULL;
    pmh_realelement *c = head;
    while (c != NULL)
    {
        if (c->type == pmh_RAW && c->pos >= c->end)
        {
            if (parent != NULL)
                parent->next = c->next;
            else
                head = c->next;
            parent = c;
            c = c->next;
            continue;
        }
        parent = c;
        c = c->next;
    }
    return head;
}

#if pmh_DEBUG_OUTPUT
/*
Print null-terminated string s.t. some characters are
represented by their corresponding espace sequences
*/
static void print_str_literal_escapes(char *str)
{
    char *c = str;
    pmh_PRINTF("'");
    while (*c != '\0')
    {
        if (*c == '\n')         pmh_PRINTF("\\n");
        else if (*c == '\t')    pmh_PRINTF("\\t");
        else putchar(*c);
        c++;
    }
    pmh_PRINTF("'");
}
#endif

#if pmh_DEBUG_OUTPUT
/*
Print elements in a linked list of
pmh_RAW, pmh_SEPARATOR, pmh_EXTRA_TEXT elements
*/
static void print_raw_spans_inline(pmh_realelement *elem)
{
    pmh_realelement *cur = elem;
    while (cur != NULL)
    {
        if (cur->type == pmh_SEPARATOR)
            pmh_PRINTF("<pmh_SEP %ld> ", cur->pos);
        else if (cur->type == pmh_EXTRA_TEXT) {
            pmh_PRINTF("{pmh_ETEXT ");
            print_str_literal_escapes(cur->text);
            pmh_PRINTF("}");
        }
        else
            pmh_PRINTF("(%ld-%ld) ", cur->pos, cur->end);
        cur = cur->next;
    }
}
#endif

/*
Perform postprocessing parsing runs for pmh_RAW_LIST elements in `elem`,
iteratively until no such elements exist.
*/
static void process_raw_blocks(parser_data *p_data)
{
    pmh_PRINTF("--------process_raw_blocks---------\n");
    while (p_data->head_elems[pmh_RAW_LIST] != NULL)
    {
        pmh_PRINTF("new iteration.\n");
        pmh_realelement *cursor = p_data->head_elems[pmh_RAW_LIST];
        p_data->head_elems[pmh_RAW_LIST] = NULL;
        while (cursor != NULL)
        {
            pmh_realelement *span_list = (pmh_realelement*)cursor->children;
            
            span_list = remove_zero_length_raw_spans(span_list);
            
            #if pmh_DEBUG_OUTPUT
            pmh_PRINTF("  process: ");
            print_raw_spans_inline(span_list);
            pmh_PRINTF("\n");
            #endif
            
            while (span_list != NULL)
            {
                pmh_PRINTF("next: span_list: %ld-%ld\n",
                           span_list->pos, span_list->end);
                
                // Skip separators in the beginning, as well as
                // separators after another separator:
                if (span_list->type == pmh_SEPARATOR) {
                    span_list = span_list->next;
                    continue;
                }
                
                // Store list of spans until next separator in subspan_list:
                pmh_realelement *subspan_list = span_list;
                pmh_realelement *previous = NULL;
                while (span_list != NULL && span_list->type != pmh_SEPARATOR) {
                    previous = span_list;
                    span_list = span_list->next;
                }
                if (span_list != NULL && span_list->type == pmh_SEPARATOR) {
                    span_list = span_list->next;
                    previous->next = NULL;
                }
                
                #if pmh_DEBUG_OUTPUT
                pmh_PRINTF("    subspan process: ");
                print_raw_spans_inline(subspan_list);
                pmh_PRINTF("\n");
                #endif
                
                // Process subspan_list:
                parser_data *raw_p_data = mk_parser_data(
                    p_data->original_input,
                    p_data->strip_positions,
                    p_data->strip_positions_len,
                    p_data->charbuf,
                    subspan_list,
                    subspan_list->pos,
                    p_data->extensions,
                    p_data->head_elems,
                    p_data->references
                );
                parse_markdown(raw_p_data);
                free(raw_p_data);
                
                pmh_PRINTF("parse over\n");
            }
            
            cursor = cursor->next;
        }
    }
}


#if pmh_DEBUG_OUTPUT
static void print_raw_blocks(char *text, pmh_realelement *elem[])
{
    pmh_PRINTF("--------print_raw_blocks---------\n");
    pmh_PRINTF("block:\n");
    pmh_realelement *cursor = elem[pmh_RAW_LIST];
    while (cursor != NULL)
    {
        print_raw_spans_inline(cursor->children);
        cursor = cursor->next;
    }
}
#endif




/* Free all elements created while parsing */
void pmh_free_elements(pmh_element **elems)
{
    pmh_realelement *cursor = (pmh_realelement*)elems[pmh_ALL];
    while (cursor != NULL) {
        pmh_realelement *tofree = cursor;
        cursor = cursor->all_elems_next;
        if (tofree->text != NULL)
            free(tofree->text);
        if (tofree->label != NULL)
            free(tofree->label);
        if (tofree->address != NULL)
            free(tofree->address);
        free(tofree);
    }
    elems[pmh_ALL] = NULL;
    
    free(elems);
}






#define IS_CONTINUATION_BYTE(x) ((x & 0xC0) == 0x80)
#define HAS_UTF8_BOM(x)         ( ((*x & 0xFF) == 0xEF)\
                                  && ((*(x+1) & 0xFF) == 0xBB)\
                                  && ((*(x+2) & 0xFF) == 0xBF) )
#define ADD_STRIP_POS(x) \
    /* reallocate more space for the array, if needed: */ \
    if (strip_positions_size <= strip_positions_pos) { \
        size_t new_size = strip_positions_size * 2; \
        unsigned long *new_arr = (unsigned long *) \
                                 calloc(new_size, \
                                        sizeof(unsigned long)); \
        memcpy(new_arr, strip_positions, \
               (sizeof(unsigned long) * strip_positions_size)); \
        strip_positions_size = new_size; \
        free(strip_positions); \
        strip_positions = new_arr; \
    } \
    strip_positions[strip_positions_pos] = x; \
    strip_positions_pos++;

/*
Copy `str` to `out`, while doing the following:
  - remove UTF-8 continuation bytes
  - remove possible UTF-8 BOM (byte order mark)
  - append two newlines to the end (like peg-markdown does)
  - keep track of which bytes we have stripped (in strip_positions)
*/
static int strcpy_preformat(char *str, char **out,
                            unsigned long **out_strip_positions,
                            size_t *out_strip_positions_len)
{
    size_t strip_positions_size = 1024;
    size_t strip_positions_pos = 0;
    unsigned long *strip_positions = (unsigned long *)
                                     calloc(strip_positions_size,
                                            sizeof(unsigned long));
    
    
    // +2 in the following is due to the "\n\n" suffix:
    char *new_str = (char *)malloc(sizeof(char) * strlen(str) + 1 + 2);
    char *c = str;
    int i = 0;
    
    if (HAS_UTF8_BOM(c)) {
        c += 3;
        ADD_STRIP_POS(0);
        ADD_STRIP_POS(1);
        ADD_STRIP_POS(2);
    }
    
    while (*c != '\0')
    {
        if (!IS_CONTINUATION_BYTE(*c)) {
            *(new_str+i) = *c, i++;
        } else {
            ADD_STRIP_POS((int)(c-str));
        }
        c++;
    }
    
    *(new_str+(i++)) = '\n';
    *(new_str+(i++)) = '\n';
    *(new_str+i) = '\0';
    
    *out = new_str;
    *out_strip_positions = strip_positions;
    *out_strip_positions_len = strip_positions_pos;
    return i;
}



void pmh_markdown_to_elements(char *text, int extensions,
                              pmh_element **out_result[])
{
    char *text_copy = NULL;
    unsigned long *strip_positions = NULL;
    size_t strip_positions_len = 0;
    int text_copy_len = strcpy_preformat(text, &text_copy, &strip_positions,
                                         &strip_positions_len);
    
    pmh_realelement *parsing_elem = (pmh_realelement *)
                                    malloc(sizeof(pmh_realelement));
    parsing_elem->type = pmh_RAW;
    parsing_elem->pos = 0;
    parsing_elem->end = text_copy_len;
    parsing_elem->next = NULL;
    
    parser_data *p_data = mk_parser_data(
        text,
        strip_positions,
        strip_positions_len,
        text_copy,
        parsing_elem,
        0,
        extensions,
        NULL,
        NULL
    );
    pmh_realelement **result = p_data->head_elems;
    
    if (*text_copy != '\0')
    {
        // Get reference definitions into p_data->references
        parse_references(p_data);
        
        // Reset parser state to beginning of input
        p_data->offset = 0;
        p_data->current_elem = p_data->elem_head;
        
        // Parse whole document
        parse_markdown(p_data);
        
        #if pmh_DEBUG_OUTPUT
        print_raw_blocks(text_copy, result);
        #endif
        
        process_raw_blocks(p_data);
    }
    
    free(strip_positions);
    free(p_data);
    free(parsing_elem);
    free(text_copy);
    
    *out_result = (pmh_element**)result;
}



/*
Mergesort linked list of elements (using comparison function `compare`),
return new head. (Adapted slightly from Simon Tatham's algorithm.)
*/
static pmh_element *ll_mergesort(pmh_element *list,
                                 int (*compare)(const pmh_element*,
                                                const pmh_element*))
{
    if (!list)
        return NULL;
    
    pmh_element *out_head = list;
    
    /* Merge widths of doubling size until done */
    int merge_width = 1;
    while (1)
    {
        pmh_element *l, *r; /* left & right segment pointers */
        pmh_element *tail = NULL; /* tail of sorted section */
        
        l = out_head;
        out_head = NULL;
        
        int merge_count = 0;
        
        while (l)
        {
            merge_count++;
            
            /* Position r, determine lsize & rsize */
            r = l;
            int lsize = 0;
            int i;
            for (i = 0; i < merge_width; i++) {
                lsize++;
                r = r->next;
                if (!r)
                    break;
            }
            int rsize = merge_width;
            
            /* Merge l & r */
            while (lsize > 0 || (rsize > 0 && r))
            {
                bool get_from_left = false;
                if (lsize == 0)             get_from_left = false;
                else if (rsize == 0 || !r)  get_from_left = true;
                else if (compare(l,r) <= 0) get_from_left = true;
                
                pmh_element *e;
                if (get_from_left) {
                    e = l; l = l->next; lsize--;
                } else {
                    e = r; r = r->next; rsize--;
                }
                
                /* add the next pmh_element to the merged list */
                if (tail)
                    tail->next = e;
                else
                    out_head = e;
                tail = e;
            }
            
            l = r;
        }
        tail->next = NULL;
        
        if (merge_count <= 1)
            return out_head;
        
        merge_width *= 2;
    }
}

static int elem_compare_by_pos(const pmh_element *a, const pmh_element *b)
{
    return a->pos - b->pos;
}

void pmh_sort_elements_by_pos(pmh_element *element_lists[])
{
    int i;
    for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
        element_lists[i] = ll_mergesort(element_lists[i], &elem_compare_by_pos);
}








/* return true if extension is selected */
static bool extension(parser_data *p_data, int ext)
{
    return ((p_data->extensions & ext) != 0);
}

/* return reference pmh_realelement for a given label */
static pmh_realelement *get_reference(parser_data *p_data, char *label)
{
    if (!label)
        return NULL;
    
    pmh_realelement *cursor = p_data->references;
    while (cursor != NULL)
    {
        if (cursor->label && strcmp(label, cursor->label) == 0)
            return cursor;
        cursor = cursor->next;
    }
    return NULL;
}


/* cons an element/list onto a list, returning pointer to new head */
static pmh_realelement *cons(pmh_realelement *elem, pmh_realelement *list)
{
    assert(elem != NULL);
    
    pmh_realelement *cur = elem;
    while (cur->next != NULL) {
        cur = cur->next;
    }
    cur->next = list;
    
    return elem;
}


/* reverse a list, returning pointer to new list */
static pmh_realelement *reverse(pmh_realelement *list)
{
    pmh_realelement *new_head = NULL;
    pmh_realelement *next = NULL;
    while (list != NULL) {
        next = list->next;
        list->next = new_head;
        new_head = list;
        list = next;
    }
    return new_head;
}



/* construct pmh_realelement */
static pmh_realelement *mk_element(parser_data *p_data, pmh_element_type type,
                                   long pos, long end)
{
    pmh_realelement *result = (pmh_realelement *)malloc(sizeof(pmh_realelement));
    memset(result, 0, sizeof(*result));
    result->type = type;
    result->pos = pos;
    result->end = end;
    
    pmh_realelement *old_all_elements_head = p_data->head_elems[pmh_ALL];
    p_data->head_elems[pmh_ALL] = result;
    result->all_elems_next = old_all_elements_head;
    
    //pmh_PRINTF("  mk_element: %s [%ld - %ld]\n", pmh_element_name_from_type(type), pos, end);
    
    return result;
}

static pmh_realelement *copy_element(parser_data *p_data, pmh_realelement *elem)
{
    pmh_realelement *result = mk_element(p_data, elem->type, elem->pos, elem->end);
    result->label = strdup_or_null(elem->label);
    result->text = strdup_or_null(elem->text);
    result->address = strdup_or_null(elem->address);
    return result;
}

/* construct pmh_EXTRA_TEXT pmh_realelement */
static pmh_realelement *mk_etext(parser_data *p_data, char *string)
{
    pmh_realelement *result;
    assert(string != NULL);
    result = mk_element(p_data, pmh_EXTRA_TEXT, 0,0);
    result->text = strdup_or_null(string);
    return result;
}


/*
Given an element where the offsets {pos, end} represent
locations in the *parsed text* (defined by the linked list of pmh_RAW and
pmh_EXTRA_TEXT elements in p_data->current_elem), fix these offsets to represent
corresponding offsets in the parse buffer (p_data->charbuf). Also split
the given pmh_realelement into multiple parts if its offsets span multiple
p_data->current_elem elements. Return the (list of) elements with real offsets.
*/
static pmh_realelement *fix_offsets(parser_data *p_data, pmh_realelement *elem)
{
    if (elem->type == pmh_EXTRA_TEXT)
        return mk_etext(p_data, elem->text);
    
    pmh_realelement *new_head = copy_element(p_data, elem);
    
    pmh_realelement *tail = new_head;
    pmh_realelement *prev = NULL;
    
    bool found_start = false;
    bool found_end = false;
    bool tail_needs_pos = false;
    unsigned long previous_end = 0;
    unsigned long c = 0;
    
    pmh_realelement *cursor = p_data->elem_head;
    while (cursor != NULL)
    {
        int thislen = (cursor->type == pmh_EXTRA_TEXT)
                        ? strlen(cursor->text)
                        : cursor->end - cursor->pos;
        
        if (tail_needs_pos && cursor->type != pmh_EXTRA_TEXT) {
            tail->pos = cursor->pos;
            tail_needs_pos = false;
        }
        
        unsigned int this_pos = cursor->pos;
        
        if (!found_start && (c <= elem->pos && elem->pos <= c+thislen)) {
            tail->pos = (cursor->type == pmh_EXTRA_TEXT)
                        ? previous_end
                        : cursor->pos + (elem->pos - c);
            this_pos = tail->pos;
            found_start = true;
        }
        
        if (!found_end && (c <= elem->end && elem->end <= c+thislen)) {
            tail->end = (cursor->type == pmh_EXTRA_TEXT)
                        ? previous_end
                        : cursor->pos + (elem->end - c);
            found_end = true;
        }
        
        if (found_start && found_end)
            break;
        
        if (cursor->type != pmh_EXTRA_TEXT)
            previous_end = cursor->end;
        
        if (found_start) {
            pmh_realelement *new_elem = copy_element(p_data, tail);
            new_elem->pos = this_pos;
            new_elem->end = cursor->end;
            new_elem->next = tail;
            if (prev != NULL)
                prev->next = new_elem;
            if (new_head == tail)
                new_head = new_elem;
            prev = new_elem;
            tail_needs_pos = true;
        }
        
        c += thislen;
        cursor = cursor->next;
    }
    
    return new_head;
}



/* Add an element to p_data->head_elems. */
static void add(parser_data *p_data, pmh_realelement *elem)
{
    if (elem->type != pmh_RAW_LIST)
    {
        pmh_PRINTF("  add: %s [%ld - %ld]\n",
                   pmh_element_name_from_type(elem->type), elem->pos, elem->end);
        elem = fix_offsets(p_data, elem);
        pmh_PRINTF("     > %s [%ld - %ld]\n",
                   pmh_element_name_from_type(elem->type), elem->pos, elem->end);
    }
    else
    {
        pmh_PRINTF("  add: pmh_RAW_LIST ");
        pmh_realelement *cursor = elem->children;
        pmh_realelement *previous = NULL;
        while (cursor != NULL)
        {
            pmh_realelement *next = cursor->next;
            pmh_PRINTF("(%ld-%ld)>", cursor->pos, cursor->end);
            pmh_realelement *new_cursor = fix_offsets(p_data, cursor);
            if (previous != NULL)
                previous->next = new_cursor;
            else
                elem->children = new_cursor;
            pmh_PRINTF("(%ld-%ld)", new_cursor->pos, new_cursor->end);
            while (new_cursor->next != NULL) {
                new_cursor = new_cursor->next;
                pmh_PRINTF("(%ld-%ld)", new_cursor->pos, new_cursor->end);
            }
            pmh_PRINTF(" ");
            if (next != NULL)
                new_cursor->next = next;
            previous = new_cursor;
            cursor = next;
        }
        pmh_PRINTF("\n");
    }
    
    if (p_data->head_elems[elem->type] == NULL)
        p_data->head_elems[elem->type] = elem;
    else
    {
        pmh_realelement *last = elem;
        while (last->next != NULL)
            last = last->next;
        last->next = p_data->head_elems[elem->type];
        p_data->head_elems[elem->type] = elem;
    }
}


// Given a range in the list of spans we use for parsing (pos, end), return
// a copy of the corresponding section in the original input, with all of
// the UTF-8 bytes intact:
static char *copy_input_span(parser_data *p_data,
                             unsigned long pos, unsigned long end)
{
    if (end <= pos)
        return NULL;
    
    char *ret = NULL;
    
    // Adjust (pos,end) to match actual indexes in charbuf:
    pmh_realelement *dummy = mk_element(p_data, pmh_NO_TYPE, pos, end);
    pmh_realelement *fixed_dummies = fix_offsets(p_data, dummy);
    pmh_realelement *cursor = fixed_dummies;
    while (cursor != NULL)
    {
        if (cursor->end <= cursor->pos)
        {
            cursor = cursor->next;
            continue;
        }
        
        // Adjust cursor's span to take bytes stripped from the original
        // input into account (i.e. match the corresponding span in
        // p_data->original_input):
        unsigned long adjusted_pos = cursor->pos;
        unsigned long adjusted_end = cursor->end;
        size_t i;
        for (i = 0; i < p_data->strip_positions_len; i++)
        {
            unsigned long strip_position = p_data->strip_positions[i];
            if (strip_position <= adjusted_pos)
                adjusted_pos++;
            if (strip_position <= adjusted_end)
                adjusted_end++;
            else
                break;
        }
        
        // Copy span from original input:
        size_t adjusted_len = adjusted_end - adjusted_pos;
        char *str = (char *)malloc(sizeof(char)*adjusted_len + 1);
        *str = '\0';
        strncat(str, (p_data->original_input + adjusted_pos), adjusted_len);
        
        if (ret == NULL)
            ret = str;
        else
        {
            // append str to ret:
            char *new_ret = (char *)malloc(sizeof(char)
                                           *(strlen(str) + strlen(ret)) + 1);
            *new_ret = '\0';
            strcat(new_ret, ret);
            strcat(new_ret, str);
            free(ret);
            free(str);
            ret = new_ret;
        }
        
        cursor = cursor->next;
    }
    
    return ret;
}





# define YYSTYPE pmh_realelement *
#ifdef __DEBUG__
# define YY_DEBUG 1
#endif

#define YY_INPUT(buf, result, max_size)\
        yy_input_func(buf, &result, max_size, (parser_data *)G->data)

static void yy_input_func(char *buf, int *result, int max_size,
                          parser_data *p_data)
{
    if (p_data->current_elem == NULL)
    {
        (*result) = 0;
        return;
    }
    
    if (p_data->current_elem->type == pmh_EXTRA_TEXT)
    {
        int yyc;
        bool moreToRead = (p_data->current_elem->text
                           && *(p_data->current_elem->text
                                + p_data->current_elem->text_offset) != '\0');
        if (moreToRead)
        {
            yyc = *(p_data->current_elem->text + p_data->current_elem->text_offset++);
            pmh_PRINTF("\e[47;30m"); pmh_PUTCHAR(yyc); pmh_PRINTF("\e[0m");
            pmh_IF(yyc == '\n') pmh_PRINTF("\e[47m \e[0m");
        }
        else
        {
            yyc = EOF;
            p_data->current_elem = p_data->current_elem->next;
            pmh_PRINTF("\e[41m \e[0m");
            if (p_data->current_elem != NULL)
                p_data->offset = p_data->current_elem->pos;
        }
        (*result) = (EOF == yyc) ? 0 : (*(buf) = yyc, 1);
        return;
    }
    
    *(buf) = *(p_data->charbuf + p_data->offset);
    (*result) = (*buf != '\0');
    p_data->offset++;
    
    pmh_PRINTF("\e[43;30m"); pmh_PUTCHAR(*buf); pmh_PRINTF("\e[0m");
    pmh_IF(*buf == '\n') pmh_PRINTF("\e[42m \e[0m");
    
    if (p_data->offset >= p_data->current_elem->end)
    {
        p_data->current_elem = p_data->current_elem->next;
        pmh_PRINTF("\e[41m \e[0m");
        if (p_data->current_elem != NULL)
            p_data->offset = p_data->current_elem->pos;
    }
}



#define elem(x)     mk_element((parser_data *)G->data, x, thunk->begin, thunk->end)
#define elem_s(x)   mk_element((parser_data *)G->data, x, s->pos, thunk->end)
#define mk_sep      mk_element((parser_data *)G->data, pmh_SEPARATOR, 0,0)
#define mk_notype   mk_element((parser_data *)G->data, pmh_NO_TYPE, 0,0)
#define etext(x)    mk_etext((parser_data *)G->data, x)
#define ADD(x)      add((parser_data *)G->data, x)
#define EXT(x)      extension((parser_data *)G->data, x)
#define REF_EXISTS(x) reference_exists((parser_data *)G->data, x)
#define GET_REF(x)  get_reference((parser_data *)G->data, x)
#define PARSING_REFERENCES ((parser_data *)G->data)->parsing_only_references
#define FREE_LABEL(l) { free(l->label); l->label = NULL; }
#define FREE_ADDRESS(l) { free(l->address); l->address = NULL; }

// This gives us the text matched with < > as it appears in the original input:
#define COPY_YYTEXT_ORIG() copy_input_span((parser_data *)G->data, thunk->begin, thunk->end)

// pos is the postition of next char to parse after matching one $
// Do not allow $, digit, and \ before opening $
bool inline_equation_predict_pre(char *buf, int pos)
{
    if (pos < 2) {
        return true;
    }

    unsigned char ch = buf[pos - 2];
    return ch != '$' && (ch < '0' || ch > '9') && ch != '\\';
}

// Do not allow \, space before ending $
bool inline_equation_predict_post(char *buf, int pos)
{
    unsigned char ch = buf[pos - 2];
    return ch != '\\' && ch != ' ' && ch != '\t';
}

#define IEP_PRE         inline_equation_predict_pre(G->buf, G->pos)
#define IEP_POST        inline_equation_predict_post(G->buf, G->pos)

// Do not allow whitespace before ending ~~
bool strike_predict_post(char *buf, int pos)
{
    unsigned char ch = buf[pos - 1];
    return ch != '\n' && ch != '\r' && ch != ' ' && ch != '\t';
}

#define STRIKE_POST     strike_predict_post(G->buf, G->pos)

// Number of display formula raw opening
unsigned char nr_dfr = 0;

bool start_dfr()
{
    nr_dfr = 1;
    return true;
}

bool inc_dfr()
{
    ++nr_dfr;
    return true;
}

bool dec_dfr()
{
    --nr_dfr;
    return true;
}

bool nested_dfr()
{
    return nr_dfr > 1;
}


#ifndef YY_ALLOC
#define YY_ALLOC(N, D) malloc(N)
#endif
#ifndef YY_CALLOC
#define YY_CALLOC(N, S, D) calloc(N, S)
#endif
#ifndef YY_REALLOC
#define YY_REALLOC(B, N, D) realloc(B, N)
#endif
#ifndef YY_FREE
#define YY_FREE free
#endif
#ifndef YY_LOCAL
#define YY_LOCAL(T)     static T
#endif
#ifndef YY_ACTION
#define YY_ACTION(T)    static T
#endif
#ifndef YY_RULE
#define YY_RULE(T)      static T
#endif
#ifndef YY_PARSE
#define YY_PARSE(T)     T
#endif
#ifndef YY_NAME
#define YY_NAME(N) yy##N
#endif
#ifndef YY_INPUT
#define YY_INPUT(buf, result, max_size)                 \
  {                                                     \
    int yyc= getchar();                                 \
    result= (EOF == yyc) ? 0 : (*(buf)= yyc, 1);        \
    yyprintf((stderr, "<%c>", yyc));                  \
  }
#endif
#ifndef YY_BEGIN
#define YY_BEGIN        ( G->begin= G->pos, 1)
#endif
#ifndef YY_END
#define YY_END          ( G->end= G->pos, 1)
#endif
#ifdef YY_DEBUG
# define yyprintf(args) fprintf args
#else
# define yyprintf(args)
#endif
#ifndef YYSTYPE
#define YYSTYPE int
#endif
#ifndef YY_XTYPE
#define YY_XTYPE void *
#endif
#ifndef YY_XVAR
#define YY_XVAR yyxvar
#endif

#ifndef YY_STACK_SIZE
#define YY_STACK_SIZE 128
#endif

#ifndef YY_BUFFER_START_SIZE
#define YY_BUFFER_START_SIZE 1024
#endif

#ifndef YY_PART
#define yydata G->data
#define yy G->ss

struct _yythunk; // forward declaration
typedef void (*yyaction)(struct _GREG *G, char *yytext, int yyleng, struct _yythunk *thunkpos, YY_XTYPE YY_XVAR);
typedef struct _yythunk { int begin, end;  yyaction  action;  struct _yythunk *next; } yythunk;

typedef struct _GREG {
  char *buf;
  int buflen;
  int   offset;
  int   pos;
  int   limit;
  char *text;
  int   textlen;
  int   begin;
  int   end;
  yythunk *thunks;
  int   thunkslen;
  int thunkpos;
  YYSTYPE ss;
  YYSTYPE *val;
  YYSTYPE *vals;
  int valslen;
  YY_XTYPE data;
} GREG;

YY_LOCAL(int) yyrefill(GREG *G)
{
  int yyn;
  while (G->buflen - G->pos < 512)
    {
      G->buflen *= 2;
      G->buf= (char *)YY_REALLOC(G->buf, G->buflen, G->data);
    }
  YY_INPUT((G->buf + G->pos), yyn, (G->buflen - G->pos));
  if (!yyn) return 0;
  G->limit += yyn;
  return 1;
}

YY_LOCAL(int) yymatchDot(GREG *G)
{
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  ++G->pos;
  return 1;
}

YY_LOCAL(int) yymatchChar(GREG *G, int c)
{
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  if ((unsigned char)G->buf[G->pos] == c)
    {
      ++G->pos;
      yyprintf((stderr, "  ok   yymatchChar(%c) @ %s\n", c, G->buf+G->pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchChar(%c) @ %s\n", c, G->buf+G->pos));
  return 0;
}

YY_LOCAL(int) yymatchString(GREG *G, char *s)
{
  int yysav= G->pos;
  while (*s)
    {
      if (G->pos >= G->limit && !yyrefill(G)) return 0;
      if (G->buf[G->pos] != *s)
        {
          G->pos= yysav;
          return 0;
        }
      ++s;
      ++G->pos;
    }
  return 1;
}

YY_LOCAL(int) yymatchClass(GREG *G, unsigned char *bits)
{
  int c;
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  c= (unsigned char)G->buf[G->pos];
  if (bits[c >> 3] & (1 << (c & 7)))
    {
      ++G->pos;
      yyprintf((stderr, "  ok   yymatchClass @ %s\n", G->buf+G->pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchClass @ %s\n", G->buf+G->pos));
  return 0;
}

YY_LOCAL(void) yyDo(GREG *G, yyaction action, int begin, int end)
{
  while (G->thunkpos >= G->thunkslen)
    {
      G->thunkslen *= 2;
      G->thunks= (yythunk *)YY_REALLOC(G->thunks, sizeof(yythunk) * G->thunkslen, G->data);
    }
  G->thunks[G->thunkpos].begin=  begin;
  G->thunks[G->thunkpos].end=    end;
  G->thunks[G->thunkpos].action= action;
  ++G->thunkpos;
}

YY_LOCAL(int) yyText(GREG *G, int begin, int end)
{
  int yyleng= end - begin;
  if (yyleng <= 0)
    yyleng= 0;
  else
    {
      while (G->textlen < (yyleng + 1))
        {
          G->textlen *= 2;
          G->text= (char *)YY_REALLOC(G->text, G->textlen, G->data);
        }
      memcpy(G->text, G->buf + begin, yyleng);
    }
  G->text[yyleng]= '\0';
  return yyleng;
}

YY_LOCAL(void) yyDone(GREG *G)
{
  int pos;
  for (pos= 0; pos < G->thunkpos; ++pos)
    {
      yythunk *thunk= &G->thunks[pos];
      int yyleng= thunk->end ? yyText(G, thunk->begin, thunk->end) : thunk->begin;
      yyprintf((stderr, "DO [%d] %p %s\n", pos, thunk->action, G->text));
      thunk->action(G, G->text, yyleng, thunk, G->data);
    }
  G->thunkpos= 0;
}

YY_LOCAL(void) yyCommit(GREG *G)
{
  if ((G->limit -= G->pos))
    {
      memmove(G->buf, G->buf + G->pos, G->limit);
    }
  G->offset += G->pos;
  G->begin -= G->pos;
  G->end -= G->pos;
  G->pos= G->thunkpos= 0;
}

YY_LOCAL(int) yyAccept(GREG *G, int tp0)
{
  if (tp0)
    {
      fprintf(stderr, "accept denied at %d\n", tp0);
      return 0;
    }
  else
    {
      yyDone(G);
      yyCommit(G);
    }
  return 1;
}

YY_LOCAL(void) yyPush(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR) { G->val += count; }
YY_LOCAL(void) yyPop(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR)  { G->val -= count; }
YY_LOCAL(void) yySet(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR)  { G->val[count]= G->ss; }

#endif /* YY_PART */

#define YYACCEPT        yyAccept(G, yythunkpos0)

YY_RULE(int) yy_RawNoteBlock(GREG *G); /* 266 */
YY_RULE(int) yy_RawNoteReference(GREG *G); /* 265 */
YY_RULE(int) yy_ExtendedSpecialChar(GREG *G); /* 264 */
YY_RULE(int) yy_AlphanumericAscii(GREG *G); /* 263 */
YY_RULE(int) yy_Quoted(GREG *G); /* 262 */
YY_RULE(int) yy_HtmlTag(GREG *G); /* 261 */
YY_RULE(int) yy_MarkTagClose(GREG *G); /* 260 */
YY_RULE(int) yy_MarkTagText(GREG *G); /* 259 */
YY_RULE(int) yy_MarkTagOpen(GREG *G); /* 258 */
YY_RULE(int) yy_Ticks5(GREG *G); /* 257 */
YY_RULE(int) yy_Ticks4(GREG *G); /* 256 */
YY_RULE(int) yy_Ticks3(GREG *G); /* 255 */
YY_RULE(int) yy_Ticks2(GREG *G); /* 254 */
YY_RULE(int) yy_Ticks1(GREG *G); /* 253 */
YY_RULE(int) yy_SkipBlock(GREG *G); /* 252 */
YY_RULE(int) yy_References(GREG *G); /* 251 */
YY_RULE(int) yy_EmptyTitle(GREG *G); /* 250 */
YY_RULE(int) yy_RefTitleParens(GREG *G); /* 249 */
YY_RULE(int) yy_RefTitleDouble(GREG *G); /* 248 */
YY_RULE(int) yy_RefTitleSingle(GREG *G); /* 247 */
YY_RULE(int) yy_LabelInline(GREG *G); /* 246 */
YY_RULE(int) yy_RefTitle(GREG *G); /* 245 */
YY_RULE(int) yy_RefSrc(GREG *G); /* 244 */
YY_RULE(int) yy_AutoLinkEmail(GREG *G); /* 243 */
YY_RULE(int) yy_AutoLinkUrl(GREG *G); /* 242 */
YY_RULE(int) yy_ImageSizeHeight(GREG *G); /* 241 */
YY_RULE(int) yy_ImageSizeWidth(GREG *G); /* 240 */
YY_RULE(int) yy_ImageSizeComplete(GREG *G); /* 239 */
YY_RULE(int) yy_TitleDoubleExt(GREG *G); /* 238 */
YY_RULE(int) yy_TitleSingleExt(GREG *G); /* 237 */
YY_RULE(int) yy_TitleDouble(GREG *G); /* 236 */
YY_RULE(int) yy_TitleSingle(GREG *G); /* 235 */
YY_RULE(int) yy_SourceContents(GREG *G); /* 234 */
YY_RULE(int) yy_ImageSize(GREG *G); /* 233 */
YY_RULE(int) yy_TitleExt(GREG *G); /* 232 */
YY_RULE(int) yy_Title(GREG *G); /* 231 */
YY_RULE(int) yy_Source(GREG *G); /* 230 */
YY_RULE(int) yy_Label(GREG *G); /* 229 */
YY_RULE(int) yy_ReferenceLinkSingle(GREG *G); /* 228 */
YY_RULE(int) yy_ReferenceLinkDouble(GREG *G); /* 227 */
YY_RULE(int) yy_AutoLink(GREG *G); /* 226 */
YY_RULE(int) yy_ReferenceLink(GREG *G); /* 225 */
YY_RULE(int) yy_ExplicitLinkSize(GREG *G); /* 224 */
YY_RULE(int) yy_ExplicitLink(GREG *G); /* 223 */
YY_RULE(int) yy_StrongUl(GREG *G); /* 222 */
YY_RULE(int) yy_StrongStar(GREG *G); /* 221 */
YY_RULE(int) yy_Whitespace(GREG *G); /* 220 */
YY_RULE(int) yy_EmphUl(GREG *G); /* 219 */
YY_RULE(int) yy_EmphStar(GREG *G); /* 218 */
YY_RULE(int) yy_StarLine(GREG *G); /* 217 */
YY_RULE(int) yy_UlLine(GREG *G); /* 216 */
YY_RULE(int) yy_SpecialChar(GREG *G); /* 215 */
YY_RULE(int) yy_Eof(GREG *G); /* 214 */
YY_RULE(int) yy_NormalEndline(GREG *G); /* 213 */
YY_RULE(int) yy_TerminalEndline(GREG *G); /* 212 */
YY_RULE(int) yy_LineBreak(GREG *G); /* 211 */
YY_RULE(int) yy_CharEntity(GREG *G); /* 210 */
YY_RULE(int) yy_DecEntity(GREG *G); /* 209 */
YY_RULE(int) yy_HexEntity(GREG *G); /* 208 */
YY_RULE(int) yy_Alphanumeric(GREG *G); /* 207 */
YY_RULE(int) yy_NormalChar(GREG *G); /* 206 */
YY_RULE(int) yy_Symbol(GREG *G); /* 205 */
YY_RULE(int) yy_EscapedChar(GREG *G); /* 204 */
YY_RULE(int) yy_Entity(GREG *G); /* 203 */
YY_RULE(int) yy_RawHtml(GREG *G); /* 202 */
YY_RULE(int) yy_Mark(GREG *G); /* 201 */
YY_RULE(int) yy_Code(GREG *G); /* 200 */
YY_RULE(int) yy_InlineNote(GREG *G); /* 199 */
YY_RULE(int) yy_NoteReference(GREG *G); /* 198 */
YY_RULE(int) yy_Link(GREG *G); /* 197 */
YY_RULE(int) yy_Image(GREG *G); /* 196 */
YY_RULE(int) yy_Strike(GREG *G); /* 195 */
YY_RULE(int) yy_Emph(GREG *G); /* 194 */
YY_RULE(int) yy_Strong(GREG *G); /* 193 */
YY_RULE(int) yy_UlOrStarLine(GREG *G); /* 192 */
YY_RULE(int) yy_Str(GREG *G); /* 191 */
YY_RULE(int) yy_InStyleTags(GREG *G); /* 190 */
YY_RULE(int) yy_StyleClose(GREG *G); /* 189 */
YY_RULE(int) yy_StyleOpen(GREG *G); /* 188 */
YY_RULE(int) yy_HtmlBlockType(GREG *G); /* 187 */
YY_RULE(int) yy_HtmlBlockSelfClosing(GREG *G); /* 186 */
YY_RULE(int) yy_HtmlComment(GREG *G); /* 185 */
YY_RULE(int) yy_HtmlBlockInTags(GREG *G); /* 184 */
YY_RULE(int) yy_HtmlBlockHead(GREG *G); /* 183 */
YY_RULE(int) yy_HtmlBlockCloseHead(GREG *G); /* 182 */
YY_RULE(int) yy_HtmlBlockOpenHead(GREG *G); /* 181 */
YY_RULE(int) yy_HtmlBlockScript(GREG *G); /* 180 */
YY_RULE(int) yy_HtmlBlockCloseScript(GREG *G); /* 179 */
YY_RULE(int) yy_HtmlBlockOpenScript(GREG *G); /* 178 */
YY_RULE(int) yy_HtmlBlockTr(GREG *G); /* 177 */
YY_RULE(int) yy_HtmlBlockCloseTr(GREG *G); /* 176 */
YY_RULE(int) yy_HtmlBlockOpenTr(GREG *G); /* 175 */
YY_RULE(int) yy_HtmlBlockThead(GREG *G); /* 174 */
YY_RULE(int) yy_HtmlBlockCloseThead(GREG *G); /* 173 */
YY_RULE(int) yy_HtmlBlockOpenThead(GREG *G); /* 172 */
YY_RULE(int) yy_HtmlBlockTh(GREG *G); /* 171 */
YY_RULE(int) yy_HtmlBlockCloseTh(GREG *G); /* 170 */
YY_RULE(int) yy_HtmlBlockOpenTh(GREG *G); /* 169 */
YY_RULE(int) yy_HtmlBlockTfoot(GREG *G); /* 168 */
YY_RULE(int) yy_HtmlBlockCloseTfoot(GREG *G); /* 167 */
YY_RULE(int) yy_HtmlBlockOpenTfoot(GREG *G); /* 166 */
YY_RULE(int) yy_HtmlBlockTd(GREG *G); /* 165 */
YY_RULE(int) yy_HtmlBlockCloseTd(GREG *G); /* 164 */
YY_RULE(int) yy_HtmlBlockOpenTd(GREG *G); /* 163 */
YY_RULE(int) yy_HtmlBlockTbody(GREG *G); /* 162 */
YY_RULE(int) yy_HtmlBlockCloseTbody(GREG *G); /* 161 */
YY_RULE(int) yy_HtmlBlockOpenTbody(GREG *G); /* 160 */
YY_RULE(int) yy_HtmlBlockLi(GREG *G); /* 159 */
YY_RULE(int) yy_HtmlBlockCloseLi(GREG *G); /* 158 */
YY_RULE(int) yy_HtmlBlockOpenLi(GREG *G); /* 157 */
YY_RULE(int) yy_HtmlBlockFrameset(GREG *G); /* 156 */
YY_RULE(int) yy_HtmlBlockCloseFrameset(GREG *G); /* 155 */
YY_RULE(int) yy_HtmlBlockOpenFrameset(GREG *G); /* 154 */
YY_RULE(int) yy_HtmlBlockDt(GREG *G); /* 153 */
YY_RULE(int) yy_HtmlBlockCloseDt(GREG *G); /* 152 */
YY_RULE(int) yy_HtmlBlockOpenDt(GREG *G); /* 151 */
YY_RULE(int) yy_HtmlBlockDd(GREG *G); /* 150 */
YY_RULE(int) yy_HtmlBlockCloseDd(GREG *G); /* 149 */
YY_RULE(int) yy_HtmlBlockOpenDd(GREG *G); /* 148 */
YY_RULE(int) yy_HtmlBlockUl(GREG *G); /* 147 */
YY_RULE(int) yy_HtmlBlockCloseUl(GREG *G); /* 146 */
YY_RULE(int) yy_HtmlBlockOpenUl(GREG *G); /* 145 */
YY_RULE(int) yy_HtmlBlockTable(GREG *G); /* 144 */
YY_RULE(int) yy_HtmlBlockCloseTable(GREG *G); /* 143 */
YY_RULE(int) yy_HtmlBlockOpenTable(GREG *G); /* 142 */
YY_RULE(int) yy_HtmlBlockPre(GREG *G); /* 141 */
YY_RULE(int) yy_HtmlBlockClosePre(GREG *G); /* 140 */
YY_RULE(int) yy_HtmlBlockOpenPre(GREG *G); /* 139 */
YY_RULE(int) yy_HtmlBlockP(GREG *G); /* 138 */
YY_RULE(int) yy_HtmlBlockCloseP(GREG *G); /* 137 */
YY_RULE(int) yy_HtmlBlockOpenP(GREG *G); /* 136 */
YY_RULE(int) yy_HtmlBlockOl(GREG *G); /* 135 */
YY_RULE(int) yy_HtmlBlockCloseOl(GREG *G); /* 134 */
YY_RULE(int) yy_HtmlBlockOpenOl(GREG *G); /* 133 */
YY_RULE(int) yy_HtmlBlockNoscript(GREG *G); /* 132 */
YY_RULE(int) yy_HtmlBlockCloseNoscript(GREG *G); /* 131 */
YY_RULE(int) yy_HtmlBlockOpenNoscript(GREG *G); /* 130 */
YY_RULE(int) yy_HtmlBlockNoframes(GREG *G); /* 129 */
YY_RULE(int) yy_HtmlBlockCloseNoframes(GREG *G); /* 128 */
YY_RULE(int) yy_HtmlBlockOpenNoframes(GREG *G); /* 127 */
YY_RULE(int) yy_HtmlBlockMenu(GREG *G); /* 126 */
YY_RULE(int) yy_HtmlBlockCloseMenu(GREG *G); /* 125 */
YY_RULE(int) yy_HtmlBlockOpenMenu(GREG *G); /* 124 */
YY_RULE(int) yy_HtmlBlockH6(GREG *G); /* 123 */
YY_RULE(int) yy_HtmlBlockCloseH6(GREG *G); /* 122 */
YY_RULE(int) yy_HtmlBlockOpenH6(GREG *G); /* 121 */
YY_RULE(int) yy_HtmlBlockH5(GREG *G); /* 120 */
YY_RULE(int) yy_HtmlBlockCloseH5(GREG *G); /* 119 */
YY_RULE(int) yy_HtmlBlockOpenH5(GREG *G); /* 118 */
YY_RULE(int) yy_HtmlBlockH4(GREG *G); /* 117 */
YY_RULE(int) yy_HtmlBlockCloseH4(GREG *G); /* 116 */
YY_RULE(int) yy_HtmlBlockOpenH4(GREG *G); /* 115 */
YY_RULE(int) yy_HtmlBlockH3(GREG *G); /* 114 */
YY_RULE(int) yy_HtmlBlockCloseH3(GREG *G); /* 113 */
YY_RULE(int) yy_HtmlBlockOpenH3(GREG *G); /* 112 */
YY_RULE(int) yy_HtmlBlockH2(GREG *G); /* 111 */
YY_RULE(int) yy_HtmlBlockCloseH2(GREG *G); /* 110 */
YY_RULE(int) yy_HtmlBlockOpenH2(GREG *G); /* 109 */
YY_RULE(int) yy_HtmlBlockH1(GREG *G); /* 108 */
YY_RULE(int) yy_HtmlBlockCloseH1(GREG *G); /* 107 */
YY_RULE(int) yy_HtmlBlockOpenH1(GREG *G); /* 106 */
YY_RULE(int) yy_HtmlBlockForm(GREG *G); /* 105 */
YY_RULE(int) yy_HtmlBlockCloseForm(GREG *G); /* 104 */
YY_RULE(int) yy_HtmlBlockOpenForm(GREG *G); /* 103 */
YY_RULE(int) yy_HtmlBlockFieldset(GREG *G); /* 102 */
YY_RULE(int) yy_HtmlBlockCloseFieldset(GREG *G); /* 101 */
YY_RULE(int) yy_HtmlBlockOpenFieldset(GREG *G); /* 100 */
YY_RULE(int) yy_HtmlBlockDl(GREG *G); /* 99 */
YY_RULE(int) yy_HtmlBlockCloseDl(GREG *G); /* 98 */
YY_RULE(int) yy_HtmlBlockOpenDl(GREG *G); /* 97 */
YY_RULE(int) yy_HtmlBlockDiv(GREG *G); /* 96 */
YY_RULE(int) yy_HtmlBlockCloseDiv(GREG *G); /* 95 */
YY_RULE(int) yy_HtmlBlockOpenDiv(GREG *G); /* 94 */
YY_RULE(int) yy_HtmlBlockDir(GREG *G); /* 93 */
YY_RULE(int) yy_HtmlBlockCloseDir(GREG *G); /* 92 */
YY_RULE(int) yy_HtmlBlockOpenDir(GREG *G); /* 91 */
YY_RULE(int) yy_HtmlBlockCenter(GREG *G); /* 90 */
YY_RULE(int) yy_HtmlBlockCloseCenter(GREG *G); /* 89 */
YY_RULE(int) yy_HtmlBlockOpenCenter(GREG *G); /* 88 */
YY_RULE(int) yy_HtmlBlockBlockquote(GREG *G); /* 87 */
YY_RULE(int) yy_HtmlBlockCloseBlockquote(GREG *G); /* 86 */
YY_RULE(int) yy_HtmlBlockOpenBlockquote(GREG *G); /* 85 */
YY_RULE(int) yy_HtmlBlockAddress(GREG *G); /* 84 */
YY_RULE(int) yy_HtmlBlockCloseAddress(GREG *G); /* 83 */
YY_RULE(int) yy_HtmlAttribute(GREG *G); /* 82 */
YY_RULE(int) yy_HtmlBlockOpenAddress(GREG *G); /* 81 */
YY_RULE(int) yy_OptionallyIndentedLine(GREG *G); /* 80 */
YY_RULE(int) yy_Indent(GREG *G); /* 79 */
YY_RULE(int) yy_ListBlockLine(GREG *G); /* 78 */
YY_RULE(int) yy_ListContinuationBlock(GREG *G); /* 77 */
YY_RULE(int) yy_ListBlock(GREG *G); /* 76 */
YY_RULE(int) yy_ListItem(GREG *G); /* 75 */
YY_RULE(int) yy_Enumerator(GREG *G); /* 74 */
YY_RULE(int) yy_ListItemTight(GREG *G); /* 73 */
YY_RULE(int) yy_ListLoose(GREG *G); /* 72 */
YY_RULE(int) yy_ListTight(GREG *G); /* 71 */
YY_RULE(int) yy_Bullet(GREG *G); /* 70 */
YY_RULE(int) yy_TableCell(GREG *G); /* 69 */
YY_RULE(int) yy_TableBorder(GREG *G); /* 68 */
YY_RULE(int) yy_TableLine(GREG *G); /* 67 */
YY_RULE(int) yy_TableDelimiter(GREG *G); /* 66 */
YY_RULE(int) yy_TableHeader(GREG *G); /* 65 */
YY_RULE(int) yy_InlineEquationMultiple(GREG *G); /* 64 */
YY_RULE(int) yy_InlineEquationSingle(GREG *G); /* 63 */
YY_RULE(int) yy_InlineEquation(GREG *G); /* 62 */
YY_RULE(int) yy_Nonspacechar(GREG *G); /* 61 */
YY_RULE(int) yy_DisplayFormulaRawMark(GREG *G); /* 60 */
YY_RULE(int) yy_DisplayFormulaRawEnd(GREG *G); /* 59 */
YY_RULE(int) yy_Spnl(GREG *G); /* 58 */
YY_RULE(int) yy_DisplayFormulaRawStart(GREG *G); /* 57 */
YY_RULE(int) yy_FormulaNumber(GREG *G); /* 56 */
YY_RULE(int) yy_DisplayFormulaRaw(GREG *G); /* 55 */
YY_RULE(int) yy_DisplayFormulaDollar(GREG *G); /* 54 */
YY_RULE(int) yy_FencedCodeBlockTidle(GREG *G); /* 53 */
YY_RULE(int) yy_FencedCodeBlockTick(GREG *G); /* 52 */
YY_RULE(int) yy_FencedCodeBlockEndTidle(GREG *G); /* 51 */
YY_RULE(int) yy_FencedCodeBlockChunkTidle(GREG *G); /* 50 */
YY_RULE(int) yy_FencedCodeBlockStartTidle(GREG *G); /* 49 */
YY_RULE(int) yy_Spacechar(GREG *G); /* 48 */
YY_RULE(int) yy_FencedCodeBlockEndTick(GREG *G); /* 47 */
YY_RULE(int) yy_FencedCodeBlockChunkTick(GREG *G); /* 46 */
YY_RULE(int) yy_FencedCodeBlockStartTick(GREG *G); /* 45 */
YY_RULE(int) yy_VerbatimChunk(GREG *G); /* 44 */
YY_RULE(int) yy_IndentedLine(GREG *G); /* 43 */
YY_RULE(int) yy_NonblankIndentedLine(GREG *G); /* 42 */
YY_RULE(int) yy_Line(GREG *G); /* 41 */
YY_RULE(int) yy_StartList(GREG *G); /* 40 */
YY_RULE(int) yy_BlockQuoteRaw(GREG *G); /* 39 */
YY_RULE(int) yy_Endline(GREG *G); /* 38 */
YY_RULE(int) yy_RawLine(GREG *G); /* 37 */
YY_RULE(int) yy_SetextBottom2(GREG *G); /* 36 */
YY_RULE(int) yy_SetextBottom1(GREG *G); /* 35 */
YY_RULE(int) yy_SetextHeading2(GREG *G); /* 34 */
YY_RULE(int) yy_SetextHeading1(GREG *G); /* 33 */
YY_RULE(int) yy_SetextHeading(GREG *G); /* 32 */
YY_RULE(int) yy_Space(GREG *G); /* 31 */
YY_RULE(int) yy_AtxHeading(GREG *G); /* 30 */
YY_RULE(int) yy_AtxStart(GREG *G); /* 29 */
YY_RULE(int) yy_Inline(GREG *G); /* 28 */
YY_RULE(int) yy_Sp(GREG *G); /* 27 */
YY_RULE(int) yy_AtxInline(GREG *G); /* 26 */
YY_RULE(int) yy_Inlines(GREG *G); /* 25 */
YY_RULE(int) yy_NonindentSpace(GREG *G); /* 24 */
YY_RULE(int) yy_Plain(GREG *G); /* 23 */
YY_RULE(int) yy_Para(GREG *G); /* 22 */
YY_RULE(int) yy_StyleBlock(GREG *G); /* 21 */
YY_RULE(int) yy_HtmlBlock(GREG *G); /* 20 */
YY_RULE(int) yy_Table(GREG *G); /* 19 */
YY_RULE(int) yy_BulletList(GREG *G); /* 18 */
YY_RULE(int) yy_OrderedList(GREG *G); /* 17 */
YY_RULE(int) yy_Heading(GREG *G); /* 16 */
YY_RULE(int) yy_HorizontalRule(GREG *G); /* 15 */
YY_RULE(int) yy_Reference(GREG *G); /* 14 */
YY_RULE(int) yy_Note(GREG *G); /* 13 */
YY_RULE(int) yy_DisplayFormula(GREG *G); /* 12 */
YY_RULE(int) yy_FencedCodeBlock(GREG *G); /* 11 */
YY_RULE(int) yy_Verbatim(GREG *G); /* 10 */
YY_RULE(int) yy_BlockQuote(GREG *G); /* 9 */
YY_RULE(int) yy_BlankLine(GREG *G); /* 8 */
YY_RULE(int) yy_FrontMatterEndMark(GREG *G); /* 7 */
YY_RULE(int) yy_FrontMatterBlock(GREG *G); /* 6 */
YY_RULE(int) yy_Newline(GREG *G); /* 5 */
YY_RULE(int) yy_LocMarker(GREG *G); /* 4 */
YY_RULE(int) yy_Block(GREG *G); /* 3 */
YY_RULE(int) yy_FrontMatter(GREG *G); /* 2 */
YY_RULE(int) yy_Doc(GREG *G); /* 1 */

YY_ACTION(void) yy_1_InlineNote(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_InlineNote\n"));
   ADD(elem_s(pmh_NOTE)); ;
#undef s
}
YY_ACTION(void) yy_1_Note(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Note\n"));
   ADD(elem_s(pmh_NOTE)); ;
#undef s
}
YY_ACTION(void) yy_1_NoteReference(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_NoteReference\n"));
   ADD(elem(pmh_NOTE)); ;
}
YY_ACTION(void) yy_1_RawLine(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_RawLine\n"));
   yy = elem(pmh_RAW); ;
}
YY_ACTION(void) yy_1_Line(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Line\n"));
   yy = mk_element((parser_data *)G->data, pmh_RAW, yy->pos, yy->end); ;
}
YY_ACTION(void) yy_1_StartList(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_StartList\n"));
   yy = NULL; ;
}
YY_ACTION(void) yy_1_HtmlComment(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlComment\n"));
   ADD(elem_s(pmh_COMMENT)); ;
#undef s
}
YY_ACTION(void) yy_1_RawHtml(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_RawHtml\n"));
   ADD(elem_s(pmh_HTML)); ;
#undef s
}
YY_ACTION(void) yy_1_MarkTagClose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_MarkTagClose\n"));
   yy = elem_s(pmh_HTML); ;
#undef s
}
YY_ACTION(void) yy_1_MarkTagOpen(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_MarkTagOpen\n"));
   yy = elem_s(pmh_HTML); ;
#undef s
}
YY_ACTION(void) yy_1_Mark(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define c G->val[-1]
#define o G->val[-2]
  yyprintf((stderr, "do yy_1_Mark\n"));
  
         ADD(o);
         ADD(mk_element((parser_data *)G->data, pmh_MARK, o->end, c->pos));
         ADD(c);
       ;
#undef c
#undef o
}
YY_ACTION(void) yy_1_Code(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Code\n"));
   ADD(elem_s(pmh_CODE)); ;
#undef s
}
YY_ACTION(void) yy_1_Ticks5(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks5\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks4(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks4\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks3(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks3\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks2\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks1\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_RefSrc(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_RefSrc\n"));
   yy = mk_notype; yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_2_Label(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_Label\n"));
  
            s->pos = s->pos;
            s->end = thunk->end;
            yy = s;
        ;
#undef s
}
YY_ACTION(void) yy_1_Label(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Label\n"));
   s->label = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_Reference(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define r G->val[-1]
#define l G->val[-2]
#define s G->val[-3]
  yyprintf((stderr, "do yy_1_Reference\n"));
  
                pmh_realelement *el = elem_s(pmh_REFERENCE);
                el->label = strdup_or_null(l->label);
                el->address = strdup_or_null(r->address);
                ADD(el);
                FREE_LABEL(l);
                FREE_ADDRESS(r);
              ;
#undef r
#undef l
#undef s
}
YY_ACTION(void) yy_3_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_3_AutoLinkEmail\n"));
  
                s->end = thunk->end;
                ADD(s);
               ;
#undef s
}
YY_ACTION(void) yy_2_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_AutoLinkEmail\n"));
   s->address = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AutoLinkEmail\n"));
   s->type = pmh_AUTO_LINK_EMAIL; ;
#undef s
}
YY_ACTION(void) yy_3_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_3_AutoLinkUrl\n"));
  
                s->end = thunk->end;
                ADD(s);
               ;
#undef s
}
YY_ACTION(void) yy_2_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_AutoLinkUrl\n"));
   s->address = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AutoLinkUrl\n"));
   s->type = pmh_AUTO_LINK_URL; ;
#undef s
}
YY_ACTION(void) yy_3_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_3_Source\n"));
   yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_2_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_2_Source\n"));
   yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_1_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Source\n"));
   yy = mk_notype; ;
}
YY_ACTION(void) yy_1_ExplicitLinkSize(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ExplicitLinkSize\n"));
  
                        yy = elem_s(pmh_LINK);
                        if (l->address != NULL)
                            yy->address = strdup_or_null(l->address);
                        FREE_LABEL(s);
                        FREE_ADDRESS(l);
                    ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_ExplicitLink(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ExplicitLink\n"));
  
                    yy = elem_s(pmh_LINK);
                    if (l->address != NULL)
                        yy->address = strdup_or_null(l->address);
                    FREE_LABEL(s);
                    FREE_ADDRESS(l);
                ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_ReferenceLinkSingle(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_ReferenceLinkSingle\n"));
  
                        	pmh_realelement *reference = GET_REF(s->label);
                            if (reference) {
                                yy = elem_s(pmh_LINK);
                                yy->label = strdup_or_null(s->label);
                                yy->address = strdup_or_null(reference->address);
                            } else
                                yy = NULL;
                            FREE_LABEL(s);
                        ;
#undef s
}
YY_ACTION(void) yy_1_ReferenceLinkDouble(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ReferenceLinkDouble\n"));
  
                        	pmh_realelement *reference = GET_REF(l->label);
                            if (reference) {
                                yy = elem_s(pmh_LINK);
                                yy->label = strdup_or_null(l->label);
                                yy->address = strdup_or_null(reference->address);
                            } else
                                yy = NULL;
                            FREE_LABEL(s);
                            FREE_LABEL(l);
                        ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_Link(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Link\n"));
   if (yy) ADD(yy); ;
}
YY_ACTION(void) yy_1_Image(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Image\n"));
  
            if (yy != NULL) {
                yy->type = pmh_IMAGE;
                yy->pos -= 1;
                ADD(yy);
            }
        ;
}
YY_ACTION(void) yy_1_Strike(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Strike\n"));
   ADD(elem_s(pmh_STRIKE)); ;
#undef s
}
YY_ACTION(void) yy_1_StrongUl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StrongUl\n"));
   ADD(elem_s(pmh_STRONG)); ;
#undef s
}
YY_ACTION(void) yy_1_StrongStar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StrongStar\n"));
   ADD(elem_s(pmh_STRONG)); ;
#undef s
}
YY_ACTION(void) yy_1_EmphUl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_EmphUl\n"));
   ADD(elem_s(pmh_EMPH)); ;
#undef s
}
YY_ACTION(void) yy_1_EmphStar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_EmphStar\n"));
   ADD(elem_s(pmh_EMPH)); ;
#undef s
}
YY_ACTION(void) yy_1_Entity(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Entity\n"));
   ADD(elem_s(pmh_HTML_ENTITY)); ;
#undef s
}
YY_ACTION(void) yy_1_StyleBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StyleBlock\n"));
   ADD(elem_s(pmh_HTMLBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlock\n"));
   ADD(elem_s(pmh_HTMLBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH6(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH6\n"));
   ADD(elem_s(pmh_H6)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH5(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH5\n"));
   ADD(elem_s(pmh_H5)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH4(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH4\n"));
   ADD(elem_s(pmh_H4)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH3(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH3\n"));
   ADD(elem_s(pmh_H3)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH2\n"));
   ADD(elem_s(pmh_H2)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH1\n"));
   ADD(elem_s(pmh_H1)); ;
#undef s
}
YY_ACTION(void) yy_1_Enumerator(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Enumerator\n"));
   ADD(elem(pmh_LIST_ENUMERATOR)); ;
}
YY_ACTION(void) yy_3_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListContinuationBlock\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListContinuationBlock\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListContinuationBlock\n"));
   if (*yytext == '\0') /* if strlen(yytext) == 0 */
                                a = cons(elem(pmh_SEPARATOR), a);
                            else
                                a = cons(elem(pmh_RAW), a);
                          ;
#undef a
}
YY_ACTION(void) yy_3_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListBlock\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListBlock\n"));
   a = cons(elem(pmh_RAW), a); ;
#undef a
}
YY_ACTION(void) yy_1_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListBlock\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_3_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListItemTight\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListItemTight\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListItemTight\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_3_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListItem\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListItem\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListItem\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_2_ListLoose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define b G->val[-1]
#define a G->val[-2]
  yyprintf((stderr, "do yy_2_ListLoose\n"));
   pmh_realelement *cur = a;
              while (cur != NULL) {
                  pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
                  rawlist->children = reverse(cur->children);
                  ADD(rawlist);
                  cur = cur->next;
              }
            ;
#undef b
#undef a
}
YY_ACTION(void) yy_1_ListLoose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define b G->val[-1]
#define a G->val[-2]
  yyprintf((stderr, "do yy_1_ListLoose\n"));
   b = cons(etext("\n\n"), b); /* In loose list, \n\n added to end of each element */
                pmh_realelement *el = mk_notype;
                el->children = b;
                a = cons(el, a);
              ;
#undef b
#undef a
}
YY_ACTION(void) yy_2_ListTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListTight\n"));
   pmh_realelement *cur = a;
              while (cur != NULL) {
                  pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
                  rawlist->children = reverse(cur->children);
                  ADD(rawlist);
                  cur = cur->next;
              }
            ;
#undef a
}
YY_ACTION(void) yy_1_ListTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListTight\n"));
   pmh_realelement *el = mk_notype;
                el->children = yy;
                a = cons(el, a);
              ;
#undef a
}
YY_ACTION(void) yy_1_Bullet(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Bullet\n"));
   ADD(elem(pmh_LIST_BULLET)); ;
}
YY_ACTION(void) yy_1_HorizontalRule(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_HorizontalRule\n"));
   ADD(elem(pmh_HRULE)); ;
}
YY_ACTION(void) yy_1_TableBorder(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_TableBorder\n"));
   ADD(elem(pmh_TABLEBORDER)); ;
}
YY_ACTION(void) yy_1_TableHeader(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_TableHeader\n"));
   ADD(elem_s(pmh_TABLEHEADER)); ;
#undef s
}
YY_ACTION(void) yy_1_Table(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Table\n"));
   ADD(elem_s(pmh_TABLE)); ;
#undef s
}
YY_ACTION(void) yy_1_InlineEquation(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_InlineEquation\n"));
   ADD(elem(pmh_INLINEEQUATION)); ;
}
YY_ACTION(void) yy_1_DisplayFormulaRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_DisplayFormulaRaw\n"));
   ADD(elem(pmh_DISPLAYFORMULA)); ;
}
YY_ACTION(void) yy_1_DisplayFormulaDollar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_DisplayFormulaDollar\n"));
   ADD(elem(pmh_DISPLAYFORMULA)); ;
}
YY_ACTION(void) yy_1_FencedCodeBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_FencedCodeBlock\n"));
   ADD(elem(pmh_FENCEDCODEBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_Verbatim(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Verbatim\n"));
   ADD(elem_s(pmh_VERBATIM)); ;
#undef s
}
YY_ACTION(void) yy_5_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_5_BlockQuoteRaw\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_4_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_4_BlockQuoteRaw\n"));
   a = cons(etext("\n"), a); ;
#undef a
}
YY_ACTION(void) yy_3_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_BlockQuoteRaw\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_2_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_BlockQuoteRaw\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_BlockQuoteRaw\n"));
   ADD(elem(pmh_BLOCKQUOTE)); ;
#undef a
}
YY_ACTION(void) yy_1_BlockQuote(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_BlockQuote\n"));
   pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
              rawlist->children = reverse(a);
              ADD(rawlist);
            ;
#undef a
}
YY_ACTION(void) yy_1_SetextHeading2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_SetextHeading2\n"));
   ADD(elem_s(pmh_H2)); ;
#undef s
}
YY_ACTION(void) yy_1_SetextHeading1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_SetextHeading1\n"));
   ADD(elem_s(pmh_H1)); ;
#undef s
}
YY_ACTION(void) yy_1_AtxHeading(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AtxHeading\n"));
   ADD(elem_s(s->type)); ;
#undef s
}
YY_ACTION(void) yy_1_AtxStart(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_AtxStart\n"));
   yy = elem((pmh_element_type)(pmh_H1 + (strlen(yytext) - 1))); ;
}
YY_ACTION(void) yy_1_FrontMatter(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_FrontMatter\n"));
  
                    ADD(elem(pmh_FRONTMATTER));
                ;
#undef s
}
YY_ACTION(void) yy_1_LocMarker(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_LocMarker\n"));
   yy = elem(pmh_NO_TYPE); ;
}

YY_RULE(int) yy_RawNoteBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawNoteBlock"));
  {  int yypos4= G->pos, yythunkpos4= G->thunkpos;  if (!yy_BlankLine(G)) { goto l4; }  goto l1;
  l4:;	  G->pos= yypos4; G->thunkpos= yythunkpos4;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l1; }
  l2:;	
  {  int yypos3= G->pos, yythunkpos3= G->thunkpos;
  {  int yypos5= G->pos, yythunkpos5= G->thunkpos;  if (!yy_BlankLine(G)) { goto l5; }  goto l3;
  l5:;	  G->pos= yypos5; G->thunkpos= yythunkpos5;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l3; }  goto l2;
  l3:;	  G->pos= yypos3; G->thunkpos= yythunkpos3;
  }
  l6:;	
  {  int yypos7= G->pos, yythunkpos7= G->thunkpos;  if (!yy_BlankLine(G)) { goto l7; }  goto l6;
  l7:;	  G->pos= yypos7; G->thunkpos= yythunkpos7;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "RawNoteBlock", G->buf+G->pos));
  return 1;
  l1:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawNoteBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawNoteReference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawNoteReference"));  if (!yymatchString(G, "[^")) goto l8;
  {  int yypos11= G->pos, yythunkpos11= G->thunkpos;  if (!yy_Newline(G)) { goto l11; }  goto l8;
  l11:;	  G->pos= yypos11; G->thunkpos= yythunkpos11;
  }
  {  int yypos12= G->pos, yythunkpos12= G->thunkpos;  if (!yymatchChar(G, ']')) goto l12;  goto l8;
  l12:;	  G->pos= yypos12; G->thunkpos= yythunkpos12;
  }  if (!yymatchDot(G)) goto l8;
  l9:;	
  {  int yypos10= G->pos, yythunkpos10= G->thunkpos;
  {  int yypos13= G->pos, yythunkpos13= G->thunkpos;  if (!yy_Newline(G)) { goto l13; }  goto l10;
  l13:;	  G->pos= yypos13; G->thunkpos= yythunkpos13;
  }
  {  int yypos14= G->pos, yythunkpos14= G->thunkpos;  if (!yymatchChar(G, ']')) goto l14;  goto l10;
  l14:;	  G->pos= yypos14; G->thunkpos= yythunkpos14;
  }  if (!yymatchDot(G)) goto l10;  goto l9;
  l10:;	  G->pos= yypos10; G->thunkpos= yythunkpos10;
  }  if (!yymatchChar(G, ']')) goto l8;
  yyprintf((stderr, "  ok   %s @ %s\n", "RawNoteReference", G->buf+G->pos));
  return 1;
  l8:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawNoteReference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExtendedSpecialChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ExtendedSpecialChar"));
  {  int yypos16= G->pos, yythunkpos16= G->thunkpos;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l17;  if (!yymatchChar(G, '^')) goto l17;  goto l16;
  l17:;	  G->pos= yypos16; G->thunkpos= yythunkpos16;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH) )) goto l18;  if (!yymatchChar(G, '$')) goto l18;  goto l16;
  l18:;	  G->pos= yypos16; G->thunkpos= yythunkpos16;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_TABLE) )) goto l15;  if (!yymatchChar(G, '|')) goto l15;
  }
  l16:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ExtendedSpecialChar", G->buf+G->pos));
  return 1;
  l15:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExtendedSpecialChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AlphanumericAscii(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AlphanumericAscii"));  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l19;
  yyprintf((stderr, "  ok   %s @ %s\n", "AlphanumericAscii", G->buf+G->pos));
  return 1;
  l19:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AlphanumericAscii", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Quoted(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Quoted"));
  {  int yypos21= G->pos, yythunkpos21= G->thunkpos;  if (!yymatchChar(G, '"')) goto l22;
  l23:;	
  {  int yypos24= G->pos, yythunkpos24= G->thunkpos;
  {  int yypos25= G->pos, yythunkpos25= G->thunkpos;  if (!yymatchChar(G, '"')) goto l25;  goto l24;
  l25:;	  G->pos= yypos25; G->thunkpos= yythunkpos25;
  }  if (!yymatchDot(G)) goto l24;  goto l23;
  l24:;	  G->pos= yypos24; G->thunkpos= yythunkpos24;
  }  if (!yymatchChar(G, '"')) goto l22;  goto l21;
  l22:;	  G->pos= yypos21; G->thunkpos= yythunkpos21;  if (!yymatchChar(G, '\'')) goto l20;
  l26:;	
  {  int yypos27= G->pos, yythunkpos27= G->thunkpos;
  {  int yypos28= G->pos, yythunkpos28= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l28;  goto l27;
  l28:;	  G->pos= yypos28; G->thunkpos= yythunkpos28;
  }  if (!yymatchDot(G)) goto l27;  goto l26;
  l27:;	  G->pos= yypos27; G->thunkpos= yythunkpos27;
  }  if (!yymatchChar(G, '\'')) goto l20;
  }
  l21:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Quoted", G->buf+G->pos));
  return 1;
  l20:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Quoted", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlTag(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlTag"));  if (!yymatchChar(G, '<')) goto l29;  if (!yy_Spnl(G)) { goto l29; }
  {  int yypos30= G->pos, yythunkpos30= G->thunkpos;  if (!yymatchChar(G, '/')) goto l30;  goto l31;
  l30:;	  G->pos= yypos30; G->thunkpos= yythunkpos30;
  }
  l31:;	  if (!yy_AlphanumericAscii(G)) { goto l29; }
  l32:;	
  {  int yypos33= G->pos, yythunkpos33= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l33; }  goto l32;
  l33:;	  G->pos= yypos33; G->thunkpos= yythunkpos33;
  }  if (!yy_Spnl(G)) { goto l29; }
  l34:;	
  {  int yypos35= G->pos, yythunkpos35= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l35; }  goto l34;
  l35:;	  G->pos= yypos35; G->thunkpos= yythunkpos35;
  }
  {  int yypos36= G->pos, yythunkpos36= G->thunkpos;  if (!yymatchChar(G, '/')) goto l36;  goto l37;
  l36:;	  G->pos= yypos36; G->thunkpos= yythunkpos36;
  }
  l37:;	  if (!yy_Spnl(G)) { goto l29; }  if (!yymatchChar(G, '>')) goto l29;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlTag", G->buf+G->pos));
  return 1;
  l29:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlTag", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_MarkTagClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "MarkTagClose"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l38;  if (!yy_LocMarker(G)) { goto l38; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '<')) goto l38;  if (!yy_Spnl(G)) { goto l38; }  if (!yymatchChar(G, '/')) goto l38;
  {  int yypos39= G->pos, yythunkpos39= G->thunkpos;  if (!yymatchString(G, "mark")) goto l40;  goto l39;
  l40:;	  G->pos= yypos39; G->thunkpos= yythunkpos39;  if (!yymatchString(G, "MARK")) goto l38;
  }
  l39:;	  if (!yy_Spnl(G)) { goto l38; }  if (!yymatchChar(G, '>')) goto l38;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l38;  yyDo(G, yy_1_MarkTagClose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "MarkTagClose", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l38:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "MarkTagClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_MarkTagText(GREG *G)
{
  yyprintf((stderr, "%s\n", "MarkTagText"));
  l42:;	
  {  int yypos43= G->pos, yythunkpos43= G->thunkpos;
  {  int yypos44= G->pos, yythunkpos44= G->thunkpos;  if (!yy_MarkTagClose(G)) { goto l44; }  goto l43;
  l44:;	  G->pos= yypos44; G->thunkpos= yythunkpos44;
  }
  {  int yypos45= G->pos, yythunkpos45= G->thunkpos;  if (!yy_RawHtml(G)) { goto l46; }  goto l45;
  l46:;	  G->pos= yypos45; G->thunkpos= yythunkpos45;  if (!yymatchDot(G)) goto l43;
  }
  l45:;	  goto l42;
  l43:;	  G->pos= yypos43; G->thunkpos= yythunkpos43;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "MarkTagText", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_MarkTagOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "MarkTagOpen"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l47;  if (!yy_LocMarker(G)) { goto l47; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '<')) goto l47;  if (!yy_Spnl(G)) { goto l47; }
  {  int yypos48= G->pos, yythunkpos48= G->thunkpos;  if (!yymatchString(G, "mark")) goto l49;  goto l48;
  l49:;	  G->pos= yypos48; G->thunkpos= yythunkpos48;  if (!yymatchString(G, "MARK")) goto l47;
  }
  l48:;	  if (!yy_Spnl(G)) { goto l47; }
  l50:;	
  {  int yypos51= G->pos, yythunkpos51= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l51; }  goto l50;
  l51:;	  G->pos= yypos51; G->thunkpos= yythunkpos51;
  }  if (!yymatchChar(G, '>')) goto l47;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l47;  yyDo(G, yy_1_MarkTagOpen, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "MarkTagOpen", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l47:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "MarkTagOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks5"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l52;  if (!yymatchString(G, "`````")) goto l52;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l52;
  {  int yypos53= G->pos, yythunkpos53= G->thunkpos;  if (!yymatchChar(G, '`')) goto l53;  goto l52;
  l53:;	  G->pos= yypos53; G->thunkpos= yythunkpos53;
  }  yyDo(G, yy_1_Ticks5, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks5", G->buf+G->pos));
  return 1;
  l52:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks4"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l54;  if (!yymatchString(G, "````")) goto l54;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l54;
  {  int yypos55= G->pos, yythunkpos55= G->thunkpos;  if (!yymatchChar(G, '`')) goto l55;  goto l54;
  l55:;	  G->pos= yypos55; G->thunkpos= yythunkpos55;
  }  yyDo(G, yy_1_Ticks4, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks4", G->buf+G->pos));
  return 1;
  l54:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks3"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l56;  if (!yymatchString(G, "```")) goto l56;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l56;
  {  int yypos57= G->pos, yythunkpos57= G->thunkpos;  if (!yymatchChar(G, '`')) goto l57;  goto l56;
  l57:;	  G->pos= yypos57; G->thunkpos= yythunkpos57;
  }  yyDo(G, yy_1_Ticks3, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks3", G->buf+G->pos));
  return 1;
  l56:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks2"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l58;  if (!yymatchString(G, "``")) goto l58;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l58;
  {  int yypos59= G->pos, yythunkpos59= G->thunkpos;  if (!yymatchChar(G, '`')) goto l59;  goto l58;
  l59:;	  G->pos= yypos59; G->thunkpos= yythunkpos59;
  }  yyDo(G, yy_1_Ticks2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks2", G->buf+G->pos));
  return 1;
  l58:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks1"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l60;  if (!yymatchChar(G, '`')) goto l60;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l60;
  {  int yypos61= G->pos, yythunkpos61= G->thunkpos;  if (!yymatchChar(G, '`')) goto l61;  goto l60;
  l61:;	  G->pos= yypos61; G->thunkpos= yythunkpos61;
  }  yyDo(G, yy_1_Ticks1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks1", G->buf+G->pos));
  return 1;
  l60:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SkipBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SkipBlock"));
  {  int yypos63= G->pos, yythunkpos63= G->thunkpos;
  {  int yypos67= G->pos, yythunkpos67= G->thunkpos;  if (!yy_BlankLine(G)) { goto l67; }  goto l64;
  l67:;	  G->pos= yypos67; G->thunkpos= yythunkpos67;
  }  if (!yy_RawLine(G)) { goto l64; }
  l65:;	
  {  int yypos66= G->pos, yythunkpos66= G->thunkpos;
  {  int yypos68= G->pos, yythunkpos68= G->thunkpos;  if (!yy_BlankLine(G)) { goto l68; }  goto l66;
  l68:;	  G->pos= yypos68; G->thunkpos= yythunkpos68;
  }  if (!yy_RawLine(G)) { goto l66; }  goto l65;
  l66:;	  G->pos= yypos66; G->thunkpos= yythunkpos66;
  }
  l69:;	
  {  int yypos70= G->pos, yythunkpos70= G->thunkpos;  if (!yy_BlankLine(G)) { goto l70; }  goto l69;
  l70:;	  G->pos= yypos70; G->thunkpos= yythunkpos70;
  }  goto l63;
  l64:;	  G->pos= yypos63; G->thunkpos= yythunkpos63;  if (!yy_BlankLine(G)) { goto l62; }
  l71:;	
  {  int yypos72= G->pos, yythunkpos72= G->thunkpos;  if (!yy_BlankLine(G)) { goto l72; }  goto l71;
  l72:;	  G->pos= yypos72; G->thunkpos= yythunkpos72;
  }
  }
  l63:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SkipBlock", G->buf+G->pos));
  return 1;
  l62:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SkipBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_References(GREG *G)
{
  yyprintf((stderr, "%s\n", "References"));
  l74:;	
  {  int yypos75= G->pos, yythunkpos75= G->thunkpos;
  {  int yypos76= G->pos, yythunkpos76= G->thunkpos;  if (!yy_Reference(G)) { goto l77; }  goto l76;
  l77:;	  G->pos= yypos76; G->thunkpos= yythunkpos76;  if (!yy_SkipBlock(G)) { goto l75; }
  }
  l76:;	  goto l74;
  l75:;	  G->pos= yypos75; G->thunkpos= yythunkpos75;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "References", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_EmptyTitle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "EmptyTitle"));  if (!yymatchString(G, "")) goto l78;
  yyprintf((stderr, "  ok   %s @ %s\n", "EmptyTitle", G->buf+G->pos));
  return 1;
  l78:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmptyTitle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleParens(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleParens"));  if (!yy_Spnl(G)) { goto l79; }  if (!yymatchChar(G, '(')) goto l79;
  l80:;	
  {  int yypos81= G->pos, yythunkpos81= G->thunkpos;
  {  int yypos82= G->pos, yythunkpos82= G->thunkpos;
  {  int yypos83= G->pos, yythunkpos83= G->thunkpos;  if (!yymatchChar(G, ')')) goto l84;  if (!yy_Sp(G)) { goto l84; }  if (!yy_Newline(G)) { goto l84; }  goto l83;
  l84:;	  G->pos= yypos83; G->thunkpos= yythunkpos83;  if (!yy_Newline(G)) { goto l82; }
  }
  l83:;	  goto l81;
  l82:;	  G->pos= yypos82; G->thunkpos= yythunkpos82;
  }  if (!yymatchDot(G)) goto l81;  goto l80;
  l81:;	  G->pos= yypos81; G->thunkpos= yythunkpos81;
  }  if (!yymatchChar(G, ')')) goto l79;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleParens", G->buf+G->pos));
  return 1;
  l79:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleParens", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleDouble"));  if (!yy_Spnl(G)) { goto l85; }  if (!yymatchChar(G, '"')) goto l85;
  l86:;	
  {  int yypos87= G->pos, yythunkpos87= G->thunkpos;
  {  int yypos88= G->pos, yythunkpos88= G->thunkpos;
  {  int yypos89= G->pos, yythunkpos89= G->thunkpos;  if (!yymatchChar(G, '"')) goto l90;  if (!yy_Sp(G)) { goto l90; }  if (!yy_Newline(G)) { goto l90; }  goto l89;
  l90:;	  G->pos= yypos89; G->thunkpos= yythunkpos89;  if (!yy_Newline(G)) { goto l88; }
  }
  l89:;	  goto l87;
  l88:;	  G->pos= yypos88; G->thunkpos= yythunkpos88;
  }  if (!yymatchDot(G)) goto l87;  goto l86;
  l87:;	  G->pos= yypos87; G->thunkpos= yythunkpos87;
  }  if (!yymatchChar(G, '"')) goto l85;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleDouble", G->buf+G->pos));
  return 1;
  l85:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleSingle"));  if (!yy_Spnl(G)) { goto l91; }  if (!yymatchChar(G, '\'')) goto l91;
  l92:;	
  {  int yypos93= G->pos, yythunkpos93= G->thunkpos;
  {  int yypos94= G->pos, yythunkpos94= G->thunkpos;
  {  int yypos95= G->pos, yythunkpos95= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l96;  if (!yy_Sp(G)) { goto l96; }  if (!yy_Newline(G)) { goto l96; }  goto l95;
  l96:;	  G->pos= yypos95; G->thunkpos= yythunkpos95;  if (!yy_Newline(G)) { goto l94; }
  }
  l95:;	  goto l93;
  l94:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;
  }  if (!yymatchDot(G)) goto l93;  goto l92;
  l93:;	  G->pos= yypos93; G->thunkpos= yythunkpos93;
  }  if (!yymatchChar(G, '\'')) goto l91;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleSingle", G->buf+G->pos));
  return 1;
  l91:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LabelInline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LabelInline"));
  {  int yypos98= G->pos, yythunkpos98= G->thunkpos;  if (!yy_Str(G)) { goto l99; }  goto l98;
  l99:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;  if (!yy_Endline(G)) { goto l100; }  goto l98;
  l100:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;  if (!yy_Space(G)) { goto l101; }  goto l98;
  l101:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;  if (!yy_RawHtml(G)) { goto l102; }  goto l98;
  l102:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;  if (!yy_Entity(G)) { goto l103; }  goto l98;
  l103:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;  if (!yy_EscapedChar(G)) { goto l104; }  goto l98;
  l104:;	  G->pos= yypos98; G->thunkpos= yythunkpos98;  if (!yy_Symbol(G)) { goto l97; }
  }
  l98:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "LabelInline", G->buf+G->pos));
  return 1;
  l97:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LabelInline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitle"));
  {  int yypos106= G->pos, yythunkpos106= G->thunkpos;  if (!yy_RefTitleSingle(G)) { goto l107; }  goto l106;
  l107:;	  G->pos= yypos106; G->thunkpos= yythunkpos106;  if (!yy_RefTitleDouble(G)) { goto l108; }  goto l106;
  l108:;	  G->pos= yypos106; G->thunkpos= yythunkpos106;  if (!yy_RefTitleParens(G)) { goto l109; }  goto l106;
  l109:;	  G->pos= yypos106; G->thunkpos= yythunkpos106;  if (!yy_EmptyTitle(G)) { goto l105; }
  }
  l106:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitle", G->buf+G->pos));
  return 1;
  l105:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefSrc(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefSrc"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l110;  if (!yy_Nonspacechar(G)) { goto l110; }
  l111:;	
  {  int yypos112= G->pos, yythunkpos112= G->thunkpos;  if (!yy_Nonspacechar(G)) { goto l112; }  goto l111;
  l112:;	  G->pos= yypos112; G->thunkpos= yythunkpos112;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l110;  yyDo(G, yy_1_RefSrc, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RefSrc", G->buf+G->pos));
  return 1;
  l110:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefSrc", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLinkEmail(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AutoLinkEmail"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l113;  if (!yy_LocMarker(G)) { goto l113; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_AutoLinkEmail, G->begin, G->end);  if (!yymatchChar(G, '<')) goto l113;
  {  int yypos114= G->pos, yythunkpos114= G->thunkpos;  if (!yymatchString(G, "mailto:")) goto l114;  goto l115;
  l114:;	  G->pos= yypos114; G->thunkpos= yythunkpos114;
  }
  l115:;	  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l113;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\062\350\377\003\376\377\377\207\376\377\377\107\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l113;
  l116:;	
  {  int yypos117= G->pos, yythunkpos117= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\062\350\377\003\376\377\377\207\376\377\377\107\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l117;  goto l116;
  l117:;	  G->pos= yypos117; G->thunkpos= yythunkpos117;
  }  if (!yymatchChar(G, '@')) goto l113;
  {  int yypos120= G->pos, yythunkpos120= G->thunkpos;  if (!yy_Newline(G)) { goto l120; }  goto l113;
  l120:;	  G->pos= yypos120; G->thunkpos= yythunkpos120;
  }
  {  int yypos121= G->pos, yythunkpos121= G->thunkpos;  if (!yymatchChar(G, '>')) goto l121;  goto l113;
  l121:;	  G->pos= yypos121; G->thunkpos= yythunkpos121;
  }  if (!yymatchDot(G)) goto l113;
  l118:;	
  {  int yypos119= G->pos, yythunkpos119= G->thunkpos;
  {  int yypos122= G->pos, yythunkpos122= G->thunkpos;  if (!yy_Newline(G)) { goto l122; }  goto l119;
  l122:;	  G->pos= yypos122; G->thunkpos= yythunkpos122;
  }
  {  int yypos123= G->pos, yythunkpos123= G->thunkpos;  if (!yymatchChar(G, '>')) goto l123;  goto l119;
  l123:;	  G->pos= yypos123; G->thunkpos= yythunkpos123;
  }  if (!yymatchDot(G)) goto l119;  goto l118;
  l119:;	  G->pos= yypos119; G->thunkpos= yythunkpos119;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l113;  yyDo(G, yy_2_AutoLinkEmail, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l113;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l113;  yyDo(G, yy_3_AutoLinkEmail, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLinkEmail", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l113:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLinkEmail", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLinkUrl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AutoLinkUrl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l124;  if (!yy_LocMarker(G)) { goto l124; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_AutoLinkUrl, G->begin, G->end);  if (!yymatchChar(G, '<')) goto l124;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l124;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l124;
  l125:;	
  {  int yypos126= G->pos, yythunkpos126= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l126;  goto l125;
  l126:;	  G->pos= yypos126; G->thunkpos= yythunkpos126;
  }  if (!yymatchString(G, "://")) goto l124;
  {  int yypos129= G->pos, yythunkpos129= G->thunkpos;  if (!yy_Newline(G)) { goto l129; }  goto l124;
  l129:;	  G->pos= yypos129; G->thunkpos= yythunkpos129;
  }
  {  int yypos130= G->pos, yythunkpos130= G->thunkpos;  if (!yymatchChar(G, '>')) goto l130;  goto l124;
  l130:;	  G->pos= yypos130; G->thunkpos= yythunkpos130;
  }  if (!yymatchDot(G)) goto l124;
  l127:;	
  {  int yypos128= G->pos, yythunkpos128= G->thunkpos;
  {  int yypos131= G->pos, yythunkpos131= G->thunkpos;  if (!yy_Newline(G)) { goto l131; }  goto l128;
  l131:;	  G->pos= yypos131; G->thunkpos= yythunkpos131;
  }
  {  int yypos132= G->pos, yythunkpos132= G->thunkpos;  if (!yymatchChar(G, '>')) goto l132;  goto l128;
  l132:;	  G->pos= yypos132; G->thunkpos= yythunkpos132;
  }  if (!yymatchDot(G)) goto l128;  goto l127;
  l128:;	  G->pos= yypos128; G->thunkpos= yythunkpos128;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l124;  yyDo(G, yy_2_AutoLinkUrl, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l124;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l124;  yyDo(G, yy_3_AutoLinkUrl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLinkUrl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l124:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLinkUrl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ImageSizeHeight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSizeHeight"));  if (!yymatchString(G, "=x")) goto l133;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l133;
  l134:;	
  {  int yypos135= G->pos, yythunkpos135= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l135;  goto l134;
  l135:;	  G->pos= yypos135; G->thunkpos= yythunkpos135;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSizeHeight", G->buf+G->pos));
  return 1;
  l133:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSizeHeight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ImageSizeWidth(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSizeWidth"));  if (!yymatchChar(G, '=')) goto l136;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l136;
  l137:;	
  {  int yypos138= G->pos, yythunkpos138= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l138;  goto l137;
  l138:;	  G->pos= yypos138; G->thunkpos= yythunkpos138;
  }  if (!yymatchChar(G, 'x')) goto l136;
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSizeWidth", G->buf+G->pos));
  return 1;
  l136:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSizeWidth", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ImageSizeComplete(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSizeComplete"));  if (!yymatchChar(G, '=')) goto l139;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l139;
  l140:;	
  {  int yypos141= G->pos, yythunkpos141= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l141;  goto l140;
  l141:;	  G->pos= yypos141; G->thunkpos= yythunkpos141;
  }  if (!yymatchChar(G, 'x')) goto l139;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l139;
  l142:;	
  {  int yypos143= G->pos, yythunkpos143= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l143;  goto l142;
  l143:;	  G->pos= yypos143; G->thunkpos= yythunkpos143;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSizeComplete", G->buf+G->pos));
  return 1;
  l139:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSizeComplete", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleDoubleExt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleDoubleExt"));  if (!yymatchChar(G, '"')) goto l144;
  l145:;	
  {  int yypos146= G->pos, yythunkpos146= G->thunkpos;
  {  int yypos147= G->pos, yythunkpos147= G->thunkpos;
  {  int yypos148= G->pos, yythunkpos148= G->thunkpos;  if (!yymatchChar(G, '"')) goto l149;  goto l148;
  l149:;	  G->pos= yypos148; G->thunkpos= yythunkpos148;  if (!yy_Newline(G)) { goto l147; }
  }
  l148:;	  goto l146;
  l147:;	  G->pos= yypos147; G->thunkpos= yythunkpos147;
  }  if (!yymatchDot(G)) goto l146;  goto l145;
  l146:;	  G->pos= yypos146; G->thunkpos= yythunkpos146;
  }  if (!yymatchChar(G, '"')) goto l144;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleDoubleExt", G->buf+G->pos));
  return 1;
  l144:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleDoubleExt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleSingleExt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleSingleExt"));  if (!yymatchChar(G, '\'')) goto l150;
  l151:;	
  {  int yypos152= G->pos, yythunkpos152= G->thunkpos;
  {  int yypos153= G->pos, yythunkpos153= G->thunkpos;
  {  int yypos154= G->pos, yythunkpos154= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l155;  goto l154;
  l155:;	  G->pos= yypos154; G->thunkpos= yythunkpos154;  if (!yy_Newline(G)) { goto l153; }
  }
  l154:;	  goto l152;
  l153:;	  G->pos= yypos153; G->thunkpos= yythunkpos153;
  }  if (!yymatchDot(G)) goto l152;  goto l151;
  l152:;	  G->pos= yypos152; G->thunkpos= yythunkpos152;
  }  if (!yymatchChar(G, '\'')) goto l150;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleSingleExt", G->buf+G->pos));
  return 1;
  l150:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleSingleExt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleDouble"));  if (!yymatchChar(G, '"')) goto l156;
  l157:;	
  {  int yypos158= G->pos, yythunkpos158= G->thunkpos;
  {  int yypos159= G->pos, yythunkpos159= G->thunkpos;  if (!yymatchChar(G, '"')) goto l159;  if (!yy_Sp(G)) { goto l159; }
  {  int yypos160= G->pos, yythunkpos160= G->thunkpos;  if (!yymatchChar(G, ')')) goto l161;  goto l160;
  l161:;	  G->pos= yypos160; G->thunkpos= yythunkpos160;  if (!yy_Newline(G)) { goto l159; }
  }
  l160:;	  goto l158;
  l159:;	  G->pos= yypos159; G->thunkpos= yythunkpos159;
  }  if (!yymatchDot(G)) goto l158;  goto l157;
  l158:;	  G->pos= yypos158; G->thunkpos= yythunkpos158;
  }  if (!yymatchChar(G, '"')) goto l156;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleDouble", G->buf+G->pos));
  return 1;
  l156:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleSingle"));  if (!yymatchChar(G, '\'')) goto l162;
  l163:;	
  {  int yypos164= G->pos, yythunkpos164= G->thunkpos;
  {  int yypos165= G->pos, yythunkpos165= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l165;  if (!yy_Sp(G)) { goto l165; }
  {  int yypos166= G->pos, yythunkpos166= G->thunkpos;  if (!yymatchChar(G, ')')) goto l167;  goto l166;
  l167:;	  G->pos= yypos166; G->thunkpos= yythunkpos166;  if (!yy_Newline(G)) { goto l165; }
  }
  l166:;	  goto l164;
  l165:;	  G->pos= yypos165; G->thunkpos= yythunkpos165;
  }  if (!yymatchDot(G)) goto l164;  goto l163;
  l164:;	  G->pos= yypos164; G->thunkpos= yythunkpos164;
  }  if (!yymatchChar(G, '\'')) goto l162;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleSingle", G->buf+G->pos));
  return 1;
  l162:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SourceContents(GREG *G)
{
  yyprintf((stderr, "%s\n", "SourceContents"));
  l169:;	
  {  int yypos170= G->pos, yythunkpos170= G->thunkpos;
  {  int yypos171= G->pos, yythunkpos171= G->thunkpos;
  {  int yypos175= G->pos, yythunkpos175= G->thunkpos;  if (!yymatchChar(G, '(')) goto l175;  goto l172;
  l175:;	  G->pos= yypos175; G->thunkpos= yythunkpos175;
  }
  {  int yypos176= G->pos, yythunkpos176= G->thunkpos;  if (!yymatchChar(G, ')')) goto l176;  goto l172;
  l176:;	  G->pos= yypos176; G->thunkpos= yythunkpos176;
  }
  {  int yypos177= G->pos, yythunkpos177= G->thunkpos;  if (!yymatchChar(G, '>')) goto l177;  goto l172;
  l177:;	  G->pos= yypos177; G->thunkpos= yythunkpos177;
  }  if (!yy_Nonspacechar(G)) { goto l172; }
  l173:;	
  {  int yypos174= G->pos, yythunkpos174= G->thunkpos;
  {  int yypos178= G->pos, yythunkpos178= G->thunkpos;  if (!yymatchChar(G, '(')) goto l178;  goto l174;
  l178:;	  G->pos= yypos178; G->thunkpos= yythunkpos178;
  }
  {  int yypos179= G->pos, yythunkpos179= G->thunkpos;  if (!yymatchChar(G, ')')) goto l179;  goto l174;
  l179:;	  G->pos= yypos179; G->thunkpos= yythunkpos179;
  }
  {  int yypos180= G->pos, yythunkpos180= G->thunkpos;  if (!yymatchChar(G, '>')) goto l180;  goto l174;
  l180:;	  G->pos= yypos180; G->thunkpos= yythunkpos180;
  }  if (!yy_Nonspacechar(G)) { goto l174; }  goto l173;
  l174:;	  G->pos= yypos174; G->thunkpos= yythunkpos174;
  }  goto l171;
  l172:;	  G->pos= yypos171; G->thunkpos= yythunkpos171;  if (!yymatchChar(G, '(')) goto l170;  if (!yy_SourceContents(G)) { goto l170; }  if (!yymatchChar(G, ')')) goto l170;
  }
  l171:;	  goto l169;
  l170:;	  G->pos= yypos170; G->thunkpos= yythunkpos170;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "SourceContents", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_ImageSize(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSize"));
  {  int yypos182= G->pos, yythunkpos182= G->thunkpos;  if (!yy_ImageSizeComplete(G)) { goto l183; }  goto l182;
  l183:;	  G->pos= yypos182; G->thunkpos= yythunkpos182;  if (!yy_ImageSizeWidth(G)) { goto l184; }  goto l182;
  l184:;	  G->pos= yypos182; G->thunkpos= yythunkpos182;  if (!yy_ImageSizeHeight(G)) { goto l185; }  goto l182;
  l185:;	  G->pos= yypos182; G->thunkpos= yythunkpos182;  if (!yymatchString(G, "")) goto l181;
  }
  l182:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSize", G->buf+G->pos));
  return 1;
  l181:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSize", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleExt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleExt"));
  {  int yypos187= G->pos, yythunkpos187= G->thunkpos;  if (!yy_TitleSingleExt(G)) { goto l188; }  goto l187;
  l188:;	  G->pos= yypos187; G->thunkpos= yythunkpos187;  if (!yy_TitleDoubleExt(G)) { goto l189; }  goto l187;
  l189:;	  G->pos= yypos187; G->thunkpos= yythunkpos187;  if (!yymatchString(G, "")) goto l186;
  }
  l187:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleExt", G->buf+G->pos));
  return 1;
  l186:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleExt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Title(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Title"));
  {  int yypos191= G->pos, yythunkpos191= G->thunkpos;  if (!yy_TitleSingle(G)) { goto l192; }  goto l191;
  l192:;	  G->pos= yypos191; G->thunkpos= yythunkpos191;  if (!yy_TitleDouble(G)) { goto l193; }  goto l191;
  l193:;	  G->pos= yypos191; G->thunkpos= yythunkpos191;  if (!yymatchString(G, "")) goto l190;
  }
  l191:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Title", G->buf+G->pos));
  return 1;
  l190:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Title", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Source(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Source"));  yyDo(G, yy_1_Source, G->begin, G->end);
  {  int yypos195= G->pos, yythunkpos195= G->thunkpos;  if (!yymatchChar(G, '<')) goto l196;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l196;  if (!yy_SourceContents(G)) { goto l196; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l196;  yyDo(G, yy_2_Source, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l196;  goto l195;
  l196:;	  G->pos= yypos195; G->thunkpos= yythunkpos195;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l194;  if (!yy_SourceContents(G)) { goto l194; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l194;  yyDo(G, yy_3_Source, G->begin, G->end);
  }
  l195:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Source", G->buf+G->pos));
  return 1;
  l194:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Source", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Label(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Label"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l197;  if (!yy_LocMarker(G)) { goto l197; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '[')) goto l197;
  {  int yypos198= G->pos, yythunkpos198= G->thunkpos;
  {  int yypos200= G->pos, yythunkpos200= G->thunkpos;  if (!yymatchChar(G, '^')) goto l200;  goto l199;
  l200:;	  G->pos= yypos200; G->thunkpos= yythunkpos200;
  }  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l199;  goto l198;
  l199:;	  G->pos= yypos198; G->thunkpos= yythunkpos198;
  {  int yypos201= G->pos, yythunkpos201= G->thunkpos;  if (!yymatchDot(G)) goto l197;  G->pos= yypos201; G->thunkpos= yythunkpos201;
  }  yyText(G, G->begin, G->end);  if (!( !EXT(pmh_EXT_NOTES) )) goto l197;
  }
  l198:;	  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l197;
  l202:;	
  {  int yypos203= G->pos, yythunkpos203= G->thunkpos;
  {  int yypos204= G->pos, yythunkpos204= G->thunkpos;  if (!yymatchChar(G, ']')) goto l204;  goto l203;
  l204:;	  G->pos= yypos204; G->thunkpos= yythunkpos204;
  }
  {  int yypos205= G->pos, yythunkpos205= G->thunkpos;  if (!yymatchChar(G, '[')) goto l205;  goto l203;
  l205:;	  G->pos= yypos205; G->thunkpos= yythunkpos205;
  }  if (!yy_LabelInline(G)) { goto l203; }  goto l202;
  l203:;	  G->pos= yypos203; G->thunkpos= yythunkpos203;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l197;  yyDo(G, yy_1_Label, G->begin, G->end);  if (!yymatchChar(G, ']')) goto l197;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l197;  yyDo(G, yy_2_Label, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Label", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l197:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Label", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLinkSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ReferenceLinkSingle"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l206;  if (!yy_Label(G)) { goto l206; }  yyDo(G, yySet, -1, 0);
  {  int yypos207= G->pos, yythunkpos207= G->thunkpos;  if (!yy_Spnl(G)) { goto l207; }  if (!yymatchString(G, "[]")) goto l207;  goto l208;
  l207:;	  G->pos= yypos207; G->thunkpos= yythunkpos207;
  }
  l208:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l206;  yyDo(G, yy_1_ReferenceLinkSingle, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLinkSingle", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l206:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLinkSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLinkDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ReferenceLinkDouble"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l209;  if (!yy_Label(G)) { goto l209; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l209; }
  {  int yypos210= G->pos, yythunkpos210= G->thunkpos;  if (!yymatchString(G, "[]")) goto l210;  goto l209;
  l210:;	  G->pos= yypos210; G->thunkpos= yythunkpos210;
  }  if (!yy_Label(G)) { goto l209; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l209;  yyDo(G, yy_1_ReferenceLinkDouble, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLinkDouble", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l209:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLinkDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AutoLink"));
  {  int yypos212= G->pos, yythunkpos212= G->thunkpos;  if (!yy_AutoLinkUrl(G)) { goto l213; }  goto l212;
  l213:;	  G->pos= yypos212; G->thunkpos= yythunkpos212;  if (!yy_AutoLinkEmail(G)) { goto l211; }
  }
  l212:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLink", G->buf+G->pos));
  return 1;
  l211:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ReferenceLink"));
  {  int yypos215= G->pos, yythunkpos215= G->thunkpos;  if (!yy_ReferenceLinkDouble(G)) { goto l216; }  goto l215;
  l216:;	  G->pos= yypos215; G->thunkpos= yythunkpos215;  if (!yy_ReferenceLinkSingle(G)) { goto l214; }
  }
  l215:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLink", G->buf+G->pos));
  return 1;
  l214:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExplicitLinkSize(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ExplicitLinkSize"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l217;  if (!yy_Label(G)) { goto l217; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l217; }  if (!yymatchChar(G, '(')) goto l217;  if (!yy_Sp(G)) { goto l217; }  if (!yy_Source(G)) { goto l217; }  yyDo(G, yySet, -1, 0);  if (!yy_Spnl(G)) { goto l217; }  if (!yy_TitleExt(G)) { goto l217; }  if (!yy_Sp(G)) { goto l217; }  if (!yy_ImageSize(G)) { goto l217; }  if (!yy_Sp(G)) { goto l217; }  if (!yymatchChar(G, ')')) goto l217;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l217;  yyDo(G, yy_1_ExplicitLinkSize, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ExplicitLinkSize", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l217:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExplicitLinkSize", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExplicitLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ExplicitLink"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l218;  if (!yy_Label(G)) { goto l218; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l218; }  if (!yymatchChar(G, '(')) goto l218;  if (!yy_Sp(G)) { goto l218; }  if (!yy_Source(G)) { goto l218; }  yyDo(G, yySet, -1, 0);  if (!yy_Spnl(G)) { goto l218; }  if (!yy_Title(G)) { goto l218; }  if (!yy_Sp(G)) { goto l218; }  if (!yymatchChar(G, ')')) goto l218;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l218;  yyDo(G, yy_1_ExplicitLink, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ExplicitLink", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l218:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExplicitLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StrongUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StrongUl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l219;  if (!yy_LocMarker(G)) { goto l219; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "__")) goto l219;
  {  int yypos220= G->pos, yythunkpos220= G->thunkpos;  if (!yy_Whitespace(G)) { goto l220; }  goto l219;
  l220:;	  G->pos= yypos220; G->thunkpos= yythunkpos220;
  }
  {  int yypos223= G->pos, yythunkpos223= G->thunkpos;  if (!yymatchString(G, "__")) goto l223;  goto l219;
  l223:;	  G->pos= yypos223; G->thunkpos= yythunkpos223;
  }  if (!yy_Inline(G)) { goto l219; }
  l221:;	
  {  int yypos222= G->pos, yythunkpos222= G->thunkpos;
  {  int yypos224= G->pos, yythunkpos224= G->thunkpos;  if (!yymatchString(G, "__")) goto l224;  goto l222;
  l224:;	  G->pos= yypos224; G->thunkpos= yythunkpos224;
  }  if (!yy_Inline(G)) { goto l222; }  goto l221;
  l222:;	  G->pos= yypos222; G->thunkpos= yythunkpos222;
  }  if (!yymatchString(G, "__")) goto l219;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l219;  yyDo(G, yy_1_StrongUl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StrongUl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l219:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StrongUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StrongStar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StrongStar"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l225;  if (!yy_LocMarker(G)) { goto l225; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "**")) goto l225;
  {  int yypos226= G->pos, yythunkpos226= G->thunkpos;  if (!yy_Whitespace(G)) { goto l226; }  goto l225;
  l226:;	  G->pos= yypos226; G->thunkpos= yythunkpos226;
  }
  {  int yypos229= G->pos, yythunkpos229= G->thunkpos;  if (!yymatchString(G, "**")) goto l229;  goto l225;
  l229:;	  G->pos= yypos229; G->thunkpos= yythunkpos229;
  }  if (!yy_Inline(G)) { goto l225; }
  l227:;	
  {  int yypos228= G->pos, yythunkpos228= G->thunkpos;
  {  int yypos230= G->pos, yythunkpos230= G->thunkpos;  if (!yymatchString(G, "**")) goto l230;  goto l228;
  l230:;	  G->pos= yypos230; G->thunkpos= yythunkpos230;
  }  if (!yy_Inline(G)) { goto l228; }  goto l227;
  l228:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;
  }  if (!yymatchString(G, "**")) goto l225;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l225;  yyDo(G, yy_1_StrongStar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StrongStar", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l225:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StrongStar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Whitespace(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Whitespace"));
  {  int yypos232= G->pos, yythunkpos232= G->thunkpos;  if (!yy_Spacechar(G)) { goto l233; }  goto l232;
  l233:;	  G->pos= yypos232; G->thunkpos= yythunkpos232;  if (!yy_Newline(G)) { goto l231; }
  }
  l232:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Whitespace", G->buf+G->pos));
  return 1;
  l231:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Whitespace", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EmphUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "EmphUl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l234;  if (!yy_LocMarker(G)) { goto l234; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '_')) goto l234;
  {  int yypos235= G->pos, yythunkpos235= G->thunkpos;  if (!yy_Whitespace(G)) { goto l235; }  goto l234;
  l235:;	  G->pos= yypos235; G->thunkpos= yythunkpos235;
  }
  {  int yypos238= G->pos, yythunkpos238= G->thunkpos;
  {  int yypos240= G->pos, yythunkpos240= G->thunkpos;  if (!yymatchChar(G, '_')) goto l240;  goto l239;
  l240:;	  G->pos= yypos240; G->thunkpos= yythunkpos240;
  }  if (!yy_Inline(G)) { goto l239; }  goto l238;
  l239:;	  G->pos= yypos238; G->thunkpos= yythunkpos238;  if (!yy_StrongUl(G)) { goto l234; }
  }
  l238:;	
  l236:;	
  {  int yypos237= G->pos, yythunkpos237= G->thunkpos;
  {  int yypos241= G->pos, yythunkpos241= G->thunkpos;
  {  int yypos243= G->pos, yythunkpos243= G->thunkpos;  if (!yymatchChar(G, '_')) goto l243;  goto l242;
  l243:;	  G->pos= yypos243; G->thunkpos= yythunkpos243;
  }  if (!yy_Inline(G)) { goto l242; }  goto l241;
  l242:;	  G->pos= yypos241; G->thunkpos= yythunkpos241;  if (!yy_StrongUl(G)) { goto l237; }
  }
  l241:;	  goto l236;
  l237:;	  G->pos= yypos237; G->thunkpos= yythunkpos237;
  }  if (!yymatchChar(G, '_')) goto l234;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l234;  yyDo(G, yy_1_EmphUl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "EmphUl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l234:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmphUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EmphStar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "EmphStar"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l244;  if (!yy_LocMarker(G)) { goto l244; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '*')) goto l244;
  {  int yypos245= G->pos, yythunkpos245= G->thunkpos;  if (!yy_Whitespace(G)) { goto l245; }  goto l244;
  l245:;	  G->pos= yypos245; G->thunkpos= yythunkpos245;
  }
  {  int yypos248= G->pos, yythunkpos248= G->thunkpos;
  {  int yypos250= G->pos, yythunkpos250= G->thunkpos;  if (!yymatchChar(G, '*')) goto l250;  goto l249;
  l250:;	  G->pos= yypos250; G->thunkpos= yythunkpos250;
  }  if (!yy_Inline(G)) { goto l249; }  goto l248;
  l249:;	  G->pos= yypos248; G->thunkpos= yythunkpos248;  if (!yy_StrongStar(G)) { goto l244; }
  }
  l248:;	
  l246:;	
  {  int yypos247= G->pos, yythunkpos247= G->thunkpos;
  {  int yypos251= G->pos, yythunkpos251= G->thunkpos;
  {  int yypos253= G->pos, yythunkpos253= G->thunkpos;  if (!yymatchChar(G, '*')) goto l253;  goto l252;
  l253:;	  G->pos= yypos253; G->thunkpos= yythunkpos253;
  }  if (!yy_Inline(G)) { goto l252; }  goto l251;
  l252:;	  G->pos= yypos251; G->thunkpos= yythunkpos251;  if (!yy_StrongStar(G)) { goto l247; }
  }
  l251:;	  goto l246;
  l247:;	  G->pos= yypos247; G->thunkpos= yythunkpos247;
  }  if (!yymatchChar(G, '*')) goto l244;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l244;  yyDo(G, yy_1_EmphStar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "EmphStar", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l244:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmphStar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StarLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StarLine"));
  {  int yypos255= G->pos, yythunkpos255= G->thunkpos;  if (!yymatchString(G, "****")) goto l256;
  l257:;	
  {  int yypos258= G->pos, yythunkpos258= G->thunkpos;  if (!yymatchChar(G, '*')) goto l258;  goto l257;
  l258:;	  G->pos= yypos258; G->thunkpos= yythunkpos258;
  }  goto l255;
  l256:;	  G->pos= yypos255; G->thunkpos= yythunkpos255;  if (!yy_Spacechar(G)) { goto l254; }  if (!yymatchChar(G, '*')) goto l254;
  l259:;	
  {  int yypos260= G->pos, yythunkpos260= G->thunkpos;  if (!yymatchChar(G, '*')) goto l260;  goto l259;
  l260:;	  G->pos= yypos260; G->thunkpos= yythunkpos260;
  }
  {  int yypos261= G->pos, yythunkpos261= G->thunkpos;  if (!yy_Spacechar(G)) { goto l254; }  G->pos= yypos261; G->thunkpos= yythunkpos261;
  }
  }
  l255:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "StarLine", G->buf+G->pos));
  return 1;
  l254:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StarLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_UlLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "UlLine"));
  {  int yypos263= G->pos, yythunkpos263= G->thunkpos;  if (!yymatchString(G, "____")) goto l264;
  l265:;	
  {  int yypos266= G->pos, yythunkpos266= G->thunkpos;  if (!yymatchChar(G, '_')) goto l266;  goto l265;
  l266:;	  G->pos= yypos266; G->thunkpos= yythunkpos266;
  }  goto l263;
  l264:;	  G->pos= yypos263; G->thunkpos= yythunkpos263;  if (!yy_Spacechar(G)) { goto l262; }  if (!yymatchChar(G, '_')) goto l262;
  l267:;	
  {  int yypos268= G->pos, yythunkpos268= G->thunkpos;  if (!yymatchChar(G, '_')) goto l268;  goto l267;
  l268:;	  G->pos= yypos268; G->thunkpos= yythunkpos268;
  }
  {  int yypos269= G->pos, yythunkpos269= G->thunkpos;  if (!yy_Spacechar(G)) { goto l262; }  G->pos= yypos269; G->thunkpos= yythunkpos269;
  }
  }
  l263:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "UlLine", G->buf+G->pos));
  return 1;
  l262:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "UlLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SpecialChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SpecialChar"));
  {  int yypos271= G->pos, yythunkpos271= G->thunkpos;  if (!yymatchChar(G, '~')) goto l272;  goto l271;
  l272:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '*')) goto l273;  goto l271;
  l273:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '_')) goto l274;  goto l271;
  l274:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '`')) goto l275;  goto l271;
  l275:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '&')) goto l276;  goto l271;
  l276:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '[')) goto l277;  goto l271;
  l277:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, ']')) goto l278;  goto l271;
  l278:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '(')) goto l279;  goto l271;
  l279:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, ')')) goto l280;  goto l271;
  l280:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '<')) goto l281;  goto l271;
  l281:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '!')) goto l282;  goto l271;
  l282:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '#')) goto l283;  goto l271;
  l283:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '\\')) goto l284;  goto l271;
  l284:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '\'')) goto l285;  goto l271;
  l285:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yymatchChar(G, '"')) goto l286;  goto l271;
  l286:;	  G->pos= yypos271; G->thunkpos= yythunkpos271;  if (!yy_ExtendedSpecialChar(G)) { goto l270; }
  }
  l271:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SpecialChar", G->buf+G->pos));
  return 1;
  l270:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SpecialChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Eof(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Eof"));
  {  int yypos288= G->pos, yythunkpos288= G->thunkpos;  if (!yymatchDot(G)) goto l288;  goto l287;
  l288:;	  G->pos= yypos288; G->thunkpos= yythunkpos288;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Eof", G->buf+G->pos));
  return 1;
  l287:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Eof", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NormalEndline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NormalEndline"));  if (!yy_Sp(G)) { goto l289; }  if (!yy_Newline(G)) { goto l289; }
  {  int yypos290= G->pos, yythunkpos290= G->thunkpos;  if (!yy_BlankLine(G)) { goto l290; }  goto l289;
  l290:;	  G->pos= yypos290; G->thunkpos= yythunkpos290;
  }
  {  int yypos291= G->pos, yythunkpos291= G->thunkpos;  if (!yymatchChar(G, '>')) goto l291;  goto l289;
  l291:;	  G->pos= yypos291; G->thunkpos= yythunkpos291;
  }
  {  int yypos292= G->pos, yythunkpos292= G->thunkpos;  if (!yy_AtxStart(G)) { goto l292; }  goto l289;
  l292:;	  G->pos= yypos292; G->thunkpos= yythunkpos292;
  }
  {  int yypos293= G->pos, yythunkpos293= G->thunkpos;  if (!yy_Line(G)) { goto l293; }
  {  int yypos294= G->pos, yythunkpos294= G->thunkpos;  if (!yymatchChar(G, '=')) goto l295;
  l296:;	
  {  int yypos297= G->pos, yythunkpos297= G->thunkpos;  if (!yymatchChar(G, '=')) goto l297;  goto l296;
  l297:;	  G->pos= yypos297; G->thunkpos= yythunkpos297;
  }  goto l294;
  l295:;	  G->pos= yypos294; G->thunkpos= yythunkpos294;  if (!yymatchChar(G, '-')) goto l293;
  l298:;	
  {  int yypos299= G->pos, yythunkpos299= G->thunkpos;  if (!yymatchChar(G, '-')) goto l299;  goto l298;
  l299:;	  G->pos= yypos299; G->thunkpos= yythunkpos299;
  }
  }
  l294:;	  if (!yy_Newline(G)) { goto l293; }  goto l289;
  l293:;	  G->pos= yypos293; G->thunkpos= yythunkpos293;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "NormalEndline", G->buf+G->pos));
  return 1;
  l289:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NormalEndline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TerminalEndline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TerminalEndline"));  if (!yy_Sp(G)) { goto l300; }  if (!yy_Newline(G)) { goto l300; }  if (!yy_Eof(G)) { goto l300; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TerminalEndline", G->buf+G->pos));
  return 1;
  l300:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TerminalEndline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LineBreak(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LineBreak"));  if (!yymatchString(G, "  ")) goto l301;  if (!yy_NormalEndline(G)) { goto l301; }
  yyprintf((stderr, "  ok   %s @ %s\n", "LineBreak", G->buf+G->pos));
  return 1;
  l301:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LineBreak", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_CharEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "CharEntity"));  if (!yymatchChar(G, '&')) goto l302;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l302;
  l303:;	
  {  int yypos304= G->pos, yythunkpos304= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l304;  goto l303;
  l304:;	  G->pos= yypos304; G->thunkpos= yythunkpos304;
  }  if (!yymatchChar(G, ';')) goto l302;
  yyprintf((stderr, "  ok   %s @ %s\n", "CharEntity", G->buf+G->pos));
  return 1;
  l302:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "CharEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DecEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DecEntity"));  if (!yymatchChar(G, '&')) goto l305;  if (!yymatchChar(G, '#')) goto l305;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l305;
  l306:;	
  {  int yypos307= G->pos, yythunkpos307= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l307;  goto l306;
  l307:;	  G->pos= yypos307; G->thunkpos= yythunkpos307;
  }  if (!yymatchChar(G, ';')) goto l305;
  yyprintf((stderr, "  ok   %s @ %s\n", "DecEntity", G->buf+G->pos));
  return 1;
  l305:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DecEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HexEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HexEntity"));  if (!yymatchChar(G, '&')) goto l308;  if (!yymatchChar(G, '#')) goto l308;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\000\000\000\001\000\000\000\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l308;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\176\000\000\000\176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l308;
  l309:;	
  {  int yypos310= G->pos, yythunkpos310= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\176\000\000\000\176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l310;  goto l309;
  l310:;	  G->pos= yypos310; G->thunkpos= yythunkpos310;
  }  if (!yymatchChar(G, ';')) goto l308;
  yyprintf((stderr, "  ok   %s @ %s\n", "HexEntity", G->buf+G->pos));
  return 1;
  l308:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HexEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Alphanumeric(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Alphanumeric"));
  {  int yypos312= G->pos, yythunkpos312= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l313;  goto l312;
  l313:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\200")) goto l314;  goto l312;
  l314:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\201")) goto l315;  goto l312;
  l315:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\202")) goto l316;  goto l312;
  l316:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\203")) goto l317;  goto l312;
  l317:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\204")) goto l318;  goto l312;
  l318:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\205")) goto l319;  goto l312;
  l319:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\206")) goto l320;  goto l312;
  l320:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\207")) goto l321;  goto l312;
  l321:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\210")) goto l322;  goto l312;
  l322:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\211")) goto l323;  goto l312;
  l323:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\212")) goto l324;  goto l312;
  l324:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\213")) goto l325;  goto l312;
  l325:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\214")) goto l326;  goto l312;
  l326:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\215")) goto l327;  goto l312;
  l327:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\216")) goto l328;  goto l312;
  l328:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\217")) goto l329;  goto l312;
  l329:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\220")) goto l330;  goto l312;
  l330:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\221")) goto l331;  goto l312;
  l331:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\222")) goto l332;  goto l312;
  l332:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\223")) goto l333;  goto l312;
  l333:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\224")) goto l334;  goto l312;
  l334:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\225")) goto l335;  goto l312;
  l335:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\226")) goto l336;  goto l312;
  l336:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\227")) goto l337;  goto l312;
  l337:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\230")) goto l338;  goto l312;
  l338:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\231")) goto l339;  goto l312;
  l339:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\232")) goto l340;  goto l312;
  l340:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\233")) goto l341;  goto l312;
  l341:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\234")) goto l342;  goto l312;
  l342:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\235")) goto l343;  goto l312;
  l343:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\236")) goto l344;  goto l312;
  l344:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\237")) goto l345;  goto l312;
  l345:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\240")) goto l346;  goto l312;
  l346:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\241")) goto l347;  goto l312;
  l347:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\242")) goto l348;  goto l312;
  l348:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\243")) goto l349;  goto l312;
  l349:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\244")) goto l350;  goto l312;
  l350:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\245")) goto l351;  goto l312;
  l351:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\246")) goto l352;  goto l312;
  l352:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\247")) goto l353;  goto l312;
  l353:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\250")) goto l354;  goto l312;
  l354:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\251")) goto l355;  goto l312;
  l355:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\252")) goto l356;  goto l312;
  l356:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\253")) goto l357;  goto l312;
  l357:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\254")) goto l358;  goto l312;
  l358:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\255")) goto l359;  goto l312;
  l359:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\256")) goto l360;  goto l312;
  l360:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\257")) goto l361;  goto l312;
  l361:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\260")) goto l362;  goto l312;
  l362:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\261")) goto l363;  goto l312;
  l363:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\262")) goto l364;  goto l312;
  l364:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\263")) goto l365;  goto l312;
  l365:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\264")) goto l366;  goto l312;
  l366:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\265")) goto l367;  goto l312;
  l367:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\266")) goto l368;  goto l312;
  l368:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\267")) goto l369;  goto l312;
  l369:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\270")) goto l370;  goto l312;
  l370:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\271")) goto l371;  goto l312;
  l371:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\272")) goto l372;  goto l312;
  l372:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\273")) goto l373;  goto l312;
  l373:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\274")) goto l374;  goto l312;
  l374:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\275")) goto l375;  goto l312;
  l375:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\276")) goto l376;  goto l312;
  l376:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\277")) goto l377;  goto l312;
  l377:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\300")) goto l378;  goto l312;
  l378:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\301")) goto l379;  goto l312;
  l379:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\302")) goto l380;  goto l312;
  l380:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\303")) goto l381;  goto l312;
  l381:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\304")) goto l382;  goto l312;
  l382:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\305")) goto l383;  goto l312;
  l383:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\306")) goto l384;  goto l312;
  l384:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\307")) goto l385;  goto l312;
  l385:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\310")) goto l386;  goto l312;
  l386:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\311")) goto l387;  goto l312;
  l387:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\312")) goto l388;  goto l312;
  l388:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\313")) goto l389;  goto l312;
  l389:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\314")) goto l390;  goto l312;
  l390:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\315")) goto l391;  goto l312;
  l391:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\316")) goto l392;  goto l312;
  l392:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\317")) goto l393;  goto l312;
  l393:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\320")) goto l394;  goto l312;
  l394:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\321")) goto l395;  goto l312;
  l395:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\322")) goto l396;  goto l312;
  l396:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\323")) goto l397;  goto l312;
  l397:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\324")) goto l398;  goto l312;
  l398:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\325")) goto l399;  goto l312;
  l399:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\326")) goto l400;  goto l312;
  l400:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\327")) goto l401;  goto l312;
  l401:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\330")) goto l402;  goto l312;
  l402:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\331")) goto l403;  goto l312;
  l403:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\332")) goto l404;  goto l312;
  l404:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\333")) goto l405;  goto l312;
  l405:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\334")) goto l406;  goto l312;
  l406:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\335")) goto l407;  goto l312;
  l407:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\336")) goto l408;  goto l312;
  l408:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\337")) goto l409;  goto l312;
  l409:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\340")) goto l410;  goto l312;
  l410:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\341")) goto l411;  goto l312;
  l411:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\342")) goto l412;  goto l312;
  l412:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\343")) goto l413;  goto l312;
  l413:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\344")) goto l414;  goto l312;
  l414:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\345")) goto l415;  goto l312;
  l415:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\346")) goto l416;  goto l312;
  l416:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\347")) goto l417;  goto l312;
  l417:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\350")) goto l418;  goto l312;
  l418:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\351")) goto l419;  goto l312;
  l419:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\352")) goto l420;  goto l312;
  l420:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\353")) goto l421;  goto l312;
  l421:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\354")) goto l422;  goto l312;
  l422:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\355")) goto l423;  goto l312;
  l423:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\356")) goto l424;  goto l312;
  l424:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\357")) goto l425;  goto l312;
  l425:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\360")) goto l426;  goto l312;
  l426:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\361")) goto l427;  goto l312;
  l427:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\362")) goto l428;  goto l312;
  l428:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\363")) goto l429;  goto l312;
  l429:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\364")) goto l430;  goto l312;
  l430:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\365")) goto l431;  goto l312;
  l431:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\366")) goto l432;  goto l312;
  l432:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\367")) goto l433;  goto l312;
  l433:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\370")) goto l434;  goto l312;
  l434:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\371")) goto l435;  goto l312;
  l435:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\372")) goto l436;  goto l312;
  l436:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\373")) goto l437;  goto l312;
  l437:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\374")) goto l438;  goto l312;
  l438:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\375")) goto l439;  goto l312;
  l439:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\376")) goto l440;  goto l312;
  l440:;	  G->pos= yypos312; G->thunkpos= yythunkpos312;  if (!yymatchString(G, "\377")) goto l311;
  }
  l312:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Alphanumeric", G->buf+G->pos));
  return 1;
  l311:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Alphanumeric", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NormalChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NormalChar"));
  {  int yypos442= G->pos, yythunkpos442= G->thunkpos;
  {  int yypos443= G->pos, yythunkpos443= G->thunkpos;  if (!yy_SpecialChar(G)) { goto l444; }  goto l443;
  l444:;	  G->pos= yypos443; G->thunkpos= yythunkpos443;  if (!yy_Spacechar(G)) { goto l445; }  goto l443;
  l445:;	  G->pos= yypos443; G->thunkpos= yythunkpos443;  if (!yy_Newline(G)) { goto l442; }
  }
  l443:;	  goto l441;
  l442:;	  G->pos= yypos442; G->thunkpos= yythunkpos442;
  }  if (!yymatchDot(G)) goto l441;
  yyprintf((stderr, "  ok   %s @ %s\n", "NormalChar", G->buf+G->pos));
  return 1;
  l441:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NormalChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Symbol(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Symbol"));  if (!yy_SpecialChar(G)) { goto l446; }
  yyprintf((stderr, "  ok   %s @ %s\n", "Symbol", G->buf+G->pos));
  return 1;
  l446:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Symbol", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EscapedChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "EscapedChar"));  if (!yymatchChar(G, '\\')) goto l447;
  {  int yypos448= G->pos, yythunkpos448= G->thunkpos;  if (!yy_Newline(G)) { goto l448; }  goto l447;
  l448:;	  G->pos= yypos448; G->thunkpos= yythunkpos448;
  }  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\012\157\000\120\000\000\000\270\001\000\000\070\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l447;
  yyprintf((stderr, "  ok   %s @ %s\n", "EscapedChar", G->buf+G->pos));
  return 1;
  l447:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EscapedChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Entity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Entity"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l449;  if (!yy_LocMarker(G)) { goto l449; }  yyDo(G, yySet, -1, 0);
  {  int yypos450= G->pos, yythunkpos450= G->thunkpos;  if (!yy_HexEntity(G)) { goto l451; }  goto l450;
  l451:;	  G->pos= yypos450; G->thunkpos= yythunkpos450;  if (!yy_DecEntity(G)) { goto l452; }  goto l450;
  l452:;	  G->pos= yypos450; G->thunkpos= yythunkpos450;  if (!yy_CharEntity(G)) { goto l449; }
  }
  l450:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l449;  yyDo(G, yy_1_Entity, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Entity", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l449:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Entity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawHtml(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "RawHtml"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l453;  if (!yy_LocMarker(G)) { goto l453; }  yyDo(G, yySet, -1, 0);
  {  int yypos454= G->pos, yythunkpos454= G->thunkpos;  if (!yy_HtmlComment(G)) { goto l455; }  goto l454;
  l455:;	  G->pos= yypos454; G->thunkpos= yythunkpos454;  if (!yy_HtmlBlockScript(G)) { goto l456; }  goto l454;
  l456:;	  G->pos= yypos454; G->thunkpos= yythunkpos454;  if (!yy_HtmlTag(G)) { goto l453; }
  }
  l454:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l453;  yyDo(G, yy_1_RawHtml, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RawHtml", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l453:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawHtml", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Mark(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "Mark"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MARK) )) goto l457;  if (!yy_MarkTagOpen(G)) { goto l457; }  yyDo(G, yySet, -2, 0);  if (!yy_MarkTagText(G)) { goto l457; }  if (!yy_MarkTagClose(G)) { goto l457; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_Mark, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Mark", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l457:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Mark", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Code(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Code"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l458;
  {  int yypos459= G->pos, yythunkpos459= G->thunkpos;  if (!yy_Ticks1(G)) { goto l460; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l460; }
  {  int yypos463= G->pos, yythunkpos463= G->thunkpos;
  {  int yypos467= G->pos, yythunkpos467= G->thunkpos;  if (!yymatchChar(G, '`')) goto l467;  goto l464;
  l467:;	  G->pos= yypos467; G->thunkpos= yythunkpos467;
  }  if (!yy_Nonspacechar(G)) { goto l464; }
  l465:;	
  {  int yypos466= G->pos, yythunkpos466= G->thunkpos;
  {  int yypos468= G->pos, yythunkpos468= G->thunkpos;  if (!yymatchChar(G, '`')) goto l468;  goto l466;
  l468:;	  G->pos= yypos468; G->thunkpos= yythunkpos468;
  }  if (!yy_Nonspacechar(G)) { goto l466; }  goto l465;
  l466:;	  G->pos= yypos466; G->thunkpos= yythunkpos466;
  }  goto l463;
  l464:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;
  {  int yypos470= G->pos, yythunkpos470= G->thunkpos;  if (!yy_Ticks1(G)) { goto l470; }  goto l469;
  l470:;	  G->pos= yypos470; G->thunkpos= yythunkpos470;
  }  if (!yymatchChar(G, '`')) goto l469;
  l471:;	
  {  int yypos472= G->pos, yythunkpos472= G->thunkpos;  if (!yymatchChar(G, '`')) goto l472;  goto l471;
  l472:;	  G->pos= yypos472; G->thunkpos= yythunkpos472;
  }  goto l463;
  l469:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;
  {  int yypos473= G->pos, yythunkpos473= G->thunkpos;  if (!yy_Sp(G)) { goto l473; }  if (!yy_Ticks1(G)) { goto l473; }  goto l460;
  l473:;	  G->pos= yypos473; G->thunkpos= yythunkpos473;
  }
  {  int yypos474= G->pos, yythunkpos474= G->thunkpos;  if (!yy_Spacechar(G)) { goto l475; }  goto l474;
  l475:;	  G->pos= yypos474; G->thunkpos= yythunkpos474;  if (!yy_Newline(G)) { goto l460; }
  {  int yypos476= G->pos, yythunkpos476= G->thunkpos;  if (!yy_BlankLine(G)) { goto l476; }  goto l460;
  l476:;	  G->pos= yypos476; G->thunkpos= yythunkpos476;
  }
  }
  l474:;	
  }
  l463:;	
  l461:;	
  {  int yypos462= G->pos, yythunkpos462= G->thunkpos;
  {  int yypos477= G->pos, yythunkpos477= G->thunkpos;
  {  int yypos481= G->pos, yythunkpos481= G->thunkpos;  if (!yymatchChar(G, '`')) goto l481;  goto l478;
  l481:;	  G->pos= yypos481; G->thunkpos= yythunkpos481;
  }  if (!yy_Nonspacechar(G)) { goto l478; }
  l479:;	
  {  int yypos480= G->pos, yythunkpos480= G->thunkpos;
  {  int yypos482= G->pos, yythunkpos482= G->thunkpos;  if (!yymatchChar(G, '`')) goto l482;  goto l480;
  l482:;	  G->pos= yypos482; G->thunkpos= yythunkpos482;
  }  if (!yy_Nonspacechar(G)) { goto l480; }  goto l479;
  l480:;	  G->pos= yypos480; G->thunkpos= yythunkpos480;
  }  goto l477;
  l478:;	  G->pos= yypos477; G->thunkpos= yythunkpos477;
  {  int yypos484= G->pos, yythunkpos484= G->thunkpos;  if (!yy_Ticks1(G)) { goto l484; }  goto l483;
  l484:;	  G->pos= yypos484; G->thunkpos= yythunkpos484;
  }  if (!yymatchChar(G, '`')) goto l483;
  l485:;	
  {  int yypos486= G->pos, yythunkpos486= G->thunkpos;  if (!yymatchChar(G, '`')) goto l486;  goto l485;
  l486:;	  G->pos= yypos486; G->thunkpos= yythunkpos486;
  }  goto l477;
  l483:;	  G->pos= yypos477; G->thunkpos= yythunkpos477;
  {  int yypos487= G->pos, yythunkpos487= G->thunkpos;  if (!yy_Sp(G)) { goto l487; }  if (!yy_Ticks1(G)) { goto l487; }  goto l462;
  l487:;	  G->pos= yypos487; G->thunkpos= yythunkpos487;
  }
  {  int yypos488= G->pos, yythunkpos488= G->thunkpos;  if (!yy_Spacechar(G)) { goto l489; }  goto l488;
  l489:;	  G->pos= yypos488; G->thunkpos= yythunkpos488;  if (!yy_Newline(G)) { goto l462; }
  {  int yypos490= G->pos, yythunkpos490= G->thunkpos;  if (!yy_BlankLine(G)) { goto l490; }  goto l462;
  l490:;	  G->pos= yypos490; G->thunkpos= yythunkpos490;
  }
  }
  l488:;	
  }
  l477:;	  goto l461;
  l462:;	  G->pos= yypos462; G->thunkpos= yythunkpos462;
  }  if (!yy_Sp(G)) { goto l460; }  if (!yy_Ticks1(G)) { goto l460; }  goto l459;
  l460:;	  G->pos= yypos459; G->thunkpos= yythunkpos459;  if (!yy_Ticks2(G)) { goto l491; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l491; }
  {  int yypos494= G->pos, yythunkpos494= G->thunkpos;
  {  int yypos498= G->pos, yythunkpos498= G->thunkpos;  if (!yymatchChar(G, '`')) goto l498;  goto l495;
  l498:;	  G->pos= yypos498; G->thunkpos= yythunkpos498;
  }  if (!yy_Nonspacechar(G)) { goto l495; }
  l496:;	
  {  int yypos497= G->pos, yythunkpos497= G->thunkpos;
  {  int yypos499= G->pos, yythunkpos499= G->thunkpos;  if (!yymatchChar(G, '`')) goto l499;  goto l497;
  l499:;	  G->pos= yypos499; G->thunkpos= yythunkpos499;
  }  if (!yy_Nonspacechar(G)) { goto l497; }  goto l496;
  l497:;	  G->pos= yypos497; G->thunkpos= yythunkpos497;
  }  goto l494;
  l495:;	  G->pos= yypos494; G->thunkpos= yythunkpos494;
  {  int yypos501= G->pos, yythunkpos501= G->thunkpos;  if (!yy_Ticks2(G)) { goto l501; }  goto l500;
  l501:;	  G->pos= yypos501; G->thunkpos= yythunkpos501;
  }  if (!yymatchChar(G, '`')) goto l500;
  l502:;	
  {  int yypos503= G->pos, yythunkpos503= G->thunkpos;  if (!yymatchChar(G, '`')) goto l503;  goto l502;
  l503:;	  G->pos= yypos503; G->thunkpos= yythunkpos503;
  }  goto l494;
  l500:;	  G->pos= yypos494; G->thunkpos= yythunkpos494;
  {  int yypos504= G->pos, yythunkpos504= G->thunkpos;  if (!yy_Sp(G)) { goto l504; }  if (!yy_Ticks2(G)) { goto l504; }  goto l491;
  l504:;	  G->pos= yypos504; G->thunkpos= yythunkpos504;
  }
  {  int yypos505= G->pos, yythunkpos505= G->thunkpos;  if (!yy_Spacechar(G)) { goto l506; }  goto l505;
  l506:;	  G->pos= yypos505; G->thunkpos= yythunkpos505;  if (!yy_Newline(G)) { goto l491; }
  {  int yypos507= G->pos, yythunkpos507= G->thunkpos;  if (!yy_BlankLine(G)) { goto l507; }  goto l491;
  l507:;	  G->pos= yypos507; G->thunkpos= yythunkpos507;
  }
  }
  l505:;	
  }
  l494:;	
  l492:;	
  {  int yypos493= G->pos, yythunkpos493= G->thunkpos;
  {  int yypos508= G->pos, yythunkpos508= G->thunkpos;
  {  int yypos512= G->pos, yythunkpos512= G->thunkpos;  if (!yymatchChar(G, '`')) goto l512;  goto l509;
  l512:;	  G->pos= yypos512; G->thunkpos= yythunkpos512;
  }  if (!yy_Nonspacechar(G)) { goto l509; }
  l510:;	
  {  int yypos511= G->pos, yythunkpos511= G->thunkpos;
  {  int yypos513= G->pos, yythunkpos513= G->thunkpos;  if (!yymatchChar(G, '`')) goto l513;  goto l511;
  l513:;	  G->pos= yypos513; G->thunkpos= yythunkpos513;
  }  if (!yy_Nonspacechar(G)) { goto l511; }  goto l510;
  l511:;	  G->pos= yypos511; G->thunkpos= yythunkpos511;
  }  goto l508;
  l509:;	  G->pos= yypos508; G->thunkpos= yythunkpos508;
  {  int yypos515= G->pos, yythunkpos515= G->thunkpos;  if (!yy_Ticks2(G)) { goto l515; }  goto l514;
  l515:;	  G->pos= yypos515; G->thunkpos= yythunkpos515;
  }  if (!yymatchChar(G, '`')) goto l514;
  l516:;	
  {  int yypos517= G->pos, yythunkpos517= G->thunkpos;  if (!yymatchChar(G, '`')) goto l517;  goto l516;
  l517:;	  G->pos= yypos517; G->thunkpos= yythunkpos517;
  }  goto l508;
  l514:;	  G->pos= yypos508; G->thunkpos= yythunkpos508;
  {  int yypos518= G->pos, yythunkpos518= G->thunkpos;  if (!yy_Sp(G)) { goto l518; }  if (!yy_Ticks2(G)) { goto l518; }  goto l493;
  l518:;	  G->pos= yypos518; G->thunkpos= yythunkpos518;
  }
  {  int yypos519= G->pos, yythunkpos519= G->thunkpos;  if (!yy_Spacechar(G)) { goto l520; }  goto l519;
  l520:;	  G->pos= yypos519; G->thunkpos= yythunkpos519;  if (!yy_Newline(G)) { goto l493; }
  {  int yypos521= G->pos, yythunkpos521= G->thunkpos;  if (!yy_BlankLine(G)) { goto l521; }  goto l493;
  l521:;	  G->pos= yypos521; G->thunkpos= yythunkpos521;
  }
  }
  l519:;	
  }
  l508:;	  goto l492;
  l493:;	  G->pos= yypos493; G->thunkpos= yythunkpos493;
  }  if (!yy_Sp(G)) { goto l491; }  if (!yy_Ticks2(G)) { goto l491; }  goto l459;
  l491:;	  G->pos= yypos459; G->thunkpos= yythunkpos459;  if (!yy_Ticks3(G)) { goto l522; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l522; }
  {  int yypos525= G->pos, yythunkpos525= G->thunkpos;
  {  int yypos529= G->pos, yythunkpos529= G->thunkpos;  if (!yymatchChar(G, '`')) goto l529;  goto l526;
  l529:;	  G->pos= yypos529; G->thunkpos= yythunkpos529;
  }  if (!yy_Nonspacechar(G)) { goto l526; }
  l527:;	
  {  int yypos528= G->pos, yythunkpos528= G->thunkpos;
  {  int yypos530= G->pos, yythunkpos530= G->thunkpos;  if (!yymatchChar(G, '`')) goto l530;  goto l528;
  l530:;	  G->pos= yypos530; G->thunkpos= yythunkpos530;
  }  if (!yy_Nonspacechar(G)) { goto l528; }  goto l527;
  l528:;	  G->pos= yypos528; G->thunkpos= yythunkpos528;
  }  goto l525;
  l526:;	  G->pos= yypos525; G->thunkpos= yythunkpos525;
  {  int yypos532= G->pos, yythunkpos532= G->thunkpos;  if (!yy_Ticks3(G)) { goto l532; }  goto l531;
  l532:;	  G->pos= yypos532; G->thunkpos= yythunkpos532;
  }  if (!yymatchChar(G, '`')) goto l531;
  l533:;	
  {  int yypos534= G->pos, yythunkpos534= G->thunkpos;  if (!yymatchChar(G, '`')) goto l534;  goto l533;
  l534:;	  G->pos= yypos534; G->thunkpos= yythunkpos534;
  }  goto l525;
  l531:;	  G->pos= yypos525; G->thunkpos= yythunkpos525;
  {  int yypos535= G->pos, yythunkpos535= G->thunkpos;  if (!yy_Sp(G)) { goto l535; }  if (!yy_Ticks3(G)) { goto l535; }  goto l522;
  l535:;	  G->pos= yypos535; G->thunkpos= yythunkpos535;
  }
  {  int yypos536= G->pos, yythunkpos536= G->thunkpos;  if (!yy_Spacechar(G)) { goto l537; }  goto l536;
  l537:;	  G->pos= yypos536; G->thunkpos= yythunkpos536;  if (!yy_Newline(G)) { goto l522; }
  {  int yypos538= G->pos, yythunkpos538= G->thunkpos;  if (!yy_BlankLine(G)) { goto l538; }  goto l522;
  l538:;	  G->pos= yypos538; G->thunkpos= yythunkpos538;
  }
  }
  l536:;	
  }
  l525:;	
  l523:;	
  {  int yypos524= G->pos, yythunkpos524= G->thunkpos;
  {  int yypos539= G->pos, yythunkpos539= G->thunkpos;
  {  int yypos543= G->pos, yythunkpos543= G->thunkpos;  if (!yymatchChar(G, '`')) goto l543;  goto l540;
  l543:;	  G->pos= yypos543; G->thunkpos= yythunkpos543;
  }  if (!yy_Nonspacechar(G)) { goto l540; }
  l541:;	
  {  int yypos542= G->pos, yythunkpos542= G->thunkpos;
  {  int yypos544= G->pos, yythunkpos544= G->thunkpos;  if (!yymatchChar(G, '`')) goto l544;  goto l542;
  l544:;	  G->pos= yypos544; G->thunkpos= yythunkpos544;
  }  if (!yy_Nonspacechar(G)) { goto l542; }  goto l541;
  l542:;	  G->pos= yypos542; G->thunkpos= yythunkpos542;
  }  goto l539;
  l540:;	  G->pos= yypos539; G->thunkpos= yythunkpos539;
  {  int yypos546= G->pos, yythunkpos546= G->thunkpos;  if (!yy_Ticks3(G)) { goto l546; }  goto l545;
  l546:;	  G->pos= yypos546; G->thunkpos= yythunkpos546;
  }  if (!yymatchChar(G, '`')) goto l545;
  l547:;	
  {  int yypos548= G->pos, yythunkpos548= G->thunkpos;  if (!yymatchChar(G, '`')) goto l548;  goto l547;
  l548:;	  G->pos= yypos548; G->thunkpos= yythunkpos548;
  }  goto l539;
  l545:;	  G->pos= yypos539; G->thunkpos= yythunkpos539;
  {  int yypos549= G->pos, yythunkpos549= G->thunkpos;  if (!yy_Sp(G)) { goto l549; }  if (!yy_Ticks3(G)) { goto l549; }  goto l524;
  l549:;	  G->pos= yypos549; G->thunkpos= yythunkpos549;
  }
  {  int yypos550= G->pos, yythunkpos550= G->thunkpos;  if (!yy_Spacechar(G)) { goto l551; }  goto l550;
  l551:;	  G->pos= yypos550; G->thunkpos= yythunkpos550;  if (!yy_Newline(G)) { goto l524; }
  {  int yypos552= G->pos, yythunkpos552= G->thunkpos;  if (!yy_BlankLine(G)) { goto l552; }  goto l524;
  l552:;	  G->pos= yypos552; G->thunkpos= yythunkpos552;
  }
  }
  l550:;	
  }
  l539:;	  goto l523;
  l524:;	  G->pos= yypos524; G->thunkpos= yythunkpos524;
  }  if (!yy_Sp(G)) { goto l522; }  if (!yy_Ticks3(G)) { goto l522; }  goto l459;
  l522:;	  G->pos= yypos459; G->thunkpos= yythunkpos459;  if (!yy_Ticks4(G)) { goto l553; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l553; }
  {  int yypos556= G->pos, yythunkpos556= G->thunkpos;
  {  int yypos560= G->pos, yythunkpos560= G->thunkpos;  if (!yymatchChar(G, '`')) goto l560;  goto l557;
  l560:;	  G->pos= yypos560; G->thunkpos= yythunkpos560;
  }  if (!yy_Nonspacechar(G)) { goto l557; }
  l558:;	
  {  int yypos559= G->pos, yythunkpos559= G->thunkpos;
  {  int yypos561= G->pos, yythunkpos561= G->thunkpos;  if (!yymatchChar(G, '`')) goto l561;  goto l559;
  l561:;	  G->pos= yypos561; G->thunkpos= yythunkpos561;
  }  if (!yy_Nonspacechar(G)) { goto l559; }  goto l558;
  l559:;	  G->pos= yypos559; G->thunkpos= yythunkpos559;
  }  goto l556;
  l557:;	  G->pos= yypos556; G->thunkpos= yythunkpos556;
  {  int yypos563= G->pos, yythunkpos563= G->thunkpos;  if (!yy_Ticks4(G)) { goto l563; }  goto l562;
  l563:;	  G->pos= yypos563; G->thunkpos= yythunkpos563;
  }  if (!yymatchChar(G, '`')) goto l562;
  l564:;	
  {  int yypos565= G->pos, yythunkpos565= G->thunkpos;  if (!yymatchChar(G, '`')) goto l565;  goto l564;
  l565:;	  G->pos= yypos565; G->thunkpos= yythunkpos565;
  }  goto l556;
  l562:;	  G->pos= yypos556; G->thunkpos= yythunkpos556;
  {  int yypos566= G->pos, yythunkpos566= G->thunkpos;  if (!yy_Sp(G)) { goto l566; }  if (!yy_Ticks4(G)) { goto l566; }  goto l553;
  l566:;	  G->pos= yypos566; G->thunkpos= yythunkpos566;
  }
  {  int yypos567= G->pos, yythunkpos567= G->thunkpos;  if (!yy_Spacechar(G)) { goto l568; }  goto l567;
  l568:;	  G->pos= yypos567; G->thunkpos= yythunkpos567;  if (!yy_Newline(G)) { goto l553; }
  {  int yypos569= G->pos, yythunkpos569= G->thunkpos;  if (!yy_BlankLine(G)) { goto l569; }  goto l553;
  l569:;	  G->pos= yypos569; G->thunkpos= yythunkpos569;
  }
  }
  l567:;	
  }
  l556:;	
  l554:;	
  {  int yypos555= G->pos, yythunkpos555= G->thunkpos;
  {  int yypos570= G->pos, yythunkpos570= G->thunkpos;
  {  int yypos574= G->pos, yythunkpos574= G->thunkpos;  if (!yymatchChar(G, '`')) goto l574;  goto l571;
  l574:;	  G->pos= yypos574; G->thunkpos= yythunkpos574;
  }  if (!yy_Nonspacechar(G)) { goto l571; }
  l572:;	
  {  int yypos573= G->pos, yythunkpos573= G->thunkpos;
  {  int yypos575= G->pos, yythunkpos575= G->thunkpos;  if (!yymatchChar(G, '`')) goto l575;  goto l573;
  l575:;	  G->pos= yypos575; G->thunkpos= yythunkpos575;
  }  if (!yy_Nonspacechar(G)) { goto l573; }  goto l572;
  l573:;	  G->pos= yypos573; G->thunkpos= yythunkpos573;
  }  goto l570;
  l571:;	  G->pos= yypos570; G->thunkpos= yythunkpos570;
  {  int yypos577= G->pos, yythunkpos577= G->thunkpos;  if (!yy_Ticks4(G)) { goto l577; }  goto l576;
  l577:;	  G->pos= yypos577; G->thunkpos= yythunkpos577;
  }  if (!yymatchChar(G, '`')) goto l576;
  l578:;	
  {  int yypos579= G->pos, yythunkpos579= G->thunkpos;  if (!yymatchChar(G, '`')) goto l579;  goto l578;
  l579:;	  G->pos= yypos579; G->thunkpos= yythunkpos579;
  }  goto l570;
  l576:;	  G->pos= yypos570; G->thunkpos= yythunkpos570;
  {  int yypos580= G->pos, yythunkpos580= G->thunkpos;  if (!yy_Sp(G)) { goto l580; }  if (!yy_Ticks4(G)) { goto l580; }  goto l555;
  l580:;	  G->pos= yypos580; G->thunkpos= yythunkpos580;
  }
  {  int yypos581= G->pos, yythunkpos581= G->thunkpos;  if (!yy_Spacechar(G)) { goto l582; }  goto l581;
  l582:;	  G->pos= yypos581; G->thunkpos= yythunkpos581;  if (!yy_Newline(G)) { goto l555; }
  {  int yypos583= G->pos, yythunkpos583= G->thunkpos;  if (!yy_BlankLine(G)) { goto l583; }  goto l555;
  l583:;	  G->pos= yypos583; G->thunkpos= yythunkpos583;
  }
  }
  l581:;	
  }
  l570:;	  goto l554;
  l555:;	  G->pos= yypos555; G->thunkpos= yythunkpos555;
  }  if (!yy_Sp(G)) { goto l553; }  if (!yy_Ticks4(G)) { goto l553; }  goto l459;
  l553:;	  G->pos= yypos459; G->thunkpos= yythunkpos459;  if (!yy_Ticks5(G)) { goto l458; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l458; }
  {  int yypos586= G->pos, yythunkpos586= G->thunkpos;
  {  int yypos590= G->pos, yythunkpos590= G->thunkpos;  if (!yymatchChar(G, '`')) goto l590;  goto l587;
  l590:;	  G->pos= yypos590; G->thunkpos= yythunkpos590;
  }  if (!yy_Nonspacechar(G)) { goto l587; }
  l588:;	
  {  int yypos589= G->pos, yythunkpos589= G->thunkpos;
  {  int yypos591= G->pos, yythunkpos591= G->thunkpos;  if (!yymatchChar(G, '`')) goto l591;  goto l589;
  l591:;	  G->pos= yypos591; G->thunkpos= yythunkpos591;
  }  if (!yy_Nonspacechar(G)) { goto l589; }  goto l588;
  l589:;	  G->pos= yypos589; G->thunkpos= yythunkpos589;
  }  goto l586;
  l587:;	  G->pos= yypos586; G->thunkpos= yythunkpos586;
  {  int yypos593= G->pos, yythunkpos593= G->thunkpos;  if (!yy_Ticks5(G)) { goto l593; }  goto l592;
  l593:;	  G->pos= yypos593; G->thunkpos= yythunkpos593;
  }  if (!yymatchChar(G, '`')) goto l592;
  l594:;	
  {  int yypos595= G->pos, yythunkpos595= G->thunkpos;  if (!yymatchChar(G, '`')) goto l595;  goto l594;
  l595:;	  G->pos= yypos595; G->thunkpos= yythunkpos595;
  }  goto l586;
  l592:;	  G->pos= yypos586; G->thunkpos= yythunkpos586;
  {  int yypos596= G->pos, yythunkpos596= G->thunkpos;  if (!yy_Sp(G)) { goto l596; }  if (!yy_Ticks5(G)) { goto l596; }  goto l458;
  l596:;	  G->pos= yypos596; G->thunkpos= yythunkpos596;
  }
  {  int yypos597= G->pos, yythunkpos597= G->thunkpos;  if (!yy_Spacechar(G)) { goto l598; }  goto l597;
  l598:;	  G->pos= yypos597; G->thunkpos= yythunkpos597;  if (!yy_Newline(G)) { goto l458; }
  {  int yypos599= G->pos, yythunkpos599= G->thunkpos;  if (!yy_BlankLine(G)) { goto l599; }  goto l458;
  l599:;	  G->pos= yypos599; G->thunkpos= yythunkpos599;
  }
  }
  l597:;	
  }
  l586:;	
  l584:;	
  {  int yypos585= G->pos, yythunkpos585= G->thunkpos;
  {  int yypos600= G->pos, yythunkpos600= G->thunkpos;
  {  int yypos604= G->pos, yythunkpos604= G->thunkpos;  if (!yymatchChar(G, '`')) goto l604;  goto l601;
  l604:;	  G->pos= yypos604; G->thunkpos= yythunkpos604;
  }  if (!yy_Nonspacechar(G)) { goto l601; }
  l602:;	
  {  int yypos603= G->pos, yythunkpos603= G->thunkpos;
  {  int yypos605= G->pos, yythunkpos605= G->thunkpos;  if (!yymatchChar(G, '`')) goto l605;  goto l603;
  l605:;	  G->pos= yypos605; G->thunkpos= yythunkpos605;
  }  if (!yy_Nonspacechar(G)) { goto l603; }  goto l602;
  l603:;	  G->pos= yypos603; G->thunkpos= yythunkpos603;
  }  goto l600;
  l601:;	  G->pos= yypos600; G->thunkpos= yythunkpos600;
  {  int yypos607= G->pos, yythunkpos607= G->thunkpos;  if (!yy_Ticks5(G)) { goto l607; }  goto l606;
  l607:;	  G->pos= yypos607; G->thunkpos= yythunkpos607;
  }  if (!yymatchChar(G, '`')) goto l606;
  l608:;	
  {  int yypos609= G->pos, yythunkpos609= G->thunkpos;  if (!yymatchChar(G, '`')) goto l609;  goto l608;
  l609:;	  G->pos= yypos609; G->thunkpos= yythunkpos609;
  }  goto l600;
  l606:;	  G->pos= yypos600; G->thunkpos= yythunkpos600;
  {  int yypos610= G->pos, yythunkpos610= G->thunkpos;  if (!yy_Sp(G)) { goto l610; }  if (!yy_Ticks5(G)) { goto l610; }  goto l585;
  l610:;	  G->pos= yypos610; G->thunkpos= yythunkpos610;
  }
  {  int yypos611= G->pos, yythunkpos611= G->thunkpos;  if (!yy_Spacechar(G)) { goto l612; }  goto l611;
  l612:;	  G->pos= yypos611; G->thunkpos= yythunkpos611;  if (!yy_Newline(G)) { goto l585; }
  {  int yypos613= G->pos, yythunkpos613= G->thunkpos;  if (!yy_BlankLine(G)) { goto l613; }  goto l585;
  l613:;	  G->pos= yypos613; G->thunkpos= yythunkpos613;
  }
  }
  l611:;	
  }
  l600:;	  goto l584;
  l585:;	  G->pos= yypos585; G->thunkpos= yythunkpos585;
  }  if (!yy_Sp(G)) { goto l458; }  if (!yy_Ticks5(G)) { goto l458; }
  }
  l459:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l458;  yyDo(G, yy_1_Code, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Code", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l458:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Code", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineNote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "InlineNote"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l614;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l614;  if (!yy_LocMarker(G)) { goto l614; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "^[")) goto l614;
  {  int yypos617= G->pos, yythunkpos617= G->thunkpos;  if (!yymatchChar(G, ']')) goto l617;  goto l614;
  l617:;	  G->pos= yypos617; G->thunkpos= yythunkpos617;
  }  if (!yy_Inline(G)) { goto l614; }
  l615:;	
  {  int yypos616= G->pos, yythunkpos616= G->thunkpos;
  {  int yypos618= G->pos, yythunkpos618= G->thunkpos;  if (!yymatchChar(G, ']')) goto l618;  goto l616;
  l618:;	  G->pos= yypos618; G->thunkpos= yythunkpos618;
  }  if (!yy_Inline(G)) { goto l616; }  goto l615;
  l616:;	  G->pos= yypos616; G->thunkpos= yythunkpos616;
  }  if (!yymatchChar(G, ']')) goto l614;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l614;  yyDo(G, yy_1_InlineNote, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineNote", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l614:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineNote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NoteReference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NoteReference"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l619;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l619;  if (!yy_RawNoteReference(G)) { goto l619; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l619;  yyDo(G, yy_1_NoteReference, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "NoteReference", G->buf+G->pos));
  return 1;
  l619:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NoteReference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Link(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Link"));
  {  int yypos621= G->pos, yythunkpos621= G->thunkpos;  if (!yy_ExplicitLink(G)) { goto l622; }  goto l621;
  l622:;	  G->pos= yypos621; G->thunkpos= yythunkpos621;  if (!yy_ReferenceLink(G)) { goto l623; }  goto l621;
  l623:;	  G->pos= yypos621; G->thunkpos= yythunkpos621;  if (!yy_AutoLink(G)) { goto l620; }
  }
  l621:;	  yyDo(G, yy_1_Link, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Link", G->buf+G->pos));
  return 1;
  l620:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Link", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Image(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Image"));  if (!yymatchChar(G, '!')) goto l624;
  {  int yypos625= G->pos, yythunkpos625= G->thunkpos;  if (!yy_ExplicitLink(G)) { goto l626; }  goto l625;
  l626:;	  G->pos= yypos625; G->thunkpos= yythunkpos625;  if (!yy_ExplicitLinkSize(G)) { goto l627; }  goto l625;
  l627:;	  G->pos= yypos625; G->thunkpos= yythunkpos625;  if (!yy_ReferenceLink(G)) { goto l624; }
  }
  l625:;	  yyDo(G, yy_1_Image, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Image", G->buf+G->pos));
  return 1;
  l624:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Image", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Strike(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Strike"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_STRIKE) )) goto l628;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l628;  if (!yy_LocMarker(G)) { goto l628; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "~~")) goto l628;
  {  int yypos629= G->pos, yythunkpos629= G->thunkpos;  if (!yy_Whitespace(G)) { goto l629; }  goto l628;
  l629:;	  G->pos= yypos629; G->thunkpos= yythunkpos629;
  }
  {  int yypos632= G->pos, yythunkpos632= G->thunkpos;  if (!yymatchString(G, "~~")) goto l632;  goto l628;
  l632:;	  G->pos= yypos632; G->thunkpos= yythunkpos632;
  }  if (!yy_Inline(G)) { goto l628; }
  l630:;	
  {  int yypos631= G->pos, yythunkpos631= G->thunkpos;
  {  int yypos633= G->pos, yythunkpos633= G->thunkpos;  if (!yymatchString(G, "~~")) goto l633;  goto l631;
  l633:;	  G->pos= yypos633; G->thunkpos= yythunkpos633;
  }  if (!yy_Inline(G)) { goto l631; }  goto l630;
  l631:;	  G->pos= yypos631; G->thunkpos= yythunkpos631;
  }  yyText(G, G->begin, G->end);  if (!( STRIKE_POST )) goto l628;  if (!yymatchString(G, "~~")) goto l628;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l628;  yyDo(G, yy_1_Strike, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Strike", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l628:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Strike", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Emph(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Emph"));
  {  int yypos635= G->pos, yythunkpos635= G->thunkpos;  if (!yy_EmphStar(G)) { goto l636; }  goto l635;
  l636:;	  G->pos= yypos635; G->thunkpos= yythunkpos635;  if (!yy_EmphUl(G)) { goto l634; }
  }
  l635:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Emph", G->buf+G->pos));
  return 1;
  l634:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Emph", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Strong(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Strong"));
  {  int yypos638= G->pos, yythunkpos638= G->thunkpos;  if (!yy_StrongStar(G)) { goto l639; }  goto l638;
  l639:;	  G->pos= yypos638; G->thunkpos= yythunkpos638;  if (!yy_StrongUl(G)) { goto l637; }
  }
  l638:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Strong", G->buf+G->pos));
  return 1;
  l637:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Strong", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_UlOrStarLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "UlOrStarLine"));
  {  int yypos641= G->pos, yythunkpos641= G->thunkpos;  if (!yy_UlLine(G)) { goto l642; }  goto l641;
  l642:;	  G->pos= yypos641; G->thunkpos= yythunkpos641;  if (!yy_StarLine(G)) { goto l640; }
  }
  l641:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "UlOrStarLine", G->buf+G->pos));
  return 1;
  l640:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "UlOrStarLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Str(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Str"));  if (!yy_NormalChar(G)) { goto l643; }
  l644:;	
  {  int yypos645= G->pos, yythunkpos645= G->thunkpos;
  {  int yypos646= G->pos, yythunkpos646= G->thunkpos;  if (!yy_NormalChar(G)) { goto l647; }  goto l646;
  l647:;	  G->pos= yypos646; G->thunkpos= yythunkpos646;  if (!yymatchChar(G, '_')) goto l645;
  l648:;	
  {  int yypos649= G->pos, yythunkpos649= G->thunkpos;  if (!yymatchChar(G, '_')) goto l649;  goto l648;
  l649:;	  G->pos= yypos649; G->thunkpos= yythunkpos649;
  }
  {  int yypos650= G->pos, yythunkpos650= G->thunkpos;  if (!yy_Alphanumeric(G)) { goto l645; }  G->pos= yypos650; G->thunkpos= yythunkpos650;
  }
  }
  l646:;	  goto l644;
  l645:;	  G->pos= yypos645; G->thunkpos= yythunkpos645;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Str", G->buf+G->pos));
  return 1;
  l643:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Str", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InStyleTags(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InStyleTags"));  if (!yy_StyleOpen(G)) { goto l651; }
  l652:;	
  {  int yypos653= G->pos, yythunkpos653= G->thunkpos;
  {  int yypos654= G->pos, yythunkpos654= G->thunkpos;  if (!yy_StyleClose(G)) { goto l654; }  goto l653;
  l654:;	  G->pos= yypos654; G->thunkpos= yythunkpos654;
  }  if (!yymatchDot(G)) goto l653;  goto l652;
  l653:;	  G->pos= yypos653; G->thunkpos= yythunkpos653;
  }  if (!yy_StyleClose(G)) { goto l651; }
  yyprintf((stderr, "  ok   %s @ %s\n", "InStyleTags", G->buf+G->pos));
  return 1;
  l651:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InStyleTags", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StyleClose"));  if (!yymatchChar(G, '<')) goto l655;  if (!yy_Spnl(G)) { goto l655; }  if (!yymatchChar(G, '/')) goto l655;
  {  int yypos656= G->pos, yythunkpos656= G->thunkpos;  if (!yymatchString(G, "style")) goto l657;  goto l656;
  l657:;	  G->pos= yypos656; G->thunkpos= yythunkpos656;  if (!yymatchString(G, "STYLE")) goto l655;
  }
  l656:;	  if (!yy_Spnl(G)) { goto l655; }  if (!yymatchChar(G, '>')) goto l655;
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleClose", G->buf+G->pos));
  return 1;
  l655:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StyleOpen"));  if (!yymatchChar(G, '<')) goto l658;  if (!yy_Spnl(G)) { goto l658; }
  {  int yypos659= G->pos, yythunkpos659= G->thunkpos;  if (!yymatchString(G, "style")) goto l660;  goto l659;
  l660:;	  G->pos= yypos659; G->thunkpos= yythunkpos659;  if (!yymatchString(G, "STYLE")) goto l658;
  }
  l659:;	  if (!yy_Spnl(G)) { goto l658; }
  l661:;	
  {  int yypos662= G->pos, yythunkpos662= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l662; }  goto l661;
  l662:;	  G->pos= yypos662; G->thunkpos= yythunkpos662;
  }  if (!yymatchChar(G, '>')) goto l658;
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleOpen", G->buf+G->pos));
  return 1;
  l658:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockType(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockType"));
  {  int yypos664= G->pos, yythunkpos664= G->thunkpos;  if (!yymatchString(G, "address")) goto l665;  goto l664;
  l665:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "blockquote")) goto l666;  goto l664;
  l666:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "center")) goto l667;  goto l664;
  l667:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "dir")) goto l668;  goto l664;
  l668:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "div")) goto l669;  goto l664;
  l669:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "dl")) goto l670;  goto l664;
  l670:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "fieldset")) goto l671;  goto l664;
  l671:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "form")) goto l672;  goto l664;
  l672:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "h1")) goto l673;  goto l664;
  l673:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "h2")) goto l674;  goto l664;
  l674:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "h3")) goto l675;  goto l664;
  l675:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "h4")) goto l676;  goto l664;
  l676:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "h5")) goto l677;  goto l664;
  l677:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "h6")) goto l678;  goto l664;
  l678:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "hr")) goto l679;  goto l664;
  l679:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "isindex")) goto l680;  goto l664;
  l680:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "menu")) goto l681;  goto l664;
  l681:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "noframes")) goto l682;  goto l664;
  l682:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "noscript")) goto l683;  goto l664;
  l683:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "ol")) goto l684;  goto l664;
  l684:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchChar(G, 'p')) goto l685;  goto l664;
  l685:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "pre")) goto l686;  goto l664;
  l686:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "table")) goto l687;  goto l664;
  l687:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "ul")) goto l688;  goto l664;
  l688:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "dd")) goto l689;  goto l664;
  l689:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "dt")) goto l690;  goto l664;
  l690:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "frameset")) goto l691;  goto l664;
  l691:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "li")) goto l692;  goto l664;
  l692:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "tbody")) goto l693;  goto l664;
  l693:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "td")) goto l694;  goto l664;
  l694:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "tfoot")) goto l695;  goto l664;
  l695:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "th")) goto l696;  goto l664;
  l696:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "thead")) goto l697;  goto l664;
  l697:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "tr")) goto l698;  goto l664;
  l698:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "script")) goto l699;  goto l664;
  l699:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "ADDRESS")) goto l700;  goto l664;
  l700:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "BLOCKQUOTE")) goto l701;  goto l664;
  l701:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "CENTER")) goto l702;  goto l664;
  l702:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "DIR")) goto l703;  goto l664;
  l703:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "DIV")) goto l704;  goto l664;
  l704:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "DL")) goto l705;  goto l664;
  l705:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "FIELDSET")) goto l706;  goto l664;
  l706:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "FORM")) goto l707;  goto l664;
  l707:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "H1")) goto l708;  goto l664;
  l708:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "H2")) goto l709;  goto l664;
  l709:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "H3")) goto l710;  goto l664;
  l710:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "H4")) goto l711;  goto l664;
  l711:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "H5")) goto l712;  goto l664;
  l712:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "H6")) goto l713;  goto l664;
  l713:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "HR")) goto l714;  goto l664;
  l714:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "ISINDEX")) goto l715;  goto l664;
  l715:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "MENU")) goto l716;  goto l664;
  l716:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "NOFRAMES")) goto l717;  goto l664;
  l717:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "NOSCRIPT")) goto l718;  goto l664;
  l718:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "OL")) goto l719;  goto l664;
  l719:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchChar(G, 'P')) goto l720;  goto l664;
  l720:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "PRE")) goto l721;  goto l664;
  l721:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "TABLE")) goto l722;  goto l664;
  l722:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "UL")) goto l723;  goto l664;
  l723:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "DD")) goto l724;  goto l664;
  l724:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "DT")) goto l725;  goto l664;
  l725:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "FRAMESET")) goto l726;  goto l664;
  l726:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "LI")) goto l727;  goto l664;
  l727:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "TBODY")) goto l728;  goto l664;
  l728:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "TD")) goto l729;  goto l664;
  l729:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "TFOOT")) goto l730;  goto l664;
  l730:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "TH")) goto l731;  goto l664;
  l731:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "THEAD")) goto l732;  goto l664;
  l732:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "TR")) goto l733;  goto l664;
  l733:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "SCRIPT")) goto l663;
  }
  l664:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockType", G->buf+G->pos));
  return 1;
  l663:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockType", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockSelfClosing(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockSelfClosing"));  if (!yymatchChar(G, '<')) goto l734;  if (!yy_Spnl(G)) { goto l734; }  if (!yy_HtmlBlockType(G)) { goto l734; }  if (!yy_Spnl(G)) { goto l734; }
  l735:;	
  {  int yypos736= G->pos, yythunkpos736= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l736; }  goto l735;
  l736:;	  G->pos= yypos736; G->thunkpos= yythunkpos736;
  }  if (!yymatchChar(G, '/')) goto l734;  if (!yy_Spnl(G)) { goto l734; }  if (!yymatchChar(G, '>')) goto l734;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockSelfClosing", G->buf+G->pos));
  return 1;
  l734:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockSelfClosing", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlComment(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlComment"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l737;  if (!yy_LocMarker(G)) { goto l737; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "<!--")) goto l737;
  l738:;	
  {  int yypos739= G->pos, yythunkpos739= G->thunkpos;
  {  int yypos740= G->pos, yythunkpos740= G->thunkpos;  if (!yymatchString(G, "-->")) goto l740;  goto l739;
  l740:;	  G->pos= yypos740; G->thunkpos= yythunkpos740;
  }  if (!yymatchDot(G)) goto l739;  goto l738;
  l739:;	  G->pos= yypos739; G->thunkpos= yythunkpos739;
  }  if (!yymatchString(G, "-->")) goto l737;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l737;  yyDo(G, yy_1_HtmlComment, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlComment", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l737:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlComment", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockInTags(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockInTags"));
  {  int yypos742= G->pos, yythunkpos742= G->thunkpos;  if (!yy_HtmlBlockAddress(G)) { goto l743; }  goto l742;
  l743:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockBlockquote(G)) { goto l744; }  goto l742;
  l744:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockCenter(G)) { goto l745; }  goto l742;
  l745:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockDir(G)) { goto l746; }  goto l742;
  l746:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockDiv(G)) { goto l747; }  goto l742;
  l747:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockDl(G)) { goto l748; }  goto l742;
  l748:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockFieldset(G)) { goto l749; }  goto l742;
  l749:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockForm(G)) { goto l750; }  goto l742;
  l750:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockH1(G)) { goto l751; }  goto l742;
  l751:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockH2(G)) { goto l752; }  goto l742;
  l752:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockH3(G)) { goto l753; }  goto l742;
  l753:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockH4(G)) { goto l754; }  goto l742;
  l754:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockH5(G)) { goto l755; }  goto l742;
  l755:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockH6(G)) { goto l756; }  goto l742;
  l756:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockMenu(G)) { goto l757; }  goto l742;
  l757:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockNoframes(G)) { goto l758; }  goto l742;
  l758:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockNoscript(G)) { goto l759; }  goto l742;
  l759:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockOl(G)) { goto l760; }  goto l742;
  l760:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockP(G)) { goto l761; }  goto l742;
  l761:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockPre(G)) { goto l762; }  goto l742;
  l762:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockTable(G)) { goto l763; }  goto l742;
  l763:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockUl(G)) { goto l764; }  goto l742;
  l764:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockDd(G)) { goto l765; }  goto l742;
  l765:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockDt(G)) { goto l766; }  goto l742;
  l766:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockFrameset(G)) { goto l767; }  goto l742;
  l767:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockLi(G)) { goto l768; }  goto l742;
  l768:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockTbody(G)) { goto l769; }  goto l742;
  l769:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockTd(G)) { goto l770; }  goto l742;
  l770:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockTfoot(G)) { goto l771; }  goto l742;
  l771:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockTh(G)) { goto l772; }  goto l742;
  l772:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockThead(G)) { goto l773; }  goto l742;
  l773:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockTr(G)) { goto l774; }  goto l742;
  l774:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockScript(G)) { goto l775; }  goto l742;
  l775:;	  G->pos= yypos742; G->thunkpos= yythunkpos742;  if (!yy_HtmlBlockHead(G)) { goto l741; }
  }
  l742:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockInTags", G->buf+G->pos));
  return 1;
  l741:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockInTags", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockHead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockHead"));  if (!yy_HtmlBlockOpenHead(G)) { goto l776; }
  l777:;	
  {  int yypos778= G->pos, yythunkpos778= G->thunkpos;
  {  int yypos779= G->pos, yythunkpos779= G->thunkpos;  if (!yy_HtmlBlockCloseHead(G)) { goto l779; }  goto l778;
  l779:;	  G->pos= yypos779; G->thunkpos= yythunkpos779;
  }  if (!yymatchDot(G)) goto l778;  goto l777;
  l778:;	  G->pos= yypos778; G->thunkpos= yythunkpos778;
  }  if (!yy_HtmlBlockCloseHead(G)) { goto l776; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockHead", G->buf+G->pos));
  return 1;
  l776:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockHead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseHead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseHead"));  if (!yymatchChar(G, '<')) goto l780;  if (!yy_Spnl(G)) { goto l780; }  if (!yymatchChar(G, '/')) goto l780;
  {  int yypos781= G->pos, yythunkpos781= G->thunkpos;  if (!yymatchString(G, "head")) goto l782;  goto l781;
  l782:;	  G->pos= yypos781; G->thunkpos= yythunkpos781;  if (!yymatchString(G, "HEAD")) goto l780;
  }
  l781:;	  if (!yy_Spnl(G)) { goto l780; }  if (!yymatchChar(G, '>')) goto l780;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseHead", G->buf+G->pos));
  return 1;
  l780:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseHead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenHead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenHead"));  if (!yymatchChar(G, '<')) goto l783;  if (!yy_Spnl(G)) { goto l783; }
  {  int yypos784= G->pos, yythunkpos784= G->thunkpos;  if (!yymatchString(G, "head")) goto l785;  goto l784;
  l785:;	  G->pos= yypos784; G->thunkpos= yythunkpos784;  if (!yymatchString(G, "HEAD")) goto l783;
  }
  l784:;	  if (!yy_Spnl(G)) { goto l783; }
  l786:;	
  {  int yypos787= G->pos, yythunkpos787= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l787; }  goto l786;
  l787:;	  G->pos= yypos787; G->thunkpos= yythunkpos787;
  }  if (!yymatchChar(G, '>')) goto l783;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenHead", G->buf+G->pos));
  return 1;
  l783:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenHead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockScript"));  if (!yy_HtmlBlockOpenScript(G)) { goto l788; }
  l789:;	
  {  int yypos790= G->pos, yythunkpos790= G->thunkpos;
  {  int yypos791= G->pos, yythunkpos791= G->thunkpos;  if (!yy_HtmlBlockCloseScript(G)) { goto l791; }  goto l790;
  l791:;	  G->pos= yypos791; G->thunkpos= yythunkpos791;
  }  if (!yymatchDot(G)) goto l790;  goto l789;
  l790:;	  G->pos= yypos790; G->thunkpos= yythunkpos790;
  }  if (!yy_HtmlBlockCloseScript(G)) { goto l788; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockScript", G->buf+G->pos));
  return 1;
  l788:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseScript"));  if (!yymatchChar(G, '<')) goto l792;  if (!yy_Spnl(G)) { goto l792; }  if (!yymatchChar(G, '/')) goto l792;
  {  int yypos793= G->pos, yythunkpos793= G->thunkpos;  if (!yymatchString(G, "script")) goto l794;  goto l793;
  l794:;	  G->pos= yypos793; G->thunkpos= yythunkpos793;  if (!yymatchString(G, "SCRIPT")) goto l792;
  }
  l793:;	  if (!yy_Spnl(G)) { goto l792; }  if (!yymatchChar(G, '>')) goto l792;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseScript", G->buf+G->pos));
  return 1;
  l792:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenScript"));  if (!yymatchChar(G, '<')) goto l795;  if (!yy_Spnl(G)) { goto l795; }
  {  int yypos796= G->pos, yythunkpos796= G->thunkpos;  if (!yymatchString(G, "script")) goto l797;  goto l796;
  l797:;	  G->pos= yypos796; G->thunkpos= yythunkpos796;  if (!yymatchString(G, "SCRIPT")) goto l795;
  }
  l796:;	  if (!yy_Spnl(G)) { goto l795; }
  l798:;	
  {  int yypos799= G->pos, yythunkpos799= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l799; }  goto l798;
  l799:;	  G->pos= yypos799; G->thunkpos= yythunkpos799;
  }  if (!yymatchChar(G, '>')) goto l795;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenScript", G->buf+G->pos));
  return 1;
  l795:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTr"));  if (!yy_HtmlBlockOpenTr(G)) { goto l800; }
  l801:;	
  {  int yypos802= G->pos, yythunkpos802= G->thunkpos;
  {  int yypos803= G->pos, yythunkpos803= G->thunkpos;  if (!yy_HtmlBlockTr(G)) { goto l804; }  goto l803;
  l804:;	  G->pos= yypos803; G->thunkpos= yythunkpos803;
  {  int yypos805= G->pos, yythunkpos805= G->thunkpos;  if (!yy_HtmlBlockCloseTr(G)) { goto l805; }  goto l802;
  l805:;	  G->pos= yypos805; G->thunkpos= yythunkpos805;
  }  if (!yymatchDot(G)) goto l802;
  }
  l803:;	  goto l801;
  l802:;	  G->pos= yypos802; G->thunkpos= yythunkpos802;
  }  if (!yy_HtmlBlockCloseTr(G)) { goto l800; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTr", G->buf+G->pos));
  return 1;
  l800:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTr"));  if (!yymatchChar(G, '<')) goto l806;  if (!yy_Spnl(G)) { goto l806; }  if (!yymatchChar(G, '/')) goto l806;
  {  int yypos807= G->pos, yythunkpos807= G->thunkpos;  if (!yymatchString(G, "tr")) goto l808;  goto l807;
  l808:;	  G->pos= yypos807; G->thunkpos= yythunkpos807;  if (!yymatchString(G, "TR")) goto l806;
  }
  l807:;	  if (!yy_Spnl(G)) { goto l806; }  if (!yymatchChar(G, '>')) goto l806;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTr", G->buf+G->pos));
  return 1;
  l806:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTr"));  if (!yymatchChar(G, '<')) goto l809;  if (!yy_Spnl(G)) { goto l809; }
  {  int yypos810= G->pos, yythunkpos810= G->thunkpos;  if (!yymatchString(G, "tr")) goto l811;  goto l810;
  l811:;	  G->pos= yypos810; G->thunkpos= yythunkpos810;  if (!yymatchString(G, "TR")) goto l809;
  }
  l810:;	  if (!yy_Spnl(G)) { goto l809; }
  l812:;	
  {  int yypos813= G->pos, yythunkpos813= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l813; }  goto l812;
  l813:;	  G->pos= yypos813; G->thunkpos= yythunkpos813;
  }  if (!yymatchChar(G, '>')) goto l809;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTr", G->buf+G->pos));
  return 1;
  l809:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockThead"));  if (!yy_HtmlBlockOpenThead(G)) { goto l814; }
  l815:;	
  {  int yypos816= G->pos, yythunkpos816= G->thunkpos;
  {  int yypos817= G->pos, yythunkpos817= G->thunkpos;  if (!yy_HtmlBlockThead(G)) { goto l818; }  goto l817;
  l818:;	  G->pos= yypos817; G->thunkpos= yythunkpos817;
  {  int yypos819= G->pos, yythunkpos819= G->thunkpos;  if (!yy_HtmlBlockCloseThead(G)) { goto l819; }  goto l816;
  l819:;	  G->pos= yypos819; G->thunkpos= yythunkpos819;
  }  if (!yymatchDot(G)) goto l816;
  }
  l817:;	  goto l815;
  l816:;	  G->pos= yypos816; G->thunkpos= yythunkpos816;
  }  if (!yy_HtmlBlockCloseThead(G)) { goto l814; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockThead", G->buf+G->pos));
  return 1;
  l814:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseThead"));  if (!yymatchChar(G, '<')) goto l820;  if (!yy_Spnl(G)) { goto l820; }  if (!yymatchChar(G, '/')) goto l820;
  {  int yypos821= G->pos, yythunkpos821= G->thunkpos;  if (!yymatchString(G, "thead")) goto l822;  goto l821;
  l822:;	  G->pos= yypos821; G->thunkpos= yythunkpos821;  if (!yymatchString(G, "THEAD")) goto l820;
  }
  l821:;	  if (!yy_Spnl(G)) { goto l820; }  if (!yymatchChar(G, '>')) goto l820;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseThead", G->buf+G->pos));
  return 1;
  l820:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenThead"));  if (!yymatchChar(G, '<')) goto l823;  if (!yy_Spnl(G)) { goto l823; }
  {  int yypos824= G->pos, yythunkpos824= G->thunkpos;  if (!yymatchString(G, "thead")) goto l825;  goto l824;
  l825:;	  G->pos= yypos824; G->thunkpos= yythunkpos824;  if (!yymatchString(G, "THEAD")) goto l823;
  }
  l824:;	  if (!yy_Spnl(G)) { goto l823; }
  l826:;	
  {  int yypos827= G->pos, yythunkpos827= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l827; }  goto l826;
  l827:;	  G->pos= yypos827; G->thunkpos= yythunkpos827;
  }  if (!yymatchChar(G, '>')) goto l823;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenThead", G->buf+G->pos));
  return 1;
  l823:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTh"));  if (!yy_HtmlBlockOpenTh(G)) { goto l828; }
  l829:;	
  {  int yypos830= G->pos, yythunkpos830= G->thunkpos;
  {  int yypos831= G->pos, yythunkpos831= G->thunkpos;  if (!yy_HtmlBlockTh(G)) { goto l832; }  goto l831;
  l832:;	  G->pos= yypos831; G->thunkpos= yythunkpos831;
  {  int yypos833= G->pos, yythunkpos833= G->thunkpos;  if (!yy_HtmlBlockCloseTh(G)) { goto l833; }  goto l830;
  l833:;	  G->pos= yypos833; G->thunkpos= yythunkpos833;
  }  if (!yymatchDot(G)) goto l830;
  }
  l831:;	  goto l829;
  l830:;	  G->pos= yypos830; G->thunkpos= yythunkpos830;
  }  if (!yy_HtmlBlockCloseTh(G)) { goto l828; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTh", G->buf+G->pos));
  return 1;
  l828:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTh"));  if (!yymatchChar(G, '<')) goto l834;  if (!yy_Spnl(G)) { goto l834; }  if (!yymatchChar(G, '/')) goto l834;
  {  int yypos835= G->pos, yythunkpos835= G->thunkpos;  if (!yymatchString(G, "th")) goto l836;  goto l835;
  l836:;	  G->pos= yypos835; G->thunkpos= yythunkpos835;  if (!yymatchString(G, "TH")) goto l834;
  }
  l835:;	  if (!yy_Spnl(G)) { goto l834; }  if (!yymatchChar(G, '>')) goto l834;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTh", G->buf+G->pos));
  return 1;
  l834:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTh"));  if (!yymatchChar(G, '<')) goto l837;  if (!yy_Spnl(G)) { goto l837; }
  {  int yypos838= G->pos, yythunkpos838= G->thunkpos;  if (!yymatchString(G, "th")) goto l839;  goto l838;
  l839:;	  G->pos= yypos838; G->thunkpos= yythunkpos838;  if (!yymatchString(G, "TH")) goto l837;
  }
  l838:;	  if (!yy_Spnl(G)) { goto l837; }
  l840:;	
  {  int yypos841= G->pos, yythunkpos841= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l841; }  goto l840;
  l841:;	  G->pos= yypos841; G->thunkpos= yythunkpos841;
  }  if (!yymatchChar(G, '>')) goto l837;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTh", G->buf+G->pos));
  return 1;
  l837:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTfoot"));  if (!yy_HtmlBlockOpenTfoot(G)) { goto l842; }
  l843:;	
  {  int yypos844= G->pos, yythunkpos844= G->thunkpos;
  {  int yypos845= G->pos, yythunkpos845= G->thunkpos;  if (!yy_HtmlBlockTfoot(G)) { goto l846; }  goto l845;
  l846:;	  G->pos= yypos845; G->thunkpos= yythunkpos845;
  {  int yypos847= G->pos, yythunkpos847= G->thunkpos;  if (!yy_HtmlBlockCloseTfoot(G)) { goto l847; }  goto l844;
  l847:;	  G->pos= yypos847; G->thunkpos= yythunkpos847;
  }  if (!yymatchDot(G)) goto l844;
  }
  l845:;	  goto l843;
  l844:;	  G->pos= yypos844; G->thunkpos= yythunkpos844;
  }  if (!yy_HtmlBlockCloseTfoot(G)) { goto l842; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTfoot", G->buf+G->pos));
  return 1;
  l842:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTfoot"));  if (!yymatchChar(G, '<')) goto l848;  if (!yy_Spnl(G)) { goto l848; }  if (!yymatchChar(G, '/')) goto l848;
  {  int yypos849= G->pos, yythunkpos849= G->thunkpos;  if (!yymatchString(G, "tfoot")) goto l850;  goto l849;
  l850:;	  G->pos= yypos849; G->thunkpos= yythunkpos849;  if (!yymatchString(G, "TFOOT")) goto l848;
  }
  l849:;	  if (!yy_Spnl(G)) { goto l848; }  if (!yymatchChar(G, '>')) goto l848;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTfoot", G->buf+G->pos));
  return 1;
  l848:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTfoot"));  if (!yymatchChar(G, '<')) goto l851;  if (!yy_Spnl(G)) { goto l851; }
  {  int yypos852= G->pos, yythunkpos852= G->thunkpos;  if (!yymatchString(G, "tfoot")) goto l853;  goto l852;
  l853:;	  G->pos= yypos852; G->thunkpos= yythunkpos852;  if (!yymatchString(G, "TFOOT")) goto l851;
  }
  l852:;	  if (!yy_Spnl(G)) { goto l851; }
  l854:;	
  {  int yypos855= G->pos, yythunkpos855= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l855; }  goto l854;
  l855:;	  G->pos= yypos855; G->thunkpos= yythunkpos855;
  }  if (!yymatchChar(G, '>')) goto l851;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTfoot", G->buf+G->pos));
  return 1;
  l851:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTd"));  if (!yy_HtmlBlockOpenTd(G)) { goto l856; }
  l857:;	
  {  int yypos858= G->pos, yythunkpos858= G->thunkpos;
  {  int yypos859= G->pos, yythunkpos859= G->thunkpos;  if (!yy_HtmlBlockTd(G)) { goto l860; }  goto l859;
  l860:;	  G->pos= yypos859; G->thunkpos= yythunkpos859;
  {  int yypos861= G->pos, yythunkpos861= G->thunkpos;  if (!yy_HtmlBlockCloseTd(G)) { goto l861; }  goto l858;
  l861:;	  G->pos= yypos861; G->thunkpos= yythunkpos861;
  }  if (!yymatchDot(G)) goto l858;
  }
  l859:;	  goto l857;
  l858:;	  G->pos= yypos858; G->thunkpos= yythunkpos858;
  }  if (!yy_HtmlBlockCloseTd(G)) { goto l856; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTd", G->buf+G->pos));
  return 1;
  l856:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTd"));  if (!yymatchChar(G, '<')) goto l862;  if (!yy_Spnl(G)) { goto l862; }  if (!yymatchChar(G, '/')) goto l862;
  {  int yypos863= G->pos, yythunkpos863= G->thunkpos;  if (!yymatchString(G, "td")) goto l864;  goto l863;
  l864:;	  G->pos= yypos863; G->thunkpos= yythunkpos863;  if (!yymatchString(G, "TD")) goto l862;
  }
  l863:;	  if (!yy_Spnl(G)) { goto l862; }  if (!yymatchChar(G, '>')) goto l862;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTd", G->buf+G->pos));
  return 1;
  l862:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTd"));  if (!yymatchChar(G, '<')) goto l865;  if (!yy_Spnl(G)) { goto l865; }
  {  int yypos866= G->pos, yythunkpos866= G->thunkpos;  if (!yymatchString(G, "td")) goto l867;  goto l866;
  l867:;	  G->pos= yypos866; G->thunkpos= yythunkpos866;  if (!yymatchString(G, "TD")) goto l865;
  }
  l866:;	  if (!yy_Spnl(G)) { goto l865; }
  l868:;	
  {  int yypos869= G->pos, yythunkpos869= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l869; }  goto l868;
  l869:;	  G->pos= yypos869; G->thunkpos= yythunkpos869;
  }  if (!yymatchChar(G, '>')) goto l865;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTd", G->buf+G->pos));
  return 1;
  l865:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTbody"));  if (!yy_HtmlBlockOpenTbody(G)) { goto l870; }
  l871:;	
  {  int yypos872= G->pos, yythunkpos872= G->thunkpos;
  {  int yypos873= G->pos, yythunkpos873= G->thunkpos;  if (!yy_HtmlBlockTbody(G)) { goto l874; }  goto l873;
  l874:;	  G->pos= yypos873; G->thunkpos= yythunkpos873;
  {  int yypos875= G->pos, yythunkpos875= G->thunkpos;  if (!yy_HtmlBlockCloseTbody(G)) { goto l875; }  goto l872;
  l875:;	  G->pos= yypos875; G->thunkpos= yythunkpos875;
  }  if (!yymatchDot(G)) goto l872;
  }
  l873:;	  goto l871;
  l872:;	  G->pos= yypos872; G->thunkpos= yythunkpos872;
  }  if (!yy_HtmlBlockCloseTbody(G)) { goto l870; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTbody", G->buf+G->pos));
  return 1;
  l870:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTbody"));  if (!yymatchChar(G, '<')) goto l876;  if (!yy_Spnl(G)) { goto l876; }  if (!yymatchChar(G, '/')) goto l876;
  {  int yypos877= G->pos, yythunkpos877= G->thunkpos;  if (!yymatchString(G, "tbody")) goto l878;  goto l877;
  l878:;	  G->pos= yypos877; G->thunkpos= yythunkpos877;  if (!yymatchString(G, "TBODY")) goto l876;
  }
  l877:;	  if (!yy_Spnl(G)) { goto l876; }  if (!yymatchChar(G, '>')) goto l876;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTbody", G->buf+G->pos));
  return 1;
  l876:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTbody"));  if (!yymatchChar(G, '<')) goto l879;  if (!yy_Spnl(G)) { goto l879; }
  {  int yypos880= G->pos, yythunkpos880= G->thunkpos;  if (!yymatchString(G, "tbody")) goto l881;  goto l880;
  l881:;	  G->pos= yypos880; G->thunkpos= yythunkpos880;  if (!yymatchString(G, "TBODY")) goto l879;
  }
  l880:;	  if (!yy_Spnl(G)) { goto l879; }
  l882:;	
  {  int yypos883= G->pos, yythunkpos883= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l883; }  goto l882;
  l883:;	  G->pos= yypos883; G->thunkpos= yythunkpos883;
  }  if (!yymatchChar(G, '>')) goto l879;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTbody", G->buf+G->pos));
  return 1;
  l879:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockLi"));  if (!yy_HtmlBlockOpenLi(G)) { goto l884; }
  l885:;	
  {  int yypos886= G->pos, yythunkpos886= G->thunkpos;
  {  int yypos887= G->pos, yythunkpos887= G->thunkpos;  if (!yy_HtmlBlockLi(G)) { goto l888; }  goto l887;
  l888:;	  G->pos= yypos887; G->thunkpos= yythunkpos887;
  {  int yypos889= G->pos, yythunkpos889= G->thunkpos;  if (!yy_HtmlBlockCloseLi(G)) { goto l889; }  goto l886;
  l889:;	  G->pos= yypos889; G->thunkpos= yythunkpos889;
  }  if (!yymatchDot(G)) goto l886;
  }
  l887:;	  goto l885;
  l886:;	  G->pos= yypos886; G->thunkpos= yythunkpos886;
  }  if (!yy_HtmlBlockCloseLi(G)) { goto l884; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockLi", G->buf+G->pos));
  return 1;
  l884:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseLi"));  if (!yymatchChar(G, '<')) goto l890;  if (!yy_Spnl(G)) { goto l890; }  if (!yymatchChar(G, '/')) goto l890;
  {  int yypos891= G->pos, yythunkpos891= G->thunkpos;  if (!yymatchString(G, "li")) goto l892;  goto l891;
  l892:;	  G->pos= yypos891; G->thunkpos= yythunkpos891;  if (!yymatchString(G, "LI")) goto l890;
  }
  l891:;	  if (!yy_Spnl(G)) { goto l890; }  if (!yymatchChar(G, '>')) goto l890;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseLi", G->buf+G->pos));
  return 1;
  l890:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenLi"));  if (!yymatchChar(G, '<')) goto l893;  if (!yy_Spnl(G)) { goto l893; }
  {  int yypos894= G->pos, yythunkpos894= G->thunkpos;  if (!yymatchString(G, "li")) goto l895;  goto l894;
  l895:;	  G->pos= yypos894; G->thunkpos= yythunkpos894;  if (!yymatchString(G, "LI")) goto l893;
  }
  l894:;	  if (!yy_Spnl(G)) { goto l893; }
  l896:;	
  {  int yypos897= G->pos, yythunkpos897= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l897; }  goto l896;
  l897:;	  G->pos= yypos897; G->thunkpos= yythunkpos897;
  }  if (!yymatchChar(G, '>')) goto l893;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenLi", G->buf+G->pos));
  return 1;
  l893:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockFrameset"));  if (!yy_HtmlBlockOpenFrameset(G)) { goto l898; }
  l899:;	
  {  int yypos900= G->pos, yythunkpos900= G->thunkpos;
  {  int yypos901= G->pos, yythunkpos901= G->thunkpos;  if (!yy_HtmlBlockFrameset(G)) { goto l902; }  goto l901;
  l902:;	  G->pos= yypos901; G->thunkpos= yythunkpos901;
  {  int yypos903= G->pos, yythunkpos903= G->thunkpos;  if (!yy_HtmlBlockCloseFrameset(G)) { goto l903; }  goto l900;
  l903:;	  G->pos= yypos903; G->thunkpos= yythunkpos903;
  }  if (!yymatchDot(G)) goto l900;
  }
  l901:;	  goto l899;
  l900:;	  G->pos= yypos900; G->thunkpos= yythunkpos900;
  }  if (!yy_HtmlBlockCloseFrameset(G)) { goto l898; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockFrameset", G->buf+G->pos));
  return 1;
  l898:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseFrameset"));  if (!yymatchChar(G, '<')) goto l904;  if (!yy_Spnl(G)) { goto l904; }  if (!yymatchChar(G, '/')) goto l904;
  {  int yypos905= G->pos, yythunkpos905= G->thunkpos;  if (!yymatchString(G, "frameset")) goto l906;  goto l905;
  l906:;	  G->pos= yypos905; G->thunkpos= yythunkpos905;  if (!yymatchString(G, "FRAMESET")) goto l904;
  }
  l905:;	  if (!yy_Spnl(G)) { goto l904; }  if (!yymatchChar(G, '>')) goto l904;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseFrameset", G->buf+G->pos));
  return 1;
  l904:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenFrameset"));  if (!yymatchChar(G, '<')) goto l907;  if (!yy_Spnl(G)) { goto l907; }
  {  int yypos908= G->pos, yythunkpos908= G->thunkpos;  if (!yymatchString(G, "frameset")) goto l909;  goto l908;
  l909:;	  G->pos= yypos908; G->thunkpos= yythunkpos908;  if (!yymatchString(G, "FRAMESET")) goto l907;
  }
  l908:;	  if (!yy_Spnl(G)) { goto l907; }
  l910:;	
  {  int yypos911= G->pos, yythunkpos911= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l911; }  goto l910;
  l911:;	  G->pos= yypos911; G->thunkpos= yythunkpos911;
  }  if (!yymatchChar(G, '>')) goto l907;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenFrameset", G->buf+G->pos));
  return 1;
  l907:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDt"));  if (!yy_HtmlBlockOpenDt(G)) { goto l912; }
  l913:;	
  {  int yypos914= G->pos, yythunkpos914= G->thunkpos;
  {  int yypos915= G->pos, yythunkpos915= G->thunkpos;  if (!yy_HtmlBlockDt(G)) { goto l916; }  goto l915;
  l916:;	  G->pos= yypos915; G->thunkpos= yythunkpos915;
  {  int yypos917= G->pos, yythunkpos917= G->thunkpos;  if (!yy_HtmlBlockCloseDt(G)) { goto l917; }  goto l914;
  l917:;	  G->pos= yypos917; G->thunkpos= yythunkpos917;
  }  if (!yymatchDot(G)) goto l914;
  }
  l915:;	  goto l913;
  l914:;	  G->pos= yypos914; G->thunkpos= yythunkpos914;
  }  if (!yy_HtmlBlockCloseDt(G)) { goto l912; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDt", G->buf+G->pos));
  return 1;
  l912:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDt"));  if (!yymatchChar(G, '<')) goto l918;  if (!yy_Spnl(G)) { goto l918; }  if (!yymatchChar(G, '/')) goto l918;
  {  int yypos919= G->pos, yythunkpos919= G->thunkpos;  if (!yymatchString(G, "dt")) goto l920;  goto l919;
  l920:;	  G->pos= yypos919; G->thunkpos= yythunkpos919;  if (!yymatchString(G, "DT")) goto l918;
  }
  l919:;	  if (!yy_Spnl(G)) { goto l918; }  if (!yymatchChar(G, '>')) goto l918;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDt", G->buf+G->pos));
  return 1;
  l918:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDt"));  if (!yymatchChar(G, '<')) goto l921;  if (!yy_Spnl(G)) { goto l921; }
  {  int yypos922= G->pos, yythunkpos922= G->thunkpos;  if (!yymatchString(G, "dt")) goto l923;  goto l922;
  l923:;	  G->pos= yypos922; G->thunkpos= yythunkpos922;  if (!yymatchString(G, "DT")) goto l921;
  }
  l922:;	  if (!yy_Spnl(G)) { goto l921; }
  l924:;	
  {  int yypos925= G->pos, yythunkpos925= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l925; }  goto l924;
  l925:;	  G->pos= yypos925; G->thunkpos= yythunkpos925;
  }  if (!yymatchChar(G, '>')) goto l921;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDt", G->buf+G->pos));
  return 1;
  l921:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDd"));  if (!yy_HtmlBlockOpenDd(G)) { goto l926; }
  l927:;	
  {  int yypos928= G->pos, yythunkpos928= G->thunkpos;
  {  int yypos929= G->pos, yythunkpos929= G->thunkpos;  if (!yy_HtmlBlockDd(G)) { goto l930; }  goto l929;
  l930:;	  G->pos= yypos929; G->thunkpos= yythunkpos929;
  {  int yypos931= G->pos, yythunkpos931= G->thunkpos;  if (!yy_HtmlBlockCloseDd(G)) { goto l931; }  goto l928;
  l931:;	  G->pos= yypos931; G->thunkpos= yythunkpos931;
  }  if (!yymatchDot(G)) goto l928;
  }
  l929:;	  goto l927;
  l928:;	  G->pos= yypos928; G->thunkpos= yythunkpos928;
  }  if (!yy_HtmlBlockCloseDd(G)) { goto l926; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDd", G->buf+G->pos));
  return 1;
  l926:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDd"));  if (!yymatchChar(G, '<')) goto l932;  if (!yy_Spnl(G)) { goto l932; }  if (!yymatchChar(G, '/')) goto l932;
  {  int yypos933= G->pos, yythunkpos933= G->thunkpos;  if (!yymatchString(G, "dd")) goto l934;  goto l933;
  l934:;	  G->pos= yypos933; G->thunkpos= yythunkpos933;  if (!yymatchString(G, "DD")) goto l932;
  }
  l933:;	  if (!yy_Spnl(G)) { goto l932; }  if (!yymatchChar(G, '>')) goto l932;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDd", G->buf+G->pos));
  return 1;
  l932:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDd"));  if (!yymatchChar(G, '<')) goto l935;  if (!yy_Spnl(G)) { goto l935; }
  {  int yypos936= G->pos, yythunkpos936= G->thunkpos;  if (!yymatchString(G, "dd")) goto l937;  goto l936;
  l937:;	  G->pos= yypos936; G->thunkpos= yythunkpos936;  if (!yymatchString(G, "DD")) goto l935;
  }
  l936:;	  if (!yy_Spnl(G)) { goto l935; }
  l938:;	
  {  int yypos939= G->pos, yythunkpos939= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l939; }  goto l938;
  l939:;	  G->pos= yypos939; G->thunkpos= yythunkpos939;
  }  if (!yymatchChar(G, '>')) goto l935;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDd", G->buf+G->pos));
  return 1;
  l935:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockUl"));  if (!yy_HtmlBlockOpenUl(G)) { goto l940; }
  l941:;	
  {  int yypos942= G->pos, yythunkpos942= G->thunkpos;
  {  int yypos943= G->pos, yythunkpos943= G->thunkpos;  if (!yy_HtmlBlockUl(G)) { goto l944; }  goto l943;
  l944:;	  G->pos= yypos943; G->thunkpos= yythunkpos943;
  {  int yypos945= G->pos, yythunkpos945= G->thunkpos;  if (!yy_HtmlBlockCloseUl(G)) { goto l945; }  goto l942;
  l945:;	  G->pos= yypos945; G->thunkpos= yythunkpos945;
  }  if (!yymatchDot(G)) goto l942;
  }
  l943:;	  goto l941;
  l942:;	  G->pos= yypos942; G->thunkpos= yythunkpos942;
  }  if (!yy_HtmlBlockCloseUl(G)) { goto l940; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockUl", G->buf+G->pos));
  return 1;
  l940:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseUl"));  if (!yymatchChar(G, '<')) goto l946;  if (!yy_Spnl(G)) { goto l946; }  if (!yymatchChar(G, '/')) goto l946;
  {  int yypos947= G->pos, yythunkpos947= G->thunkpos;  if (!yymatchString(G, "ul")) goto l948;  goto l947;
  l948:;	  G->pos= yypos947; G->thunkpos= yythunkpos947;  if (!yymatchString(G, "UL")) goto l946;
  }
  l947:;	  if (!yy_Spnl(G)) { goto l946; }  if (!yymatchChar(G, '>')) goto l946;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseUl", G->buf+G->pos));
  return 1;
  l946:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenUl"));  if (!yymatchChar(G, '<')) goto l949;  if (!yy_Spnl(G)) { goto l949; }
  {  int yypos950= G->pos, yythunkpos950= G->thunkpos;  if (!yymatchString(G, "ul")) goto l951;  goto l950;
  l951:;	  G->pos= yypos950; G->thunkpos= yythunkpos950;  if (!yymatchString(G, "UL")) goto l949;
  }
  l950:;	  if (!yy_Spnl(G)) { goto l949; }
  l952:;	
  {  int yypos953= G->pos, yythunkpos953= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l953; }  goto l952;
  l953:;	  G->pos= yypos953; G->thunkpos= yythunkpos953;
  }  if (!yymatchChar(G, '>')) goto l949;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenUl", G->buf+G->pos));
  return 1;
  l949:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTable"));  if (!yy_HtmlBlockOpenTable(G)) { goto l954; }
  l955:;	
  {  int yypos956= G->pos, yythunkpos956= G->thunkpos;
  {  int yypos957= G->pos, yythunkpos957= G->thunkpos;  if (!yy_HtmlBlockTable(G)) { goto l958; }  goto l957;
  l958:;	  G->pos= yypos957; G->thunkpos= yythunkpos957;
  {  int yypos959= G->pos, yythunkpos959= G->thunkpos;  if (!yy_HtmlBlockCloseTable(G)) { goto l959; }  goto l956;
  l959:;	  G->pos= yypos959; G->thunkpos= yythunkpos959;
  }  if (!yymatchDot(G)) goto l956;
  }
  l957:;	  goto l955;
  l956:;	  G->pos= yypos956; G->thunkpos= yythunkpos956;
  }  if (!yy_HtmlBlockCloseTable(G)) { goto l954; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTable", G->buf+G->pos));
  return 1;
  l954:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTable"));  if (!yymatchChar(G, '<')) goto l960;  if (!yy_Spnl(G)) { goto l960; }  if (!yymatchChar(G, '/')) goto l960;
  {  int yypos961= G->pos, yythunkpos961= G->thunkpos;  if (!yymatchString(G, "table")) goto l962;  goto l961;
  l962:;	  G->pos= yypos961; G->thunkpos= yythunkpos961;  if (!yymatchString(G, "TABLE")) goto l960;
  }
  l961:;	  if (!yy_Spnl(G)) { goto l960; }  if (!yymatchChar(G, '>')) goto l960;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTable", G->buf+G->pos));
  return 1;
  l960:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTable"));  if (!yymatchChar(G, '<')) goto l963;  if (!yy_Spnl(G)) { goto l963; }
  {  int yypos964= G->pos, yythunkpos964= G->thunkpos;  if (!yymatchString(G, "table")) goto l965;  goto l964;
  l965:;	  G->pos= yypos964; G->thunkpos= yythunkpos964;  if (!yymatchString(G, "TABLE")) goto l963;
  }
  l964:;	  if (!yy_Spnl(G)) { goto l963; }
  l966:;	
  {  int yypos967= G->pos, yythunkpos967= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l967; }  goto l966;
  l967:;	  G->pos= yypos967; G->thunkpos= yythunkpos967;
  }  if (!yymatchChar(G, '>')) goto l963;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTable", G->buf+G->pos));
  return 1;
  l963:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockPre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockPre"));  if (!yy_HtmlBlockOpenPre(G)) { goto l968; }
  l969:;	
  {  int yypos970= G->pos, yythunkpos970= G->thunkpos;
  {  int yypos971= G->pos, yythunkpos971= G->thunkpos;  if (!yy_HtmlBlockPre(G)) { goto l972; }  goto l971;
  l972:;	  G->pos= yypos971; G->thunkpos= yythunkpos971;
  {  int yypos973= G->pos, yythunkpos973= G->thunkpos;  if (!yy_HtmlBlockClosePre(G)) { goto l973; }  goto l970;
  l973:;	  G->pos= yypos973; G->thunkpos= yythunkpos973;
  }  if (!yymatchDot(G)) goto l970;
  }
  l971:;	  goto l969;
  l970:;	  G->pos= yypos970; G->thunkpos= yythunkpos970;
  }  if (!yy_HtmlBlockClosePre(G)) { goto l968; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockPre", G->buf+G->pos));
  return 1;
  l968:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockPre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockClosePre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockClosePre"));  if (!yymatchChar(G, '<')) goto l974;  if (!yy_Spnl(G)) { goto l974; }  if (!yymatchChar(G, '/')) goto l974;
  {  int yypos975= G->pos, yythunkpos975= G->thunkpos;  if (!yymatchString(G, "pre")) goto l976;  goto l975;
  l976:;	  G->pos= yypos975; G->thunkpos= yythunkpos975;  if (!yymatchString(G, "PRE")) goto l974;
  }
  l975:;	  if (!yy_Spnl(G)) { goto l974; }  if (!yymatchChar(G, '>')) goto l974;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockClosePre", G->buf+G->pos));
  return 1;
  l974:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockClosePre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenPre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenPre"));  if (!yymatchChar(G, '<')) goto l977;  if (!yy_Spnl(G)) { goto l977; }
  {  int yypos978= G->pos, yythunkpos978= G->thunkpos;  if (!yymatchString(G, "pre")) goto l979;  goto l978;
  l979:;	  G->pos= yypos978; G->thunkpos= yythunkpos978;  if (!yymatchString(G, "PRE")) goto l977;
  }
  l978:;	  if (!yy_Spnl(G)) { goto l977; }
  l980:;	
  {  int yypos981= G->pos, yythunkpos981= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l981; }  goto l980;
  l981:;	  G->pos= yypos981; G->thunkpos= yythunkpos981;
  }  if (!yymatchChar(G, '>')) goto l977;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenPre", G->buf+G->pos));
  return 1;
  l977:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenPre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockP"));  if (!yy_HtmlBlockOpenP(G)) { goto l982; }
  l983:;	
  {  int yypos984= G->pos, yythunkpos984= G->thunkpos;
  {  int yypos985= G->pos, yythunkpos985= G->thunkpos;  if (!yy_HtmlBlockP(G)) { goto l986; }  goto l985;
  l986:;	  G->pos= yypos985; G->thunkpos= yythunkpos985;
  {  int yypos987= G->pos, yythunkpos987= G->thunkpos;  if (!yy_HtmlBlockCloseP(G)) { goto l987; }  goto l984;
  l987:;	  G->pos= yypos987; G->thunkpos= yythunkpos987;
  }  if (!yymatchDot(G)) goto l984;
  }
  l985:;	  goto l983;
  l984:;	  G->pos= yypos984; G->thunkpos= yythunkpos984;
  }  if (!yy_HtmlBlockCloseP(G)) { goto l982; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockP", G->buf+G->pos));
  return 1;
  l982:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseP"));  if (!yymatchChar(G, '<')) goto l988;  if (!yy_Spnl(G)) { goto l988; }  if (!yymatchChar(G, '/')) goto l988;
  {  int yypos989= G->pos, yythunkpos989= G->thunkpos;  if (!yymatchChar(G, 'p')) goto l990;  goto l989;
  l990:;	  G->pos= yypos989; G->thunkpos= yythunkpos989;  if (!yymatchChar(G, 'P')) goto l988;
  }
  l989:;	  if (!yy_Spnl(G)) { goto l988; }  if (!yymatchChar(G, '>')) goto l988;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseP", G->buf+G->pos));
  return 1;
  l988:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenP"));  if (!yymatchChar(G, '<')) goto l991;  if (!yy_Spnl(G)) { goto l991; }
  {  int yypos992= G->pos, yythunkpos992= G->thunkpos;  if (!yymatchChar(G, 'p')) goto l993;  goto l992;
  l993:;	  G->pos= yypos992; G->thunkpos= yythunkpos992;  if (!yymatchChar(G, 'P')) goto l991;
  }
  l992:;	  if (!yy_Spnl(G)) { goto l991; }
  l994:;	
  {  int yypos995= G->pos, yythunkpos995= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l995; }  goto l994;
  l995:;	  G->pos= yypos995; G->thunkpos= yythunkpos995;
  }  if (!yymatchChar(G, '>')) goto l991;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenP", G->buf+G->pos));
  return 1;
  l991:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOl"));  if (!yy_HtmlBlockOpenOl(G)) { goto l996; }
  l997:;	
  {  int yypos998= G->pos, yythunkpos998= G->thunkpos;
  {  int yypos999= G->pos, yythunkpos999= G->thunkpos;  if (!yy_HtmlBlockOl(G)) { goto l1000; }  goto l999;
  l1000:;	  G->pos= yypos999; G->thunkpos= yythunkpos999;
  {  int yypos1001= G->pos, yythunkpos1001= G->thunkpos;  if (!yy_HtmlBlockCloseOl(G)) { goto l1001; }  goto l998;
  l1001:;	  G->pos= yypos1001; G->thunkpos= yythunkpos1001;
  }  if (!yymatchDot(G)) goto l998;
  }
  l999:;	  goto l997;
  l998:;	  G->pos= yypos998; G->thunkpos= yythunkpos998;
  }  if (!yy_HtmlBlockCloseOl(G)) { goto l996; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOl", G->buf+G->pos));
  return 1;
  l996:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseOl"));  if (!yymatchChar(G, '<')) goto l1002;  if (!yy_Spnl(G)) { goto l1002; }  if (!yymatchChar(G, '/')) goto l1002;
  {  int yypos1003= G->pos, yythunkpos1003= G->thunkpos;  if (!yymatchString(G, "ol")) goto l1004;  goto l1003;
  l1004:;	  G->pos= yypos1003; G->thunkpos= yythunkpos1003;  if (!yymatchString(G, "OL")) goto l1002;
  }
  l1003:;	  if (!yy_Spnl(G)) { goto l1002; }  if (!yymatchChar(G, '>')) goto l1002;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseOl", G->buf+G->pos));
  return 1;
  l1002:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenOl"));  if (!yymatchChar(G, '<')) goto l1005;  if (!yy_Spnl(G)) { goto l1005; }
  {  int yypos1006= G->pos, yythunkpos1006= G->thunkpos;  if (!yymatchString(G, "ol")) goto l1007;  goto l1006;
  l1007:;	  G->pos= yypos1006; G->thunkpos= yythunkpos1006;  if (!yymatchString(G, "OL")) goto l1005;
  }
  l1006:;	  if (!yy_Spnl(G)) { goto l1005; }
  l1008:;	
  {  int yypos1009= G->pos, yythunkpos1009= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1009; }  goto l1008;
  l1009:;	  G->pos= yypos1009; G->thunkpos= yythunkpos1009;
  }  if (!yymatchChar(G, '>')) goto l1005;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenOl", G->buf+G->pos));
  return 1;
  l1005:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockNoscript"));  if (!yy_HtmlBlockOpenNoscript(G)) { goto l1010; }
  l1011:;	
  {  int yypos1012= G->pos, yythunkpos1012= G->thunkpos;
  {  int yypos1013= G->pos, yythunkpos1013= G->thunkpos;  if (!yy_HtmlBlockNoscript(G)) { goto l1014; }  goto l1013;
  l1014:;	  G->pos= yypos1013; G->thunkpos= yythunkpos1013;
  {  int yypos1015= G->pos, yythunkpos1015= G->thunkpos;  if (!yy_HtmlBlockCloseNoscript(G)) { goto l1015; }  goto l1012;
  l1015:;	  G->pos= yypos1015; G->thunkpos= yythunkpos1015;
  }  if (!yymatchDot(G)) goto l1012;
  }
  l1013:;	  goto l1011;
  l1012:;	  G->pos= yypos1012; G->thunkpos= yythunkpos1012;
  }  if (!yy_HtmlBlockCloseNoscript(G)) { goto l1010; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockNoscript", G->buf+G->pos));
  return 1;
  l1010:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseNoscript"));  if (!yymatchChar(G, '<')) goto l1016;  if (!yy_Spnl(G)) { goto l1016; }  if (!yymatchChar(G, '/')) goto l1016;
  {  int yypos1017= G->pos, yythunkpos1017= G->thunkpos;  if (!yymatchString(G, "noscript")) goto l1018;  goto l1017;
  l1018:;	  G->pos= yypos1017; G->thunkpos= yythunkpos1017;  if (!yymatchString(G, "NOSCRIPT")) goto l1016;
  }
  l1017:;	  if (!yy_Spnl(G)) { goto l1016; }  if (!yymatchChar(G, '>')) goto l1016;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseNoscript", G->buf+G->pos));
  return 1;
  l1016:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenNoscript"));  if (!yymatchChar(G, '<')) goto l1019;  if (!yy_Spnl(G)) { goto l1019; }
  {  int yypos1020= G->pos, yythunkpos1020= G->thunkpos;  if (!yymatchString(G, "noscript")) goto l1021;  goto l1020;
  l1021:;	  G->pos= yypos1020; G->thunkpos= yythunkpos1020;  if (!yymatchString(G, "NOSCRIPT")) goto l1019;
  }
  l1020:;	  if (!yy_Spnl(G)) { goto l1019; }
  l1022:;	
  {  int yypos1023= G->pos, yythunkpos1023= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1023; }  goto l1022;
  l1023:;	  G->pos= yypos1023; G->thunkpos= yythunkpos1023;
  }  if (!yymatchChar(G, '>')) goto l1019;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenNoscript", G->buf+G->pos));
  return 1;
  l1019:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockNoframes"));  if (!yy_HtmlBlockOpenNoframes(G)) { goto l1024; }
  l1025:;	
  {  int yypos1026= G->pos, yythunkpos1026= G->thunkpos;
  {  int yypos1027= G->pos, yythunkpos1027= G->thunkpos;  if (!yy_HtmlBlockNoframes(G)) { goto l1028; }  goto l1027;
  l1028:;	  G->pos= yypos1027; G->thunkpos= yythunkpos1027;
  {  int yypos1029= G->pos, yythunkpos1029= G->thunkpos;  if (!yy_HtmlBlockCloseNoframes(G)) { goto l1029; }  goto l1026;
  l1029:;	  G->pos= yypos1029; G->thunkpos= yythunkpos1029;
  }  if (!yymatchDot(G)) goto l1026;
  }
  l1027:;	  goto l1025;
  l1026:;	  G->pos= yypos1026; G->thunkpos= yythunkpos1026;
  }  if (!yy_HtmlBlockCloseNoframes(G)) { goto l1024; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockNoframes", G->buf+G->pos));
  return 1;
  l1024:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseNoframes"));  if (!yymatchChar(G, '<')) goto l1030;  if (!yy_Spnl(G)) { goto l1030; }  if (!yymatchChar(G, '/')) goto l1030;
  {  int yypos1031= G->pos, yythunkpos1031= G->thunkpos;  if (!yymatchString(G, "noframes")) goto l1032;  goto l1031;
  l1032:;	  G->pos= yypos1031; G->thunkpos= yythunkpos1031;  if (!yymatchString(G, "NOFRAMES")) goto l1030;
  }
  l1031:;	  if (!yy_Spnl(G)) { goto l1030; }  if (!yymatchChar(G, '>')) goto l1030;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseNoframes", G->buf+G->pos));
  return 1;
  l1030:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenNoframes"));  if (!yymatchChar(G, '<')) goto l1033;  if (!yy_Spnl(G)) { goto l1033; }
  {  int yypos1034= G->pos, yythunkpos1034= G->thunkpos;  if (!yymatchString(G, "noframes")) goto l1035;  goto l1034;
  l1035:;	  G->pos= yypos1034; G->thunkpos= yythunkpos1034;  if (!yymatchString(G, "NOFRAMES")) goto l1033;
  }
  l1034:;	  if (!yy_Spnl(G)) { goto l1033; }
  l1036:;	
  {  int yypos1037= G->pos, yythunkpos1037= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1037; }  goto l1036;
  l1037:;	  G->pos= yypos1037; G->thunkpos= yythunkpos1037;
  }  if (!yymatchChar(G, '>')) goto l1033;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenNoframes", G->buf+G->pos));
  return 1;
  l1033:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockMenu"));  if (!yy_HtmlBlockOpenMenu(G)) { goto l1038; }
  l1039:;	
  {  int yypos1040= G->pos, yythunkpos1040= G->thunkpos;
  {  int yypos1041= G->pos, yythunkpos1041= G->thunkpos;  if (!yy_HtmlBlockMenu(G)) { goto l1042; }  goto l1041;
  l1042:;	  G->pos= yypos1041; G->thunkpos= yythunkpos1041;
  {  int yypos1043= G->pos, yythunkpos1043= G->thunkpos;  if (!yy_HtmlBlockCloseMenu(G)) { goto l1043; }  goto l1040;
  l1043:;	  G->pos= yypos1043; G->thunkpos= yythunkpos1043;
  }  if (!yymatchDot(G)) goto l1040;
  }
  l1041:;	  goto l1039;
  l1040:;	  G->pos= yypos1040; G->thunkpos= yythunkpos1040;
  }  if (!yy_HtmlBlockCloseMenu(G)) { goto l1038; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockMenu", G->buf+G->pos));
  return 1;
  l1038:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseMenu"));  if (!yymatchChar(G, '<')) goto l1044;  if (!yy_Spnl(G)) { goto l1044; }  if (!yymatchChar(G, '/')) goto l1044;
  {  int yypos1045= G->pos, yythunkpos1045= G->thunkpos;  if (!yymatchString(G, "menu")) goto l1046;  goto l1045;
  l1046:;	  G->pos= yypos1045; G->thunkpos= yythunkpos1045;  if (!yymatchString(G, "MENU")) goto l1044;
  }
  l1045:;	  if (!yy_Spnl(G)) { goto l1044; }  if (!yymatchChar(G, '>')) goto l1044;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseMenu", G->buf+G->pos));
  return 1;
  l1044:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenMenu"));  if (!yymatchChar(G, '<')) goto l1047;  if (!yy_Spnl(G)) { goto l1047; }
  {  int yypos1048= G->pos, yythunkpos1048= G->thunkpos;  if (!yymatchString(G, "menu")) goto l1049;  goto l1048;
  l1049:;	  G->pos= yypos1048; G->thunkpos= yythunkpos1048;  if (!yymatchString(G, "MENU")) goto l1047;
  }
  l1048:;	  if (!yy_Spnl(G)) { goto l1047; }
  l1050:;	
  {  int yypos1051= G->pos, yythunkpos1051= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1051; }  goto l1050;
  l1051:;	  G->pos= yypos1051; G->thunkpos= yythunkpos1051;
  }  if (!yymatchChar(G, '>')) goto l1047;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenMenu", G->buf+G->pos));
  return 1;
  l1047:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH6"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1052;  if (!yy_LocMarker(G)) { goto l1052; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH6(G)) { goto l1052; }
  l1053:;	
  {  int yypos1054= G->pos, yythunkpos1054= G->thunkpos;
  {  int yypos1055= G->pos, yythunkpos1055= G->thunkpos;  if (!yy_HtmlBlockH6(G)) { goto l1056; }  goto l1055;
  l1056:;	  G->pos= yypos1055; G->thunkpos= yythunkpos1055;
  {  int yypos1057= G->pos, yythunkpos1057= G->thunkpos;  if (!yy_HtmlBlockCloseH6(G)) { goto l1057; }  goto l1054;
  l1057:;	  G->pos= yypos1057; G->thunkpos= yythunkpos1057;
  }  if (!yymatchDot(G)) goto l1054;
  }
  l1055:;	  goto l1053;
  l1054:;	  G->pos= yypos1054; G->thunkpos= yythunkpos1054;
  }  if (!yy_HtmlBlockCloseH6(G)) { goto l1052; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1052;  yyDo(G, yy_1_HtmlBlockH6, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH6", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1052:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH6"));  if (!yymatchChar(G, '<')) goto l1058;  if (!yy_Spnl(G)) { goto l1058; }  if (!yymatchChar(G, '/')) goto l1058;
  {  int yypos1059= G->pos, yythunkpos1059= G->thunkpos;  if (!yymatchString(G, "h6")) goto l1060;  goto l1059;
  l1060:;	  G->pos= yypos1059; G->thunkpos= yythunkpos1059;  if (!yymatchString(G, "H6")) goto l1058;
  }
  l1059:;	  if (!yy_Spnl(G)) { goto l1058; }  if (!yymatchChar(G, '>')) goto l1058;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH6", G->buf+G->pos));
  return 1;
  l1058:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH6"));  if (!yymatchChar(G, '<')) goto l1061;  if (!yy_Spnl(G)) { goto l1061; }
  {  int yypos1062= G->pos, yythunkpos1062= G->thunkpos;  if (!yymatchString(G, "h6")) goto l1063;  goto l1062;
  l1063:;	  G->pos= yypos1062; G->thunkpos= yythunkpos1062;  if (!yymatchString(G, "H6")) goto l1061;
  }
  l1062:;	  if (!yy_Spnl(G)) { goto l1061; }
  l1064:;	
  {  int yypos1065= G->pos, yythunkpos1065= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1065; }  goto l1064;
  l1065:;	  G->pos= yypos1065; G->thunkpos= yythunkpos1065;
  }  if (!yymatchChar(G, '>')) goto l1061;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH6", G->buf+G->pos));
  return 1;
  l1061:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH5"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1066;  if (!yy_LocMarker(G)) { goto l1066; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH5(G)) { goto l1066; }
  l1067:;	
  {  int yypos1068= G->pos, yythunkpos1068= G->thunkpos;
  {  int yypos1069= G->pos, yythunkpos1069= G->thunkpos;  if (!yy_HtmlBlockH5(G)) { goto l1070; }  goto l1069;
  l1070:;	  G->pos= yypos1069; G->thunkpos= yythunkpos1069;
  {  int yypos1071= G->pos, yythunkpos1071= G->thunkpos;  if (!yy_HtmlBlockCloseH5(G)) { goto l1071; }  goto l1068;
  l1071:;	  G->pos= yypos1071; G->thunkpos= yythunkpos1071;
  }  if (!yymatchDot(G)) goto l1068;
  }
  l1069:;	  goto l1067;
  l1068:;	  G->pos= yypos1068; G->thunkpos= yythunkpos1068;
  }  if (!yy_HtmlBlockCloseH5(G)) { goto l1066; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1066;  yyDo(G, yy_1_HtmlBlockH5, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH5", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1066:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH5"));  if (!yymatchChar(G, '<')) goto l1072;  if (!yy_Spnl(G)) { goto l1072; }  if (!yymatchChar(G, '/')) goto l1072;
  {  int yypos1073= G->pos, yythunkpos1073= G->thunkpos;  if (!yymatchString(G, "h5")) goto l1074;  goto l1073;
  l1074:;	  G->pos= yypos1073; G->thunkpos= yythunkpos1073;  if (!yymatchString(G, "H5")) goto l1072;
  }
  l1073:;	  if (!yy_Spnl(G)) { goto l1072; }  if (!yymatchChar(G, '>')) goto l1072;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH5", G->buf+G->pos));
  return 1;
  l1072:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH5"));  if (!yymatchChar(G, '<')) goto l1075;  if (!yy_Spnl(G)) { goto l1075; }
  {  int yypos1076= G->pos, yythunkpos1076= G->thunkpos;  if (!yymatchString(G, "h5")) goto l1077;  goto l1076;
  l1077:;	  G->pos= yypos1076; G->thunkpos= yythunkpos1076;  if (!yymatchString(G, "H5")) goto l1075;
  }
  l1076:;	  if (!yy_Spnl(G)) { goto l1075; }
  l1078:;	
  {  int yypos1079= G->pos, yythunkpos1079= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1079; }  goto l1078;
  l1079:;	  G->pos= yypos1079; G->thunkpos= yythunkpos1079;
  }  if (!yymatchChar(G, '>')) goto l1075;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH5", G->buf+G->pos));
  return 1;
  l1075:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH4"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1080;  if (!yy_LocMarker(G)) { goto l1080; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH4(G)) { goto l1080; }
  l1081:;	
  {  int yypos1082= G->pos, yythunkpos1082= G->thunkpos;
  {  int yypos1083= G->pos, yythunkpos1083= G->thunkpos;  if (!yy_HtmlBlockH4(G)) { goto l1084; }  goto l1083;
  l1084:;	  G->pos= yypos1083; G->thunkpos= yythunkpos1083;
  {  int yypos1085= G->pos, yythunkpos1085= G->thunkpos;  if (!yy_HtmlBlockCloseH4(G)) { goto l1085; }  goto l1082;
  l1085:;	  G->pos= yypos1085; G->thunkpos= yythunkpos1085;
  }  if (!yymatchDot(G)) goto l1082;
  }
  l1083:;	  goto l1081;
  l1082:;	  G->pos= yypos1082; G->thunkpos= yythunkpos1082;
  }  if (!yy_HtmlBlockCloseH4(G)) { goto l1080; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1080;  yyDo(G, yy_1_HtmlBlockH4, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH4", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1080:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH4"));  if (!yymatchChar(G, '<')) goto l1086;  if (!yy_Spnl(G)) { goto l1086; }  if (!yymatchChar(G, '/')) goto l1086;
  {  int yypos1087= G->pos, yythunkpos1087= G->thunkpos;  if (!yymatchString(G, "h4")) goto l1088;  goto l1087;
  l1088:;	  G->pos= yypos1087; G->thunkpos= yythunkpos1087;  if (!yymatchString(G, "H4")) goto l1086;
  }
  l1087:;	  if (!yy_Spnl(G)) { goto l1086; }  if (!yymatchChar(G, '>')) goto l1086;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH4", G->buf+G->pos));
  return 1;
  l1086:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH4"));  if (!yymatchChar(G, '<')) goto l1089;  if (!yy_Spnl(G)) { goto l1089; }
  {  int yypos1090= G->pos, yythunkpos1090= G->thunkpos;  if (!yymatchString(G, "h4")) goto l1091;  goto l1090;
  l1091:;	  G->pos= yypos1090; G->thunkpos= yythunkpos1090;  if (!yymatchString(G, "H4")) goto l1089;
  }
  l1090:;	  if (!yy_Spnl(G)) { goto l1089; }
  l1092:;	
  {  int yypos1093= G->pos, yythunkpos1093= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1093; }  goto l1092;
  l1093:;	  G->pos= yypos1093; G->thunkpos= yythunkpos1093;
  }  if (!yymatchChar(G, '>')) goto l1089;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH4", G->buf+G->pos));
  return 1;
  l1089:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH3"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1094;  if (!yy_LocMarker(G)) { goto l1094; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH3(G)) { goto l1094; }
  l1095:;	
  {  int yypos1096= G->pos, yythunkpos1096= G->thunkpos;
  {  int yypos1097= G->pos, yythunkpos1097= G->thunkpos;  if (!yy_HtmlBlockH3(G)) { goto l1098; }  goto l1097;
  l1098:;	  G->pos= yypos1097; G->thunkpos= yythunkpos1097;
  {  int yypos1099= G->pos, yythunkpos1099= G->thunkpos;  if (!yy_HtmlBlockCloseH3(G)) { goto l1099; }  goto l1096;
  l1099:;	  G->pos= yypos1099; G->thunkpos= yythunkpos1099;
  }  if (!yymatchDot(G)) goto l1096;
  }
  l1097:;	  goto l1095;
  l1096:;	  G->pos= yypos1096; G->thunkpos= yythunkpos1096;
  }  if (!yy_HtmlBlockCloseH3(G)) { goto l1094; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1094;  yyDo(G, yy_1_HtmlBlockH3, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH3", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1094:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH3"));  if (!yymatchChar(G, '<')) goto l1100;  if (!yy_Spnl(G)) { goto l1100; }  if (!yymatchChar(G, '/')) goto l1100;
  {  int yypos1101= G->pos, yythunkpos1101= G->thunkpos;  if (!yymatchString(G, "h3")) goto l1102;  goto l1101;
  l1102:;	  G->pos= yypos1101; G->thunkpos= yythunkpos1101;  if (!yymatchString(G, "H3")) goto l1100;
  }
  l1101:;	  if (!yy_Spnl(G)) { goto l1100; }  if (!yymatchChar(G, '>')) goto l1100;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH3", G->buf+G->pos));
  return 1;
  l1100:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH3"));  if (!yymatchChar(G, '<')) goto l1103;  if (!yy_Spnl(G)) { goto l1103; }
  {  int yypos1104= G->pos, yythunkpos1104= G->thunkpos;  if (!yymatchString(G, "h3")) goto l1105;  goto l1104;
  l1105:;	  G->pos= yypos1104; G->thunkpos= yythunkpos1104;  if (!yymatchString(G, "H3")) goto l1103;
  }
  l1104:;	  if (!yy_Spnl(G)) { goto l1103; }
  l1106:;	
  {  int yypos1107= G->pos, yythunkpos1107= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1107; }  goto l1106;
  l1107:;	  G->pos= yypos1107; G->thunkpos= yythunkpos1107;
  }  if (!yymatchChar(G, '>')) goto l1103;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH3", G->buf+G->pos));
  return 1;
  l1103:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH2"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1108;  if (!yy_LocMarker(G)) { goto l1108; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH2(G)) { goto l1108; }
  l1109:;	
  {  int yypos1110= G->pos, yythunkpos1110= G->thunkpos;
  {  int yypos1111= G->pos, yythunkpos1111= G->thunkpos;  if (!yy_HtmlBlockH2(G)) { goto l1112; }  goto l1111;
  l1112:;	  G->pos= yypos1111; G->thunkpos= yythunkpos1111;
  {  int yypos1113= G->pos, yythunkpos1113= G->thunkpos;  if (!yy_HtmlBlockCloseH2(G)) { goto l1113; }  goto l1110;
  l1113:;	  G->pos= yypos1113; G->thunkpos= yythunkpos1113;
  }  if (!yymatchDot(G)) goto l1110;
  }
  l1111:;	  goto l1109;
  l1110:;	  G->pos= yypos1110; G->thunkpos= yythunkpos1110;
  }  if (!yy_HtmlBlockCloseH2(G)) { goto l1108; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1108;  yyDo(G, yy_1_HtmlBlockH2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH2", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1108:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH2"));  if (!yymatchChar(G, '<')) goto l1114;  if (!yy_Spnl(G)) { goto l1114; }  if (!yymatchChar(G, '/')) goto l1114;
  {  int yypos1115= G->pos, yythunkpos1115= G->thunkpos;  if (!yymatchString(G, "h2")) goto l1116;  goto l1115;
  l1116:;	  G->pos= yypos1115; G->thunkpos= yythunkpos1115;  if (!yymatchString(G, "H2")) goto l1114;
  }
  l1115:;	  if (!yy_Spnl(G)) { goto l1114; }  if (!yymatchChar(G, '>')) goto l1114;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH2", G->buf+G->pos));
  return 1;
  l1114:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH2"));  if (!yymatchChar(G, '<')) goto l1117;  if (!yy_Spnl(G)) { goto l1117; }
  {  int yypos1118= G->pos, yythunkpos1118= G->thunkpos;  if (!yymatchString(G, "h2")) goto l1119;  goto l1118;
  l1119:;	  G->pos= yypos1118; G->thunkpos= yythunkpos1118;  if (!yymatchString(G, "H2")) goto l1117;
  }
  l1118:;	  if (!yy_Spnl(G)) { goto l1117; }
  l1120:;	
  {  int yypos1121= G->pos, yythunkpos1121= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1121; }  goto l1120;
  l1121:;	  G->pos= yypos1121; G->thunkpos= yythunkpos1121;
  }  if (!yymatchChar(G, '>')) goto l1117;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH2", G->buf+G->pos));
  return 1;
  l1117:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH1"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1122;  if (!yy_LocMarker(G)) { goto l1122; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH1(G)) { goto l1122; }
  l1123:;	
  {  int yypos1124= G->pos, yythunkpos1124= G->thunkpos;
  {  int yypos1125= G->pos, yythunkpos1125= G->thunkpos;  if (!yy_HtmlBlockH1(G)) { goto l1126; }  goto l1125;
  l1126:;	  G->pos= yypos1125; G->thunkpos= yythunkpos1125;
  {  int yypos1127= G->pos, yythunkpos1127= G->thunkpos;  if (!yy_HtmlBlockCloseH1(G)) { goto l1127; }  goto l1124;
  l1127:;	  G->pos= yypos1127; G->thunkpos= yythunkpos1127;
  }  if (!yymatchDot(G)) goto l1124;
  }
  l1125:;	  goto l1123;
  l1124:;	  G->pos= yypos1124; G->thunkpos= yythunkpos1124;
  }  if (!yy_HtmlBlockCloseH1(G)) { goto l1122; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1122;  yyDo(G, yy_1_HtmlBlockH1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH1", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1122:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH1"));  if (!yymatchChar(G, '<')) goto l1128;  if (!yy_Spnl(G)) { goto l1128; }  if (!yymatchChar(G, '/')) goto l1128;
  {  int yypos1129= G->pos, yythunkpos1129= G->thunkpos;  if (!yymatchString(G, "h1")) goto l1130;  goto l1129;
  l1130:;	  G->pos= yypos1129; G->thunkpos= yythunkpos1129;  if (!yymatchString(G, "H1")) goto l1128;
  }
  l1129:;	  if (!yy_Spnl(G)) { goto l1128; }  if (!yymatchChar(G, '>')) goto l1128;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH1", G->buf+G->pos));
  return 1;
  l1128:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH1"));  if (!yymatchChar(G, '<')) goto l1131;  if (!yy_Spnl(G)) { goto l1131; }
  {  int yypos1132= G->pos, yythunkpos1132= G->thunkpos;  if (!yymatchString(G, "h1")) goto l1133;  goto l1132;
  l1133:;	  G->pos= yypos1132; G->thunkpos= yythunkpos1132;  if (!yymatchString(G, "H1")) goto l1131;
  }
  l1132:;	  if (!yy_Spnl(G)) { goto l1131; }
  l1134:;	
  {  int yypos1135= G->pos, yythunkpos1135= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1135; }  goto l1134;
  l1135:;	  G->pos= yypos1135; G->thunkpos= yythunkpos1135;
  }  if (!yymatchChar(G, '>')) goto l1131;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH1", G->buf+G->pos));
  return 1;
  l1131:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockForm"));  if (!yy_HtmlBlockOpenForm(G)) { goto l1136; }
  l1137:;	
  {  int yypos1138= G->pos, yythunkpos1138= G->thunkpos;
  {  int yypos1139= G->pos, yythunkpos1139= G->thunkpos;  if (!yy_HtmlBlockForm(G)) { goto l1140; }  goto l1139;
  l1140:;	  G->pos= yypos1139; G->thunkpos= yythunkpos1139;
  {  int yypos1141= G->pos, yythunkpos1141= G->thunkpos;  if (!yy_HtmlBlockCloseForm(G)) { goto l1141; }  goto l1138;
  l1141:;	  G->pos= yypos1141; G->thunkpos= yythunkpos1141;
  }  if (!yymatchDot(G)) goto l1138;
  }
  l1139:;	  goto l1137;
  l1138:;	  G->pos= yypos1138; G->thunkpos= yythunkpos1138;
  }  if (!yy_HtmlBlockCloseForm(G)) { goto l1136; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockForm", G->buf+G->pos));
  return 1;
  l1136:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseForm"));  if (!yymatchChar(G, '<')) goto l1142;  if (!yy_Spnl(G)) { goto l1142; }  if (!yymatchChar(G, '/')) goto l1142;
  {  int yypos1143= G->pos, yythunkpos1143= G->thunkpos;  if (!yymatchString(G, "form")) goto l1144;  goto l1143;
  l1144:;	  G->pos= yypos1143; G->thunkpos= yythunkpos1143;  if (!yymatchString(G, "FORM")) goto l1142;
  }
  l1143:;	  if (!yy_Spnl(G)) { goto l1142; }  if (!yymatchChar(G, '>')) goto l1142;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseForm", G->buf+G->pos));
  return 1;
  l1142:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenForm"));  if (!yymatchChar(G, '<')) goto l1145;  if (!yy_Spnl(G)) { goto l1145; }
  {  int yypos1146= G->pos, yythunkpos1146= G->thunkpos;  if (!yymatchString(G, "form")) goto l1147;  goto l1146;
  l1147:;	  G->pos= yypos1146; G->thunkpos= yythunkpos1146;  if (!yymatchString(G, "FORM")) goto l1145;
  }
  l1146:;	  if (!yy_Spnl(G)) { goto l1145; }
  l1148:;	
  {  int yypos1149= G->pos, yythunkpos1149= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1149; }  goto l1148;
  l1149:;	  G->pos= yypos1149; G->thunkpos= yythunkpos1149;
  }  if (!yymatchChar(G, '>')) goto l1145;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenForm", G->buf+G->pos));
  return 1;
  l1145:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockFieldset"));  if (!yy_HtmlBlockOpenFieldset(G)) { goto l1150; }
  l1151:;	
  {  int yypos1152= G->pos, yythunkpos1152= G->thunkpos;
  {  int yypos1153= G->pos, yythunkpos1153= G->thunkpos;  if (!yy_HtmlBlockFieldset(G)) { goto l1154; }  goto l1153;
  l1154:;	  G->pos= yypos1153; G->thunkpos= yythunkpos1153;
  {  int yypos1155= G->pos, yythunkpos1155= G->thunkpos;  if (!yy_HtmlBlockCloseFieldset(G)) { goto l1155; }  goto l1152;
  l1155:;	  G->pos= yypos1155; G->thunkpos= yythunkpos1155;
  }  if (!yymatchDot(G)) goto l1152;
  }
  l1153:;	  goto l1151;
  l1152:;	  G->pos= yypos1152; G->thunkpos= yythunkpos1152;
  }  if (!yy_HtmlBlockCloseFieldset(G)) { goto l1150; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockFieldset", G->buf+G->pos));
  return 1;
  l1150:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseFieldset"));  if (!yymatchChar(G, '<')) goto l1156;  if (!yy_Spnl(G)) { goto l1156; }  if (!yymatchChar(G, '/')) goto l1156;
  {  int yypos1157= G->pos, yythunkpos1157= G->thunkpos;  if (!yymatchString(G, "fieldset")) goto l1158;  goto l1157;
  l1158:;	  G->pos= yypos1157; G->thunkpos= yythunkpos1157;  if (!yymatchString(G, "FIELDSET")) goto l1156;
  }
  l1157:;	  if (!yy_Spnl(G)) { goto l1156; }  if (!yymatchChar(G, '>')) goto l1156;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseFieldset", G->buf+G->pos));
  return 1;
  l1156:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenFieldset"));  if (!yymatchChar(G, '<')) goto l1159;  if (!yy_Spnl(G)) { goto l1159; }
  {  int yypos1160= G->pos, yythunkpos1160= G->thunkpos;  if (!yymatchString(G, "fieldset")) goto l1161;  goto l1160;
  l1161:;	  G->pos= yypos1160; G->thunkpos= yythunkpos1160;  if (!yymatchString(G, "FIELDSET")) goto l1159;
  }
  l1160:;	  if (!yy_Spnl(G)) { goto l1159; }
  l1162:;	
  {  int yypos1163= G->pos, yythunkpos1163= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1163; }  goto l1162;
  l1163:;	  G->pos= yypos1163; G->thunkpos= yythunkpos1163;
  }  if (!yymatchChar(G, '>')) goto l1159;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenFieldset", G->buf+G->pos));
  return 1;
  l1159:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDl"));  if (!yy_HtmlBlockOpenDl(G)) { goto l1164; }
  l1165:;	
  {  int yypos1166= G->pos, yythunkpos1166= G->thunkpos;
  {  int yypos1167= G->pos, yythunkpos1167= G->thunkpos;  if (!yy_HtmlBlockDl(G)) { goto l1168; }  goto l1167;
  l1168:;	  G->pos= yypos1167; G->thunkpos= yythunkpos1167;
  {  int yypos1169= G->pos, yythunkpos1169= G->thunkpos;  if (!yy_HtmlBlockCloseDl(G)) { goto l1169; }  goto l1166;
  l1169:;	  G->pos= yypos1169; G->thunkpos= yythunkpos1169;
  }  if (!yymatchDot(G)) goto l1166;
  }
  l1167:;	  goto l1165;
  l1166:;	  G->pos= yypos1166; G->thunkpos= yythunkpos1166;
  }  if (!yy_HtmlBlockCloseDl(G)) { goto l1164; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDl", G->buf+G->pos));
  return 1;
  l1164:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDl"));  if (!yymatchChar(G, '<')) goto l1170;  if (!yy_Spnl(G)) { goto l1170; }  if (!yymatchChar(G, '/')) goto l1170;
  {  int yypos1171= G->pos, yythunkpos1171= G->thunkpos;  if (!yymatchString(G, "dl")) goto l1172;  goto l1171;
  l1172:;	  G->pos= yypos1171; G->thunkpos= yythunkpos1171;  if (!yymatchString(G, "DL")) goto l1170;
  }
  l1171:;	  if (!yy_Spnl(G)) { goto l1170; }  if (!yymatchChar(G, '>')) goto l1170;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDl", G->buf+G->pos));
  return 1;
  l1170:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDl"));  if (!yymatchChar(G, '<')) goto l1173;  if (!yy_Spnl(G)) { goto l1173; }
  {  int yypos1174= G->pos, yythunkpos1174= G->thunkpos;  if (!yymatchString(G, "dl")) goto l1175;  goto l1174;
  l1175:;	  G->pos= yypos1174; G->thunkpos= yythunkpos1174;  if (!yymatchString(G, "DL")) goto l1173;
  }
  l1174:;	  if (!yy_Spnl(G)) { goto l1173; }
  l1176:;	
  {  int yypos1177= G->pos, yythunkpos1177= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1177; }  goto l1176;
  l1177:;	  G->pos= yypos1177; G->thunkpos= yythunkpos1177;
  }  if (!yymatchChar(G, '>')) goto l1173;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDl", G->buf+G->pos));
  return 1;
  l1173:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDiv"));  if (!yy_HtmlBlockOpenDiv(G)) { goto l1178; }
  l1179:;	
  {  int yypos1180= G->pos, yythunkpos1180= G->thunkpos;
  {  int yypos1181= G->pos, yythunkpos1181= G->thunkpos;  if (!yy_HtmlBlockDiv(G)) { goto l1182; }  goto l1181;
  l1182:;	  G->pos= yypos1181; G->thunkpos= yythunkpos1181;
  {  int yypos1183= G->pos, yythunkpos1183= G->thunkpos;  if (!yy_HtmlBlockCloseDiv(G)) { goto l1183; }  goto l1180;
  l1183:;	  G->pos= yypos1183; G->thunkpos= yythunkpos1183;
  }  if (!yymatchDot(G)) goto l1180;
  }
  l1181:;	  goto l1179;
  l1180:;	  G->pos= yypos1180; G->thunkpos= yythunkpos1180;
  }  if (!yy_HtmlBlockCloseDiv(G)) { goto l1178; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDiv", G->buf+G->pos));
  return 1;
  l1178:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDiv"));  if (!yymatchChar(G, '<')) goto l1184;  if (!yy_Spnl(G)) { goto l1184; }  if (!yymatchChar(G, '/')) goto l1184;
  {  int yypos1185= G->pos, yythunkpos1185= G->thunkpos;  if (!yymatchString(G, "div")) goto l1186;  goto l1185;
  l1186:;	  G->pos= yypos1185; G->thunkpos= yythunkpos1185;  if (!yymatchString(G, "DIV")) goto l1184;
  }
  l1185:;	  if (!yy_Spnl(G)) { goto l1184; }  if (!yymatchChar(G, '>')) goto l1184;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDiv", G->buf+G->pos));
  return 1;
  l1184:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDiv"));  if (!yymatchChar(G, '<')) goto l1187;  if (!yy_Spnl(G)) { goto l1187; }
  {  int yypos1188= G->pos, yythunkpos1188= G->thunkpos;  if (!yymatchString(G, "div")) goto l1189;  goto l1188;
  l1189:;	  G->pos= yypos1188; G->thunkpos= yythunkpos1188;  if (!yymatchString(G, "DIV")) goto l1187;
  }
  l1188:;	  if (!yy_Spnl(G)) { goto l1187; }
  l1190:;	
  {  int yypos1191= G->pos, yythunkpos1191= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1191; }  goto l1190;
  l1191:;	  G->pos= yypos1191; G->thunkpos= yythunkpos1191;
  }  if (!yymatchChar(G, '>')) goto l1187;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDiv", G->buf+G->pos));
  return 1;
  l1187:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDir"));  if (!yy_HtmlBlockOpenDir(G)) { goto l1192; }
  l1193:;	
  {  int yypos1194= G->pos, yythunkpos1194= G->thunkpos;
  {  int yypos1195= G->pos, yythunkpos1195= G->thunkpos;  if (!yy_HtmlBlockDir(G)) { goto l1196; }  goto l1195;
  l1196:;	  G->pos= yypos1195; G->thunkpos= yythunkpos1195;
  {  int yypos1197= G->pos, yythunkpos1197= G->thunkpos;  if (!yy_HtmlBlockCloseDir(G)) { goto l1197; }  goto l1194;
  l1197:;	  G->pos= yypos1197; G->thunkpos= yythunkpos1197;
  }  if (!yymatchDot(G)) goto l1194;
  }
  l1195:;	  goto l1193;
  l1194:;	  G->pos= yypos1194; G->thunkpos= yythunkpos1194;
  }  if (!yy_HtmlBlockCloseDir(G)) { goto l1192; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDir", G->buf+G->pos));
  return 1;
  l1192:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDir"));  if (!yymatchChar(G, '<')) goto l1198;  if (!yy_Spnl(G)) { goto l1198; }  if (!yymatchChar(G, '/')) goto l1198;
  {  int yypos1199= G->pos, yythunkpos1199= G->thunkpos;  if (!yymatchString(G, "dir")) goto l1200;  goto l1199;
  l1200:;	  G->pos= yypos1199; G->thunkpos= yythunkpos1199;  if (!yymatchString(G, "DIR")) goto l1198;
  }
  l1199:;	  if (!yy_Spnl(G)) { goto l1198; }  if (!yymatchChar(G, '>')) goto l1198;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDir", G->buf+G->pos));
  return 1;
  l1198:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDir"));  if (!yymatchChar(G, '<')) goto l1201;  if (!yy_Spnl(G)) { goto l1201; }
  {  int yypos1202= G->pos, yythunkpos1202= G->thunkpos;  if (!yymatchString(G, "dir")) goto l1203;  goto l1202;
  l1203:;	  G->pos= yypos1202; G->thunkpos= yythunkpos1202;  if (!yymatchString(G, "DIR")) goto l1201;
  }
  l1202:;	  if (!yy_Spnl(G)) { goto l1201; }
  l1204:;	
  {  int yypos1205= G->pos, yythunkpos1205= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1205; }  goto l1204;
  l1205:;	  G->pos= yypos1205; G->thunkpos= yythunkpos1205;
  }  if (!yymatchChar(G, '>')) goto l1201;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDir", G->buf+G->pos));
  return 1;
  l1201:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCenter"));  if (!yy_HtmlBlockOpenCenter(G)) { goto l1206; }
  l1207:;	
  {  int yypos1208= G->pos, yythunkpos1208= G->thunkpos;
  {  int yypos1209= G->pos, yythunkpos1209= G->thunkpos;  if (!yy_HtmlBlockCenter(G)) { goto l1210; }  goto l1209;
  l1210:;	  G->pos= yypos1209; G->thunkpos= yythunkpos1209;
  {  int yypos1211= G->pos, yythunkpos1211= G->thunkpos;  if (!yy_HtmlBlockCloseCenter(G)) { goto l1211; }  goto l1208;
  l1211:;	  G->pos= yypos1211; G->thunkpos= yythunkpos1211;
  }  if (!yymatchDot(G)) goto l1208;
  }
  l1209:;	  goto l1207;
  l1208:;	  G->pos= yypos1208; G->thunkpos= yythunkpos1208;
  }  if (!yy_HtmlBlockCloseCenter(G)) { goto l1206; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCenter", G->buf+G->pos));
  return 1;
  l1206:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseCenter"));  if (!yymatchChar(G, '<')) goto l1212;  if (!yy_Spnl(G)) { goto l1212; }  if (!yymatchChar(G, '/')) goto l1212;
  {  int yypos1213= G->pos, yythunkpos1213= G->thunkpos;  if (!yymatchString(G, "center")) goto l1214;  goto l1213;
  l1214:;	  G->pos= yypos1213; G->thunkpos= yythunkpos1213;  if (!yymatchString(G, "CENTER")) goto l1212;
  }
  l1213:;	  if (!yy_Spnl(G)) { goto l1212; }  if (!yymatchChar(G, '>')) goto l1212;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseCenter", G->buf+G->pos));
  return 1;
  l1212:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenCenter"));  if (!yymatchChar(G, '<')) goto l1215;  if (!yy_Spnl(G)) { goto l1215; }
  {  int yypos1216= G->pos, yythunkpos1216= G->thunkpos;  if (!yymatchString(G, "center")) goto l1217;  goto l1216;
  l1217:;	  G->pos= yypos1216; G->thunkpos= yythunkpos1216;  if (!yymatchString(G, "CENTER")) goto l1215;
  }
  l1216:;	  if (!yy_Spnl(G)) { goto l1215; }
  l1218:;	
  {  int yypos1219= G->pos, yythunkpos1219= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1219; }  goto l1218;
  l1219:;	  G->pos= yypos1219; G->thunkpos= yythunkpos1219;
  }  if (!yymatchChar(G, '>')) goto l1215;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenCenter", G->buf+G->pos));
  return 1;
  l1215:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockBlockquote"));  if (!yy_HtmlBlockOpenBlockquote(G)) { goto l1220; }
  l1221:;	
  {  int yypos1222= G->pos, yythunkpos1222= G->thunkpos;
  {  int yypos1223= G->pos, yythunkpos1223= G->thunkpos;  if (!yy_HtmlBlockBlockquote(G)) { goto l1224; }  goto l1223;
  l1224:;	  G->pos= yypos1223; G->thunkpos= yythunkpos1223;
  {  int yypos1225= G->pos, yythunkpos1225= G->thunkpos;  if (!yy_HtmlBlockCloseBlockquote(G)) { goto l1225; }  goto l1222;
  l1225:;	  G->pos= yypos1225; G->thunkpos= yythunkpos1225;
  }  if (!yymatchDot(G)) goto l1222;
  }
  l1223:;	  goto l1221;
  l1222:;	  G->pos= yypos1222; G->thunkpos= yythunkpos1222;
  }  if (!yy_HtmlBlockCloseBlockquote(G)) { goto l1220; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockBlockquote", G->buf+G->pos));
  return 1;
  l1220:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseBlockquote"));  if (!yymatchChar(G, '<')) goto l1226;  if (!yy_Spnl(G)) { goto l1226; }  if (!yymatchChar(G, '/')) goto l1226;
  {  int yypos1227= G->pos, yythunkpos1227= G->thunkpos;  if (!yymatchString(G, "blockquote")) goto l1228;  goto l1227;
  l1228:;	  G->pos= yypos1227; G->thunkpos= yythunkpos1227;  if (!yymatchString(G, "BLOCKQUOTE")) goto l1226;
  }
  l1227:;	  if (!yy_Spnl(G)) { goto l1226; }  if (!yymatchChar(G, '>')) goto l1226;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseBlockquote", G->buf+G->pos));
  return 1;
  l1226:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenBlockquote"));  if (!yymatchChar(G, '<')) goto l1229;  if (!yy_Spnl(G)) { goto l1229; }
  {  int yypos1230= G->pos, yythunkpos1230= G->thunkpos;  if (!yymatchString(G, "blockquote")) goto l1231;  goto l1230;
  l1231:;	  G->pos= yypos1230; G->thunkpos= yythunkpos1230;  if (!yymatchString(G, "BLOCKQUOTE")) goto l1229;
  }
  l1230:;	  if (!yy_Spnl(G)) { goto l1229; }
  l1232:;	
  {  int yypos1233= G->pos, yythunkpos1233= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1233; }  goto l1232;
  l1233:;	  G->pos= yypos1233; G->thunkpos= yythunkpos1233;
  }  if (!yymatchChar(G, '>')) goto l1229;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenBlockquote", G->buf+G->pos));
  return 1;
  l1229:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockAddress"));  if (!yy_HtmlBlockOpenAddress(G)) { goto l1234; }
  l1235:;	
  {  int yypos1236= G->pos, yythunkpos1236= G->thunkpos;
  {  int yypos1237= G->pos, yythunkpos1237= G->thunkpos;  if (!yy_HtmlBlockAddress(G)) { goto l1238; }  goto l1237;
  l1238:;	  G->pos= yypos1237; G->thunkpos= yythunkpos1237;
  {  int yypos1239= G->pos, yythunkpos1239= G->thunkpos;  if (!yy_HtmlBlockCloseAddress(G)) { goto l1239; }  goto l1236;
  l1239:;	  G->pos= yypos1239; G->thunkpos= yythunkpos1239;
  }  if (!yymatchDot(G)) goto l1236;
  }
  l1237:;	  goto l1235;
  l1236:;	  G->pos= yypos1236; G->thunkpos= yythunkpos1236;
  }  if (!yy_HtmlBlockCloseAddress(G)) { goto l1234; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockAddress", G->buf+G->pos));
  return 1;
  l1234:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseAddress"));  if (!yymatchChar(G, '<')) goto l1240;  if (!yy_Spnl(G)) { goto l1240; }  if (!yymatchChar(G, '/')) goto l1240;
  {  int yypos1241= G->pos, yythunkpos1241= G->thunkpos;  if (!yymatchString(G, "address")) goto l1242;  goto l1241;
  l1242:;	  G->pos= yypos1241; G->thunkpos= yythunkpos1241;  if (!yymatchString(G, "ADDRESS")) goto l1240;
  }
  l1241:;	  if (!yy_Spnl(G)) { goto l1240; }  if (!yymatchChar(G, '>')) goto l1240;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseAddress", G->buf+G->pos));
  return 1;
  l1240:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlAttribute(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlAttribute"));
  {  int yypos1246= G->pos, yythunkpos1246= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l1247; }  goto l1246;
  l1247:;	  G->pos= yypos1246; G->thunkpos= yythunkpos1246;  if (!yymatchChar(G, '-')) goto l1243;
  }
  l1246:;	
  l1244:;	
  {  int yypos1245= G->pos, yythunkpos1245= G->thunkpos;
  {  int yypos1248= G->pos, yythunkpos1248= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l1249; }  goto l1248;
  l1249:;	  G->pos= yypos1248; G->thunkpos= yythunkpos1248;  if (!yymatchChar(G, '-')) goto l1245;
  }
  l1248:;	  goto l1244;
  l1245:;	  G->pos= yypos1245; G->thunkpos= yythunkpos1245;
  }  if (!yy_Spnl(G)) { goto l1243; }
  {  int yypos1250= G->pos, yythunkpos1250= G->thunkpos;  if (!yymatchChar(G, '=')) goto l1250;  if (!yy_Spnl(G)) { goto l1250; }
  {  int yypos1252= G->pos, yythunkpos1252= G->thunkpos;  if (!yy_Quoted(G)) { goto l1253; }  goto l1252;
  l1253:;	  G->pos= yypos1252; G->thunkpos= yythunkpos1252;
  {  int yypos1256= G->pos, yythunkpos1256= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1256;  goto l1250;
  l1256:;	  G->pos= yypos1256; G->thunkpos= yythunkpos1256;
  }  if (!yy_Nonspacechar(G)) { goto l1250; }
  l1254:;	
  {  int yypos1255= G->pos, yythunkpos1255= G->thunkpos;
  {  int yypos1257= G->pos, yythunkpos1257= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1257;  goto l1255;
  l1257:;	  G->pos= yypos1257; G->thunkpos= yythunkpos1257;
  }  if (!yy_Nonspacechar(G)) { goto l1255; }  goto l1254;
  l1255:;	  G->pos= yypos1255; G->thunkpos= yythunkpos1255;
  }
  }
  l1252:;	  goto l1251;
  l1250:;	  G->pos= yypos1250; G->thunkpos= yythunkpos1250;
  }
  l1251:;	  if (!yy_Spnl(G)) { goto l1243; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlAttribute", G->buf+G->pos));
  return 1;
  l1243:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlAttribute", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenAddress"));  if (!yymatchChar(G, '<')) goto l1258;  if (!yy_Spnl(G)) { goto l1258; }
  {  int yypos1259= G->pos, yythunkpos1259= G->thunkpos;  if (!yymatchString(G, "address")) goto l1260;  goto l1259;
  l1260:;	  G->pos= yypos1259; G->thunkpos= yythunkpos1259;  if (!yymatchString(G, "ADDRESS")) goto l1258;
  }
  l1259:;	  if (!yy_Spnl(G)) { goto l1258; }
  l1261:;	
  {  int yypos1262= G->pos, yythunkpos1262= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1262; }  goto l1261;
  l1262:;	  G->pos= yypos1262; G->thunkpos= yythunkpos1262;
  }  if (!yymatchChar(G, '>')) goto l1258;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenAddress", G->buf+G->pos));
  return 1;
  l1258:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OptionallyIndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OptionallyIndentedLine"));
  {  int yypos1264= G->pos, yythunkpos1264= G->thunkpos;  if (!yy_Indent(G)) { goto l1264; }  goto l1265;
  l1264:;	  G->pos= yypos1264; G->thunkpos= yythunkpos1264;
  }
  l1265:;	  if (!yy_Line(G)) { goto l1263; }
  yyprintf((stderr, "  ok   %s @ %s\n", "OptionallyIndentedLine", G->buf+G->pos));
  return 1;
  l1263:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OptionallyIndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Indent(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Indent"));
  {  int yypos1267= G->pos, yythunkpos1267= G->thunkpos;  if (!yymatchChar(G, '\t')) goto l1268;  goto l1267;
  l1268:;	  G->pos= yypos1267; G->thunkpos= yythunkpos1267;  if (!yymatchString(G, "    ")) goto l1266;
  }
  l1267:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Indent", G->buf+G->pos));
  return 1;
  l1266:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Indent", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListBlockLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ListBlockLine"));
  {  int yypos1270= G->pos, yythunkpos1270= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1270; }  goto l1269;
  l1270:;	  G->pos= yypos1270; G->thunkpos= yythunkpos1270;
  }
  {  int yypos1271= G->pos, yythunkpos1271= G->thunkpos;
  {  int yypos1272= G->pos, yythunkpos1272= G->thunkpos;  if (!yy_Indent(G)) { goto l1272; }  goto l1273;
  l1272:;	  G->pos= yypos1272; G->thunkpos= yythunkpos1272;
  }
  l1273:;	
  {  int yypos1274= G->pos, yythunkpos1274= G->thunkpos;  if (!yy_Bullet(G)) { goto l1275; }  goto l1274;
  l1275:;	  G->pos= yypos1274; G->thunkpos= yythunkpos1274;  if (!yy_Enumerator(G)) { goto l1271; }
  }
  l1274:;	  goto l1269;
  l1271:;	  G->pos= yypos1271; G->thunkpos= yythunkpos1271;
  }
  {  int yypos1276= G->pos, yythunkpos1276= G->thunkpos;  if (!yy_HorizontalRule(G)) { goto l1276; }  goto l1269;
  l1276:;	  G->pos= yypos1276; G->thunkpos= yythunkpos1276;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l1269; }
  yyprintf((stderr, "  ok   %s @ %s\n", "ListBlockLine", G->buf+G->pos));
  return 1;
  l1269:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListBlockLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListContinuationBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListContinuationBlock"));  if (!yy_StartList(G)) { goto l1277; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1277;
  l1278:;	
  {  int yypos1279= G->pos, yythunkpos1279= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1279; }  goto l1278;
  l1279:;	  G->pos= yypos1279; G->thunkpos= yythunkpos1279;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1277;  yyDo(G, yy_1_ListContinuationBlock, G->begin, G->end);  if (!yy_Indent(G)) { goto l1277; }  if (!yy_ListBlock(G)) { goto l1277; }  yyDo(G, yy_2_ListContinuationBlock, G->begin, G->end);
  l1280:;	
  {  int yypos1281= G->pos, yythunkpos1281= G->thunkpos;  if (!yy_Indent(G)) { goto l1281; }  if (!yy_ListBlock(G)) { goto l1281; }  yyDo(G, yy_2_ListContinuationBlock, G->begin, G->end);  goto l1280;
  l1281:;	  G->pos= yypos1281; G->thunkpos= yythunkpos1281;
  }  yyDo(G, yy_3_ListContinuationBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListContinuationBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1277:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListContinuationBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListBlock"));  if (!yy_StartList(G)) { goto l1282; }  yyDo(G, yySet, -1, 0);
  {  int yypos1283= G->pos, yythunkpos1283= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1283; }  goto l1282;
  l1283:;	  G->pos= yypos1283; G->thunkpos= yythunkpos1283;
  }  if (!yy_Line(G)) { goto l1282; }  yyDo(G, yy_1_ListBlock, G->begin, G->end);
  l1284:;	
  {  int yypos1285= G->pos, yythunkpos1285= G->thunkpos;  if (!yy_ListBlockLine(G)) { goto l1285; }  yyDo(G, yy_2_ListBlock, G->begin, G->end);  goto l1284;
  l1285:;	  G->pos= yypos1285; G->thunkpos= yythunkpos1285;
  }  yyDo(G, yy_3_ListBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1282:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListItem(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListItem"));
  {  int yypos1287= G->pos, yythunkpos1287= G->thunkpos;  if (!yy_Bullet(G)) { goto l1288; }  goto l1287;
  l1288:;	  G->pos= yypos1287; G->thunkpos= yythunkpos1287;  if (!yy_Enumerator(G)) { goto l1286; }
  }
  l1287:;	  if (!yy_StartList(G)) { goto l1286; }  yyDo(G, yySet, -1, 0);  if (!yy_ListBlock(G)) { goto l1286; }  yyDo(G, yy_1_ListItem, G->begin, G->end);
  l1289:;	
  {  int yypos1290= G->pos, yythunkpos1290= G->thunkpos;  if (!yy_ListContinuationBlock(G)) { goto l1290; }  yyDo(G, yy_2_ListItem, G->begin, G->end);  goto l1289;
  l1290:;	  G->pos= yypos1290; G->thunkpos= yythunkpos1290;
  }  yyDo(G, yy_3_ListItem, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListItem", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1286:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListItem", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Enumerator(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Enumerator"));  if (!yy_NonindentSpace(G)) { goto l1291; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1291;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1291;
  l1292:;	
  {  int yypos1293= G->pos, yythunkpos1293= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1293;  goto l1292;
  l1293:;	  G->pos= yypos1293; G->thunkpos= yythunkpos1293;
  }  if (!yymatchChar(G, '.')) goto l1291;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1291;  if (!yy_Spacechar(G)) { goto l1291; }
  l1294:;	
  {  int yypos1295= G->pos, yythunkpos1295= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1295; }  goto l1294;
  l1295:;	  G->pos= yypos1295; G->thunkpos= yythunkpos1295;
  }  yyDo(G, yy_1_Enumerator, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Enumerator", G->buf+G->pos));
  return 1;
  l1291:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Enumerator", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListItemTight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListItemTight"));
  {  int yypos1297= G->pos, yythunkpos1297= G->thunkpos;  if (!yy_Bullet(G)) { goto l1298; }  goto l1297;
  l1298:;	  G->pos= yypos1297; G->thunkpos= yythunkpos1297;  if (!yy_Enumerator(G)) { goto l1296; }
  }
  l1297:;	  if (!yy_StartList(G)) { goto l1296; }  yyDo(G, yySet, -1, 0);  if (!yy_ListBlock(G)) { goto l1296; }  yyDo(G, yy_1_ListItemTight, G->begin, G->end);
  l1299:;	
  {  int yypos1300= G->pos, yythunkpos1300= G->thunkpos;
  {  int yypos1301= G->pos, yythunkpos1301= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1301; }  goto l1300;
  l1301:;	  G->pos= yypos1301; G->thunkpos= yythunkpos1301;
  }  if (!yy_ListContinuationBlock(G)) { goto l1300; }  yyDo(G, yy_2_ListItemTight, G->begin, G->end);  goto l1299;
  l1300:;	  G->pos= yypos1300; G->thunkpos= yythunkpos1300;
  }
  {  int yypos1302= G->pos, yythunkpos1302= G->thunkpos;  if (!yy_ListContinuationBlock(G)) { goto l1302; }  goto l1296;
  l1302:;	  G->pos= yypos1302; G->thunkpos= yythunkpos1302;
  }  yyDo(G, yy_3_ListItemTight, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListItemTight", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1296:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListItemTight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListLoose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ListLoose"));  if (!yy_StartList(G)) { goto l1303; }  yyDo(G, yySet, -2, 0);  if (!yy_ListItem(G)) { goto l1303; }  yyDo(G, yySet, -1, 0);
  l1306:;	
  {  int yypos1307= G->pos, yythunkpos1307= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1307; }  goto l1306;
  l1307:;	  G->pos= yypos1307; G->thunkpos= yythunkpos1307;
  }  yyDo(G, yy_1_ListLoose, G->begin, G->end);
  l1304:;	
  {  int yypos1305= G->pos, yythunkpos1305= G->thunkpos;  if (!yy_ListItem(G)) { goto l1305; }  yyDo(G, yySet, -1, 0);
  l1308:;	
  {  int yypos1309= G->pos, yythunkpos1309= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1309; }  goto l1308;
  l1309:;	  G->pos= yypos1309; G->thunkpos= yythunkpos1309;
  }  yyDo(G, yy_1_ListLoose, G->begin, G->end);  goto l1304;
  l1305:;	  G->pos= yypos1305; G->thunkpos= yythunkpos1305;
  }  yyDo(G, yy_2_ListLoose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListLoose", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l1303:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListLoose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListTight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListTight"));  if (!yy_StartList(G)) { goto l1310; }  yyDo(G, yySet, -1, 0);  if (!yy_ListItemTight(G)) { goto l1310; }  yyDo(G, yy_1_ListTight, G->begin, G->end);
  l1311:;	
  {  int yypos1312= G->pos, yythunkpos1312= G->thunkpos;  if (!yy_ListItemTight(G)) { goto l1312; }  yyDo(G, yy_1_ListTight, G->begin, G->end);  goto l1311;
  l1312:;	  G->pos= yypos1312; G->thunkpos= yythunkpos1312;
  }
  l1313:;	
  {  int yypos1314= G->pos, yythunkpos1314= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1314; }  goto l1313;
  l1314:;	  G->pos= yypos1314; G->thunkpos= yythunkpos1314;
  }
  {  int yypos1315= G->pos, yythunkpos1315= G->thunkpos;
  {  int yypos1316= G->pos, yythunkpos1316= G->thunkpos;  if (!yy_Bullet(G)) { goto l1317; }  goto l1316;
  l1317:;	  G->pos= yypos1316; G->thunkpos= yythunkpos1316;  if (!yy_Enumerator(G)) { goto l1315; }
  }
  l1316:;	  goto l1310;
  l1315:;	  G->pos= yypos1315; G->thunkpos= yythunkpos1315;
  }  yyDo(G, yy_2_ListTight, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListTight", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1310:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListTight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Bullet(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Bullet"));
  {  int yypos1319= G->pos, yythunkpos1319= G->thunkpos;  if (!yy_HorizontalRule(G)) { goto l1319; }  goto l1318;
  l1319:;	  G->pos= yypos1319; G->thunkpos= yythunkpos1319;
  }  if (!yy_NonindentSpace(G)) { goto l1318; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1318;
  {  int yypos1320= G->pos, yythunkpos1320= G->thunkpos;  if (!yymatchChar(G, '+')) goto l1321;  goto l1320;
  l1321:;	  G->pos= yypos1320; G->thunkpos= yythunkpos1320;  if (!yymatchChar(G, '*')) goto l1322;  goto l1320;
  l1322:;	  G->pos= yypos1320; G->thunkpos= yythunkpos1320;  if (!yymatchChar(G, '-')) goto l1318;
  }
  l1320:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1318;  if (!yy_Spacechar(G)) { goto l1318; }
  l1323:;	
  {  int yypos1324= G->pos, yythunkpos1324= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1324; }  goto l1323;
  l1324:;	  G->pos= yypos1324; G->thunkpos= yythunkpos1324;
  }  yyDo(G, yy_1_Bullet, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Bullet", G->buf+G->pos));
  return 1;
  l1318:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Bullet", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableCell(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableCell"));  if (!yy_Sp(G)) { goto l1325; }
  l1326:;	
  {  int yypos1327= G->pos, yythunkpos1327= G->thunkpos;
  {  int yypos1328= G->pos, yythunkpos1328= G->thunkpos;  if (!yymatchChar(G, '|')) goto l1328;  goto l1327;
  l1328:;	  G->pos= yypos1328; G->thunkpos= yythunkpos1328;
  }
  {  int yypos1329= G->pos, yythunkpos1329= G->thunkpos;  if (!yy_Newline(G)) { goto l1329; }  goto l1327;
  l1329:;	  G->pos= yypos1329; G->thunkpos= yythunkpos1329;
  }  if (!yy_Inline(G)) { goto l1327; }  goto l1326;
  l1327:;	  G->pos= yypos1327; G->thunkpos= yythunkpos1327;
  }  if (!yy_TableBorder(G)) { goto l1325; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TableCell", G->buf+G->pos));
  return 1;
  l1325:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableCell", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableBorder(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableBorder"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1330;  if (!yymatchChar(G, '|')) goto l1330;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1330;  yyDo(G, yy_1_TableBorder, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TableBorder", G->buf+G->pos));
  return 1;
  l1330:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableBorder", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableLine"));  if (!yy_TableBorder(G)) { goto l1331; }  if (!yy_TableCell(G)) { goto l1331; }
  l1332:;	
  {  int yypos1333= G->pos, yythunkpos1333= G->thunkpos;  if (!yy_TableCell(G)) { goto l1333; }  goto l1332;
  l1333:;	  G->pos= yypos1333; G->thunkpos= yythunkpos1333;
  }  if (!yy_Sp(G)) { goto l1331; }  if (!yy_Newline(G)) { goto l1331; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TableLine", G->buf+G->pos));
  return 1;
  l1331:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableDelimiter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableDelimiter"));  if (!yy_TableBorder(G)) { goto l1334; }  if (!yy_Sp(G)) { goto l1334; }
  {  int yypos1337= G->pos, yythunkpos1337= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1337;  goto l1338;
  l1337:;	  G->pos= yypos1337; G->thunkpos= yythunkpos1337;
  }
  l1338:;	  if (!yymatchChar(G, '-')) goto l1334;
  l1339:;	
  {  int yypos1340= G->pos, yythunkpos1340= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1340;  goto l1339;
  l1340:;	  G->pos= yypos1340; G->thunkpos= yythunkpos1340;
  }
  {  int yypos1341= G->pos, yythunkpos1341= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1341;  goto l1342;
  l1341:;	  G->pos= yypos1341; G->thunkpos= yythunkpos1341;
  }
  l1342:;	  if (!yy_Sp(G)) { goto l1334; }  if (!yy_TableBorder(G)) { goto l1334; }
  l1335:;	
  {  int yypos1336= G->pos, yythunkpos1336= G->thunkpos;  if (!yy_Sp(G)) { goto l1336; }
  {  int yypos1343= G->pos, yythunkpos1343= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1343;  goto l1344;
  l1343:;	  G->pos= yypos1343; G->thunkpos= yythunkpos1343;
  }
  l1344:;	  if (!yymatchChar(G, '-')) goto l1336;
  l1345:;	
  {  int yypos1346= G->pos, yythunkpos1346= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1346;  goto l1345;
  l1346:;	  G->pos= yypos1346; G->thunkpos= yythunkpos1346;
  }
  {  int yypos1347= G->pos, yythunkpos1347= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1347;  goto l1348;
  l1347:;	  G->pos= yypos1347; G->thunkpos= yythunkpos1347;
  }
  l1348:;	  if (!yy_Sp(G)) { goto l1336; }  if (!yy_TableBorder(G)) { goto l1336; }  goto l1335;
  l1336:;	  G->pos= yypos1336; G->thunkpos= yythunkpos1336;
  }  if (!yy_Sp(G)) { goto l1334; }  if (!yy_Newline(G)) { goto l1334; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TableDelimiter", G->buf+G->pos));
  return 1;
  l1334:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableDelimiter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableHeader(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "TableHeader"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1349;  if (!yy_LocMarker(G)) { goto l1349; }  yyDo(G, yySet, -1, 0);  if (!yy_TableLine(G)) { goto l1349; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1349;  yyDo(G, yy_1_TableHeader, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TableHeader", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1349:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableHeader", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineEquationMultiple(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineEquationMultiple"));
  {  int yypos1351= G->pos, yythunkpos1351= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1351;  goto l1350;
  l1351:;	  G->pos= yypos1351; G->thunkpos= yythunkpos1351;
  }  if (!yy_Nonspacechar(G)) { goto l1350; }
  {  int yypos1354= G->pos, yythunkpos1354= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1354;  goto l1350;
  l1354:;	  G->pos= yypos1354; G->thunkpos= yythunkpos1354;
  }
  {  int yypos1355= G->pos, yythunkpos1355= G->thunkpos;  if (!yy_Newline(G)) { goto l1355; }  goto l1350;
  l1355:;	  G->pos= yypos1355; G->thunkpos= yythunkpos1355;
  }  if (!yymatchDot(G)) goto l1350;
  l1352:;	
  {  int yypos1353= G->pos, yythunkpos1353= G->thunkpos;
  {  int yypos1356= G->pos, yythunkpos1356= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1356;  goto l1353;
  l1356:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;
  }
  {  int yypos1357= G->pos, yythunkpos1357= G->thunkpos;  if (!yy_Newline(G)) { goto l1357; }  goto l1353;
  l1357:;	  G->pos= yypos1357; G->thunkpos= yythunkpos1357;
  }  if (!yymatchDot(G)) goto l1353;  goto l1352;
  l1353:;	  G->pos= yypos1353; G->thunkpos= yythunkpos1353;
  }  if (!yymatchChar(G, '$')) goto l1350;  yyText(G, G->begin, G->end);  if (!( IEP_POST )) goto l1350;
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineEquationMultiple", G->buf+G->pos));
  return 1;
  l1350:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineEquationMultiple", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineEquationSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineEquationSingle"));
  {  int yypos1359= G->pos, yythunkpos1359= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1359;  goto l1358;
  l1359:;	  G->pos= yypos1359; G->thunkpos= yythunkpos1359;
  }
  {  int yypos1360= G->pos, yythunkpos1360= G->thunkpos;  if (!yymatchChar(G, '\\')) goto l1360;  goto l1358;
  l1360:;	  G->pos= yypos1360; G->thunkpos= yythunkpos1360;
  }  if (!yy_Nonspacechar(G)) { goto l1358; }  if (!yymatchChar(G, '$')) goto l1358;
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineEquationSingle", G->buf+G->pos));
  return 1;
  l1358:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineEquationSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineEquation(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineEquation"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH) )) goto l1361;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1361;  if (!yymatchChar(G, '$')) goto l1361;  yyText(G, G->begin, G->end);  if (!( IEP_PRE )) goto l1361;
  {  int yypos1362= G->pos, yythunkpos1362= G->thunkpos;  if (!yy_InlineEquationSingle(G)) { goto l1363; }  goto l1362;
  l1363:;	  G->pos= yypos1362; G->thunkpos= yythunkpos1362;  if (!yy_InlineEquationMultiple(G)) { goto l1361; }
  }
  l1362:;	
  {  int yypos1364= G->pos, yythunkpos1364= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1364;  goto l1361;
  l1364:;	  G->pos= yypos1364; G->thunkpos= yythunkpos1364;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1361;  yyDo(G, yy_1_InlineEquation, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineEquation", G->buf+G->pos));
  return 1;
  l1361:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineEquation", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Nonspacechar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Nonspacechar"));
  {  int yypos1366= G->pos, yythunkpos1366= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1366; }  goto l1365;
  l1366:;	  G->pos= yypos1366; G->thunkpos= yythunkpos1366;
  }
  {  int yypos1367= G->pos, yythunkpos1367= G->thunkpos;  if (!yy_Newline(G)) { goto l1367; }  goto l1365;
  l1367:;	  G->pos= yypos1367; G->thunkpos= yythunkpos1367;
  }  if (!yymatchDot(G)) goto l1365;
  yyprintf((stderr, "  ok   %s @ %s\n", "Nonspacechar", G->buf+G->pos));
  return 1;
  l1365:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Nonspacechar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRawMark(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRawMark"));
  {  int yypos1371= G->pos, yythunkpos1371= G->thunkpos;  if (!yymatchChar(G, '{')) goto l1371;  goto l1368;
  l1371:;	  G->pos= yypos1371; G->thunkpos= yythunkpos1371;
  }
  {  int yypos1372= G->pos, yythunkpos1372= G->thunkpos;  if (!yymatchChar(G, '}')) goto l1372;  goto l1368;
  l1372:;	  G->pos= yypos1372; G->thunkpos= yythunkpos1372;
  }  if (!yy_Nonspacechar(G)) { goto l1368; }
  l1369:;	
  {  int yypos1370= G->pos, yythunkpos1370= G->thunkpos;
  {  int yypos1373= G->pos, yythunkpos1373= G->thunkpos;  if (!yymatchChar(G, '{')) goto l1373;  goto l1370;
  l1373:;	  G->pos= yypos1373; G->thunkpos= yythunkpos1373;
  }
  {  int yypos1374= G->pos, yythunkpos1374= G->thunkpos;  if (!yymatchChar(G, '}')) goto l1374;  goto l1370;
  l1374:;	  G->pos= yypos1374; G->thunkpos= yythunkpos1374;
  }  if (!yy_Nonspacechar(G)) { goto l1370; }  goto l1369;
  l1370:;	  G->pos= yypos1370; G->thunkpos= yythunkpos1370;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRawMark", G->buf+G->pos));
  return 1;
  l1368:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRawMark", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRawEnd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRawEnd"));  if (!yymatchString(G, "\\end")) goto l1375;  if (!yymatchChar(G, '{')) goto l1375;  if (!yy_DisplayFormulaRawMark(G)) { goto l1375; }  if (!yymatchChar(G, '}')) goto l1375;
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRawEnd", G->buf+G->pos));
  return 1;
  l1375:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRawEnd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Spnl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Spnl"));  if (!yy_Sp(G)) { goto l1376; }
  {  int yypos1377= G->pos, yythunkpos1377= G->thunkpos;  if (!yy_Newline(G)) { goto l1377; }  if (!yy_Sp(G)) { goto l1377; }  goto l1378;
  l1377:;	  G->pos= yypos1377; G->thunkpos= yythunkpos1377;
  }
  l1378:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Spnl", G->buf+G->pos));
  return 1;
  l1376:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Spnl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRawStart(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRawStart"));  if (!yymatchString(G, "\\begin")) goto l1379;  if (!yymatchChar(G, '{')) goto l1379;  if (!yy_DisplayFormulaRawMark(G)) { goto l1379; }  if (!yymatchChar(G, '}')) goto l1379;
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRawStart", G->buf+G->pos));
  return 1;
  l1379:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRawStart", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FormulaNumber(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FormulaNumber"));  if (!yymatchChar(G, '(')) goto l1380;
  {  int yypos1383= G->pos, yythunkpos1383= G->thunkpos;  if (!yymatchChar(G, ')')) goto l1383;  goto l1380;
  l1383:;	  G->pos= yypos1383; G->thunkpos= yythunkpos1383;
  }
  {  int yypos1384= G->pos, yythunkpos1384= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1384;  goto l1380;
  l1384:;	  G->pos= yypos1384; G->thunkpos= yythunkpos1384;
  }
  {  int yypos1385= G->pos, yythunkpos1385= G->thunkpos;  if (!yy_Newline(G)) { goto l1385; }  goto l1380;
  l1385:;	  G->pos= yypos1385; G->thunkpos= yythunkpos1385;
  }  if (!yymatchDot(G)) goto l1380;
  l1381:;	
  {  int yypos1382= G->pos, yythunkpos1382= G->thunkpos;
  {  int yypos1386= G->pos, yythunkpos1386= G->thunkpos;  if (!yymatchChar(G, ')')) goto l1386;  goto l1382;
  l1386:;	  G->pos= yypos1386; G->thunkpos= yythunkpos1386;
  }
  {  int yypos1387= G->pos, yythunkpos1387= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1387;  goto l1382;
  l1387:;	  G->pos= yypos1387; G->thunkpos= yythunkpos1387;
  }
  {  int yypos1388= G->pos, yythunkpos1388= G->thunkpos;  if (!yy_Newline(G)) { goto l1388; }  goto l1382;
  l1388:;	  G->pos= yypos1388; G->thunkpos= yythunkpos1388;
  }  if (!yymatchDot(G)) goto l1382;  goto l1381;
  l1382:;	  G->pos= yypos1382; G->thunkpos= yythunkpos1382;
  }  if (!yymatchChar(G, ')')) goto l1380;
  yyprintf((stderr, "  ok   %s @ %s\n", "FormulaNumber", G->buf+G->pos));
  return 1;
  l1380:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FormulaNumber", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRaw(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRaw"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH_RAW) )) goto l1389;  if (!yy_NonindentSpace(G)) { goto l1389; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1389;  if (!yy_DisplayFormulaRawStart(G)) { goto l1389; }  yyText(G, G->begin, G->end);  if (!( start_dfr() )) goto l1389;  if (!yy_Spnl(G)) { goto l1389; }
  l1390:;	
  {  int yypos1391= G->pos, yythunkpos1391= G->thunkpos;
  {  int yypos1392= G->pos, yythunkpos1392= G->thunkpos;  if (!yy_DisplayFormulaRawStart(G)) { goto l1393; }  yyText(G, G->begin, G->end);  if (!( inc_dfr() )) goto l1393;  goto l1392;
  l1393:;	  G->pos= yypos1392; G->thunkpos= yythunkpos1392;  yyText(G, G->begin, G->end);  if (!( nested_dfr() )) goto l1394;  if (!yy_DisplayFormulaRawEnd(G)) { goto l1394; }  yyText(G, G->begin, G->end);  if (!( dec_dfr() )) goto l1394;  goto l1392;
  l1394:;	  G->pos= yypos1392; G->thunkpos= yythunkpos1392;
  {  int yypos1395= G->pos, yythunkpos1395= G->thunkpos;  if (!yy_DisplayFormulaRawEnd(G)) { goto l1395; }  goto l1391;
  l1395:;	  G->pos= yypos1395; G->thunkpos= yythunkpos1395;
  }
  {  int yypos1396= G->pos, yythunkpos1396= G->thunkpos;  if (!yy_DisplayFormulaRawStart(G)) { goto l1396; }  goto l1391;
  l1396:;	  G->pos= yypos1396; G->thunkpos= yythunkpos1396;
  }
  {  int yypos1397= G->pos, yythunkpos1397= G->thunkpos;  if (!yy_Newline(G)) { goto l1397; }  if (!yy_Newline(G)) { goto l1397; }
  l1398:;	
  {  int yypos1399= G->pos, yythunkpos1399= G->thunkpos;  if (!yy_Newline(G)) { goto l1399; }  goto l1398;
  l1399:;	  G->pos= yypos1399; G->thunkpos= yythunkpos1399;
  }  goto l1391;
  l1397:;	  G->pos= yypos1397; G->thunkpos= yythunkpos1397;
  }  if (!yymatchDot(G)) goto l1391;
  }
  l1392:;	  goto l1390;
  l1391:;	  G->pos= yypos1391; G->thunkpos= yythunkpos1391;
  }  yyText(G, G->begin, G->end);  if (!( !nested_dfr() )) goto l1389;  if (!yy_DisplayFormulaRawEnd(G)) { goto l1389; }  yyText(G, G->begin, G->end);  if (!( dec_dfr() )) goto l1389;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1389;  if (!yy_Sp(G)) { goto l1389; }  if (!yy_Newline(G)) { goto l1389; }  yyDo(G, yy_1_DisplayFormulaRaw, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRaw", G->buf+G->pos));
  return 1;
  l1389:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRaw", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaDollar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaDollar"));  if (!yy_NonindentSpace(G)) { goto l1400; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1400;  if (!yymatchString(G, "$$")) goto l1400;
  l1401:;	
  {  int yypos1402= G->pos, yythunkpos1402= G->thunkpos;
  {  int yypos1403= G->pos, yythunkpos1403= G->thunkpos;  if (!yymatchString(G, "$$")) goto l1403;  goto l1402;
  l1403:;	  G->pos= yypos1403; G->thunkpos= yythunkpos1403;
  }  if (!yymatchDot(G)) goto l1402;  goto l1401;
  l1402:;	  G->pos= yypos1402; G->thunkpos= yythunkpos1402;
  }  if (!yymatchString(G, "$$")) goto l1400;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1400;  if (!yy_Sp(G)) { goto l1400; }
  {  int yypos1404= G->pos, yythunkpos1404= G->thunkpos;  if (!yy_FormulaNumber(G)) { goto l1404; }  goto l1405;
  l1404:;	  G->pos= yypos1404; G->thunkpos= yythunkpos1404;
  }
  l1405:;	  if (!yy_Sp(G)) { goto l1400; }  if (!yy_Newline(G)) { goto l1400; }  yyDo(G, yy_1_DisplayFormulaDollar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaDollar", G->buf+G->pos));
  return 1;
  l1400:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaDollar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockTidle"));  if (!yy_FencedCodeBlockStartTidle(G)) { goto l1406; }
  l1407:;	
  {  int yypos1408= G->pos, yythunkpos1408= G->thunkpos;  if (!yy_FencedCodeBlockChunkTidle(G)) { goto l1408; }  goto l1407;
  l1408:;	  G->pos= yypos1408; G->thunkpos= yythunkpos1408;
  }  if (!yy_FencedCodeBlockEndTidle(G)) { goto l1406; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockTidle", G->buf+G->pos));
  return 1;
  l1406:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockTick"));  if (!yy_FencedCodeBlockStartTick(G)) { goto l1409; }
  l1410:;	
  {  int yypos1411= G->pos, yythunkpos1411= G->thunkpos;  if (!yy_FencedCodeBlockChunkTick(G)) { goto l1411; }  goto l1410;
  l1411:;	  G->pos= yypos1411; G->thunkpos= yythunkpos1411;
  }  if (!yy_FencedCodeBlockEndTick(G)) { goto l1409; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockTick", G->buf+G->pos));
  return 1;
  l1409:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockEndTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockEndTidle"));  if (!yymatchString(G, "~~~")) goto l1412;
  l1413:;	
  {  int yypos1414= G->pos, yythunkpos1414= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1414; }  goto l1413;
  l1414:;	  G->pos= yypos1414; G->thunkpos= yythunkpos1414;
  }  if (!yy_Newline(G)) { goto l1412; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockEndTidle", G->buf+G->pos));
  return 1;
  l1412:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockEndTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockChunkTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockChunkTidle"));
  {  int yypos1416= G->pos, yythunkpos1416= G->thunkpos;  if (!yy_FencedCodeBlockEndTidle(G)) { goto l1416; }  goto l1415;
  l1416:;	  G->pos= yypos1416; G->thunkpos= yythunkpos1416;
  }
  l1417:;	
  {  int yypos1418= G->pos, yythunkpos1418= G->thunkpos;
  {  int yypos1419= G->pos, yythunkpos1419= G->thunkpos;  if (!yy_Newline(G)) { goto l1419; }  goto l1418;
  l1419:;	  G->pos= yypos1419; G->thunkpos= yythunkpos1419;
  }  if (!yymatchDot(G)) goto l1418;  goto l1417;
  l1418:;	  G->pos= yypos1418; G->thunkpos= yythunkpos1418;
  }  if (!yy_Newline(G)) { goto l1415; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockChunkTidle", G->buf+G->pos));
  return 1;
  l1415:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockChunkTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockStartTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockStartTidle"));  if (!yymatchString(G, "~~~")) goto l1420;
  l1421:;	
  {  int yypos1422= G->pos, yythunkpos1422= G->thunkpos;
  {  int yypos1423= G->pos, yythunkpos1423= G->thunkpos;  if (!yy_Newline(G)) { goto l1423; }  goto l1422;
  l1423:;	  G->pos= yypos1423; G->thunkpos= yythunkpos1423;
  }
  {  int yypos1424= G->pos, yythunkpos1424= G->thunkpos;  if (!yymatchChar(G, '~')) goto l1424;  goto l1422;
  l1424:;	  G->pos= yypos1424; G->thunkpos= yythunkpos1424;
  }  if (!yymatchDot(G)) goto l1422;  goto l1421;
  l1422:;	  G->pos= yypos1422; G->thunkpos= yythunkpos1422;
  }  if (!yy_Newline(G)) { goto l1420; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockStartTidle", G->buf+G->pos));
  return 1;
  l1420:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockStartTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Spacechar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Spacechar"));
  {  int yypos1426= G->pos, yythunkpos1426= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1427;  goto l1426;
  l1427:;	  G->pos= yypos1426; G->thunkpos= yythunkpos1426;  if (!yymatchChar(G, '\t')) goto l1425;
  }
  l1426:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Spacechar", G->buf+G->pos));
  return 1;
  l1425:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Spacechar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockEndTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockEndTick"));  if (!yymatchString(G, "```")) goto l1428;
  l1429:;	
  {  int yypos1430= G->pos, yythunkpos1430= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1430; }  goto l1429;
  l1430:;	  G->pos= yypos1430; G->thunkpos= yythunkpos1430;
  }  if (!yy_Newline(G)) { goto l1428; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockEndTick", G->buf+G->pos));
  return 1;
  l1428:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockEndTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockChunkTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockChunkTick"));
  {  int yypos1432= G->pos, yythunkpos1432= G->thunkpos;  if (!yy_FencedCodeBlockEndTick(G)) { goto l1432; }  goto l1431;
  l1432:;	  G->pos= yypos1432; G->thunkpos= yythunkpos1432;
  }
  l1433:;	
  {  int yypos1434= G->pos, yythunkpos1434= G->thunkpos;
  {  int yypos1435= G->pos, yythunkpos1435= G->thunkpos;  if (!yy_Newline(G)) { goto l1435; }  goto l1434;
  l1435:;	  G->pos= yypos1435; G->thunkpos= yythunkpos1435;
  }  if (!yymatchDot(G)) goto l1434;  goto l1433;
  l1434:;	  G->pos= yypos1434; G->thunkpos= yythunkpos1434;
  }  if (!yy_Newline(G)) { goto l1431; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockChunkTick", G->buf+G->pos));
  return 1;
  l1431:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockChunkTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockStartTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockStartTick"));  if (!yymatchString(G, "```")) goto l1436;
  l1437:;	
  {  int yypos1438= G->pos, yythunkpos1438= G->thunkpos;
  {  int yypos1439= G->pos, yythunkpos1439= G->thunkpos;  if (!yy_Newline(G)) { goto l1439; }  goto l1438;
  l1439:;	  G->pos= yypos1439; G->thunkpos= yythunkpos1439;
  }
  {  int yypos1440= G->pos, yythunkpos1440= G->thunkpos;  if (!yymatchChar(G, '`')) goto l1440;  goto l1438;
  l1440:;	  G->pos= yypos1440; G->thunkpos= yythunkpos1440;
  }  if (!yymatchDot(G)) goto l1438;  goto l1437;
  l1438:;	  G->pos= yypos1438; G->thunkpos= yythunkpos1438;
  }  if (!yy_Newline(G)) { goto l1436; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockStartTick", G->buf+G->pos));
  return 1;
  l1436:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockStartTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_VerbatimChunk(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "VerbatimChunk"));
  l1442:;	
  {  int yypos1443= G->pos, yythunkpos1443= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1443; }  goto l1442;
  l1443:;	  G->pos= yypos1443; G->thunkpos= yythunkpos1443;
  }  if (!yy_NonblankIndentedLine(G)) { goto l1441; }
  l1444:;	
  {  int yypos1445= G->pos, yythunkpos1445= G->thunkpos;  if (!yy_NonblankIndentedLine(G)) { goto l1445; }  goto l1444;
  l1445:;	  G->pos= yypos1445; G->thunkpos= yythunkpos1445;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "VerbatimChunk", G->buf+G->pos));
  return 1;
  l1441:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "VerbatimChunk", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_IndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "IndentedLine"));  if (!yy_Indent(G)) { goto l1446; }  if (!yy_Line(G)) { goto l1446; }
  yyprintf((stderr, "  ok   %s @ %s\n", "IndentedLine", G->buf+G->pos));
  return 1;
  l1446:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "IndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NonblankIndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NonblankIndentedLine"));
  {  int yypos1448= G->pos, yythunkpos1448= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1448; }  goto l1447;
  l1448:;	  G->pos= yypos1448; G->thunkpos= yythunkpos1448;
  }  if (!yy_IndentedLine(G)) { goto l1447; }
  yyprintf((stderr, "  ok   %s @ %s\n", "NonblankIndentedLine", G->buf+G->pos));
  return 1;
  l1447:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NonblankIndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Line(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Line"));  if (!yy_RawLine(G)) { goto l1449; }  yyDo(G, yy_1_Line, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Line", G->buf+G->pos));
  return 1;
  l1449:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Line", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StartList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StartList"));
  {  int yypos1451= G->pos, yythunkpos1451= G->thunkpos;  if (!yymatchDot(G)) goto l1450;  G->pos= yypos1451; G->thunkpos= yythunkpos1451;
  }  yyDo(G, yy_1_StartList, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StartList", G->buf+G->pos));
  return 1;
  l1450:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StartList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlockQuoteRaw(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "BlockQuoteRaw"));  if (!yy_StartList(G)) { goto l1452; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1452;  if (!yymatchChar(G, '>')) goto l1452;
  {  int yypos1455= G->pos, yythunkpos1455= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1455;  goto l1456;
  l1455:;	  G->pos= yypos1455; G->thunkpos= yythunkpos1455;
  }
  l1456:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1452;  yyDo(G, yy_1_BlockQuoteRaw, G->begin, G->end);  if (!yy_Line(G)) { goto l1452; }  yyDo(G, yy_2_BlockQuoteRaw, G->begin, G->end);
  l1457:;	
  {  int yypos1458= G->pos, yythunkpos1458= G->thunkpos;
  {  int yypos1459= G->pos, yythunkpos1459= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1459;  goto l1458;
  l1459:;	  G->pos= yypos1459; G->thunkpos= yythunkpos1459;
  }
  {  int yypos1460= G->pos, yythunkpos1460= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1460; }  goto l1458;
  l1460:;	  G->pos= yypos1460; G->thunkpos= yythunkpos1460;
  }  if (!yy_Line(G)) { goto l1458; }  yyDo(G, yy_3_BlockQuoteRaw, G->begin, G->end);  goto l1457;
  l1458:;	  G->pos= yypos1458; G->thunkpos= yythunkpos1458;
  }
  l1461:;	
  {  int yypos1462= G->pos, yythunkpos1462= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1462;  if (!yy_BlankLine(G)) { goto l1462; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1462;  yyDo(G, yy_4_BlockQuoteRaw, G->begin, G->end);  goto l1461;
  l1462:;	  G->pos= yypos1462; G->thunkpos= yythunkpos1462;
  }
  l1453:;	
  {  int yypos1454= G->pos, yythunkpos1454= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1454;  if (!yymatchChar(G, '>')) goto l1454;
  {  int yypos1463= G->pos, yythunkpos1463= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1463;  goto l1464;
  l1463:;	  G->pos= yypos1463; G->thunkpos= yythunkpos1463;
  }
  l1464:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1454;  yyDo(G, yy_1_BlockQuoteRaw, G->begin, G->end);  if (!yy_Line(G)) { goto l1454; }  yyDo(G, yy_2_BlockQuoteRaw, G->begin, G->end);
  l1465:;	
  {  int yypos1466= G->pos, yythunkpos1466= G->thunkpos;
  {  int yypos1467= G->pos, yythunkpos1467= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1467;  goto l1466;
  l1467:;	  G->pos= yypos1467; G->thunkpos= yythunkpos1467;
  }
  {  int yypos1468= G->pos, yythunkpos1468= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1468; }  goto l1466;
  l1468:;	  G->pos= yypos1468; G->thunkpos= yythunkpos1468;
  }  if (!yy_Line(G)) { goto l1466; }  yyDo(G, yy_3_BlockQuoteRaw, G->begin, G->end);  goto l1465;
  l1466:;	  G->pos= yypos1466; G->thunkpos= yythunkpos1466;
  }
  l1469:;	
  {  int yypos1470= G->pos, yythunkpos1470= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1470;  if (!yy_BlankLine(G)) { goto l1470; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1470;  yyDo(G, yy_4_BlockQuoteRaw, G->begin, G->end);  goto l1469;
  l1470:;	  G->pos= yypos1470; G->thunkpos= yythunkpos1470;
  }  goto l1453;
  l1454:;	  G->pos= yypos1454; G->thunkpos= yythunkpos1454;
  }  yyDo(G, yy_5_BlockQuoteRaw, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "BlockQuoteRaw", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1452:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlockQuoteRaw", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Endline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Endline"));
  {  int yypos1472= G->pos, yythunkpos1472= G->thunkpos;  if (!yy_LineBreak(G)) { goto l1473; }  goto l1472;
  l1473:;	  G->pos= yypos1472; G->thunkpos= yythunkpos1472;  if (!yy_TerminalEndline(G)) { goto l1474; }  goto l1472;
  l1474:;	  G->pos= yypos1472; G->thunkpos= yythunkpos1472;  if (!yy_NormalEndline(G)) { goto l1471; }
  }
  l1472:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Endline", G->buf+G->pos));
  return 1;
  l1471:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Endline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawLine"));
  {  int yypos1476= G->pos, yythunkpos1476= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1477;
  l1478:;	
  {  int yypos1479= G->pos, yythunkpos1479= G->thunkpos;
  {  int yypos1480= G->pos, yythunkpos1480= G->thunkpos;  if (!yymatchChar(G, '\r')) goto l1480;  goto l1479;
  l1480:;	  G->pos= yypos1480; G->thunkpos= yythunkpos1480;
  }
  {  int yypos1481= G->pos, yythunkpos1481= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1481;  goto l1479;
  l1481:;	  G->pos= yypos1481; G->thunkpos= yythunkpos1481;
  }  if (!yymatchDot(G)) goto l1479;  goto l1478;
  l1479:;	  G->pos= yypos1479; G->thunkpos= yythunkpos1479;
  }  if (!yy_Newline(G)) { goto l1477; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1477;  goto l1476;
  l1477:;	  G->pos= yypos1476; G->thunkpos= yythunkpos1476;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1475;  if (!yymatchDot(G)) goto l1475;
  l1482:;	
  {  int yypos1483= G->pos, yythunkpos1483= G->thunkpos;  if (!yymatchDot(G)) goto l1483;  goto l1482;
  l1483:;	  G->pos= yypos1483; G->thunkpos= yythunkpos1483;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1475;  if (!yy_Eof(G)) { goto l1475; }
  }
  l1476:;	  yyDo(G, yy_1_RawLine, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RawLine", G->buf+G->pos));
  return 1;
  l1475:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextBottom2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextBottom2"));  if (!yymatchChar(G, '-')) goto l1484;
  l1485:;	
  {  int yypos1486= G->pos, yythunkpos1486= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1486;  goto l1485;
  l1486:;	  G->pos= yypos1486; G->thunkpos= yythunkpos1486;
  }  if (!yy_Newline(G)) { goto l1484; }
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextBottom2", G->buf+G->pos));
  return 1;
  l1484:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextBottom2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextBottom1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextBottom1"));  if (!yymatchChar(G, '=')) goto l1487;
  l1488:;	
  {  int yypos1489= G->pos, yythunkpos1489= G->thunkpos;  if (!yymatchChar(G, '=')) goto l1489;  goto l1488;
  l1489:;	  G->pos= yypos1489; G->thunkpos= yythunkpos1489;
  }  if (!yy_Newline(G)) { goto l1487; }
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextBottom1", G->buf+G->pos));
  return 1;
  l1487:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextBottom1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "SetextHeading2"));
  {  int yypos1491= G->pos, yythunkpos1491= G->thunkpos;  if (!yy_RawLine(G)) { goto l1490; }  if (!yy_SetextBottom2(G)) { goto l1490; }  G->pos= yypos1491; G->thunkpos= yythunkpos1491;
  }  if (!yy_LocMarker(G)) { goto l1490; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1490;
  {  int yypos1494= G->pos, yythunkpos1494= G->thunkpos;  if (!yy_Endline(G)) { goto l1494; }  goto l1490;
  l1494:;	  G->pos= yypos1494; G->thunkpos= yythunkpos1494;
  }  if (!yy_Inline(G)) { goto l1490; }
  l1492:;	
  {  int yypos1493= G->pos, yythunkpos1493= G->thunkpos;
  {  int yypos1495= G->pos, yythunkpos1495= G->thunkpos;  if (!yy_Endline(G)) { goto l1495; }  goto l1493;
  l1495:;	  G->pos= yypos1495; G->thunkpos= yythunkpos1495;
  }  if (!yy_Inline(G)) { goto l1493; }  goto l1492;
  l1493:;	  G->pos= yypos1493; G->thunkpos= yythunkpos1493;
  }  if (!yy_Sp(G)) { goto l1490; }  if (!yy_Newline(G)) { goto l1490; }  if (!yy_SetextBottom2(G)) { goto l1490; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1490;  yyDo(G, yy_1_SetextHeading2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading2", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1490:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "SetextHeading1"));
  {  int yypos1497= G->pos, yythunkpos1497= G->thunkpos;  if (!yy_RawLine(G)) { goto l1496; }  if (!yy_SetextBottom1(G)) { goto l1496; }  G->pos= yypos1497; G->thunkpos= yythunkpos1497;
  }  if (!yy_LocMarker(G)) { goto l1496; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1496;
  {  int yypos1500= G->pos, yythunkpos1500= G->thunkpos;  if (!yy_Endline(G)) { goto l1500; }  goto l1496;
  l1500:;	  G->pos= yypos1500; G->thunkpos= yythunkpos1500;
  }  if (!yy_Inline(G)) { goto l1496; }
  l1498:;	
  {  int yypos1499= G->pos, yythunkpos1499= G->thunkpos;
  {  int yypos1501= G->pos, yythunkpos1501= G->thunkpos;  if (!yy_Endline(G)) { goto l1501; }  goto l1499;
  l1501:;	  G->pos= yypos1501; G->thunkpos= yythunkpos1501;
  }  if (!yy_Inline(G)) { goto l1499; }  goto l1498;
  l1499:;	  G->pos= yypos1499; G->thunkpos= yythunkpos1499;
  }  if (!yy_Sp(G)) { goto l1496; }  if (!yy_Newline(G)) { goto l1496; }  if (!yy_SetextBottom1(G)) { goto l1496; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1496;  yyDo(G, yy_1_SetextHeading1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading1", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1496:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextHeading"));
  {  int yypos1503= G->pos, yythunkpos1503= G->thunkpos;  if (!yy_SetextHeading1(G)) { goto l1504; }  goto l1503;
  l1504:;	  G->pos= yypos1503; G->thunkpos= yythunkpos1503;  if (!yy_SetextHeading2(G)) { goto l1502; }
  }
  l1503:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading", G->buf+G->pos));
  return 1;
  l1502:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Space(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Space"));  if (!yy_Spacechar(G)) { goto l1505; }
  l1506:;	
  {  int yypos1507= G->pos, yythunkpos1507= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1507; }  goto l1506;
  l1507:;	  G->pos= yypos1507; G->thunkpos= yythunkpos1507;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Space", G->buf+G->pos));
  return 1;
  l1505:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Space", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxHeading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AtxHeading"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1508;  if (!yy_AtxStart(G)) { goto l1508; }  yyDo(G, yySet, -1, 0);  if (!yy_Space(G)) { goto l1508; }  if (!yy_AtxInline(G)) { goto l1508; }
  l1509:;	
  {  int yypos1510= G->pos, yythunkpos1510= G->thunkpos;  if (!yy_AtxInline(G)) { goto l1510; }  goto l1509;
  l1510:;	  G->pos= yypos1510; G->thunkpos= yythunkpos1510;
  }
  {  int yypos1511= G->pos, yythunkpos1511= G->thunkpos;  if (!yy_Sp(G)) { goto l1511; }
  l1513:;	
  {  int yypos1514= G->pos, yythunkpos1514= G->thunkpos;  if (!yymatchChar(G, '#')) goto l1514;  goto l1513;
  l1514:;	  G->pos= yypos1514; G->thunkpos= yythunkpos1514;
  }  if (!yy_Sp(G)) { goto l1511; }  goto l1512;
  l1511:;	  G->pos= yypos1511; G->thunkpos= yythunkpos1511;
  }
  l1512:;	  if (!yy_Newline(G)) { goto l1508; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1508;  yyDo(G, yy_1_AtxHeading, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxHeading", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1508:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxHeading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxStart(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AtxStart"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1515;
  {  int yypos1516= G->pos, yythunkpos1516= G->thunkpos;  if (!yymatchString(G, "######")) goto l1517;  goto l1516;
  l1517:;	  G->pos= yypos1516; G->thunkpos= yythunkpos1516;  if (!yymatchString(G, "#####")) goto l1518;  goto l1516;
  l1518:;	  G->pos= yypos1516; G->thunkpos= yythunkpos1516;  if (!yymatchString(G, "####")) goto l1519;  goto l1516;
  l1519:;	  G->pos= yypos1516; G->thunkpos= yythunkpos1516;  if (!yymatchString(G, "###")) goto l1520;  goto l1516;
  l1520:;	  G->pos= yypos1516; G->thunkpos= yythunkpos1516;  if (!yymatchString(G, "##")) goto l1521;  goto l1516;
  l1521:;	  G->pos= yypos1516; G->thunkpos= yythunkpos1516;  if (!yymatchChar(G, '#')) goto l1515;
  }
  l1516:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1515;  yyDo(G, yy_1_AtxStart, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxStart", G->buf+G->pos));
  return 1;
  l1515:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxStart", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Inline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Inline"));
  {  int yypos1523= G->pos, yythunkpos1523= G->thunkpos;  if (!yy_Str(G)) { goto l1524; }  goto l1523;
  l1524:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Endline(G)) { goto l1525; }  goto l1523;
  l1525:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_UlOrStarLine(G)) { goto l1526; }  goto l1523;
  l1526:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Space(G)) { goto l1527; }  goto l1523;
  l1527:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Strong(G)) { goto l1528; }  goto l1523;
  l1528:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Emph(G)) { goto l1529; }  goto l1523;
  l1529:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Strike(G)) { goto l1530; }  goto l1523;
  l1530:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Image(G)) { goto l1531; }  goto l1523;
  l1531:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Link(G)) { goto l1532; }  goto l1523;
  l1532:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_NoteReference(G)) { goto l1533; }  goto l1523;
  l1533:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_InlineNote(G)) { goto l1534; }  goto l1523;
  l1534:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Code(G)) { goto l1535; }  goto l1523;
  l1535:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_InlineEquation(G)) { goto l1536; }  goto l1523;
  l1536:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Mark(G)) { goto l1537; }  goto l1523;
  l1537:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_RawHtml(G)) { goto l1538; }  goto l1523;
  l1538:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Entity(G)) { goto l1539; }  goto l1523;
  l1539:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_EscapedChar(G)) { goto l1540; }  goto l1523;
  l1540:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yy_Symbol(G)) { goto l1522; }
  }
  l1523:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Inline", G->buf+G->pos));
  return 1;
  l1522:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Inline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Sp(GREG *G)
{
  yyprintf((stderr, "%s\n", "Sp"));
  l1542:;	
  {  int yypos1543= G->pos, yythunkpos1543= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1543; }  goto l1542;
  l1543:;	  G->pos= yypos1543; G->thunkpos= yythunkpos1543;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Sp", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_AtxInline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AtxInline"));
  {  int yypos1545= G->pos, yythunkpos1545= G->thunkpos;  if (!yy_Newline(G)) { goto l1545; }  goto l1544;
  l1545:;	  G->pos= yypos1545; G->thunkpos= yythunkpos1545;
  }
  {  int yypos1546= G->pos, yythunkpos1546= G->thunkpos;  if (!yy_Sp(G)) { goto l1546; }
  l1547:;	
  {  int yypos1548= G->pos, yythunkpos1548= G->thunkpos;  if (!yymatchChar(G, '#')) goto l1548;  goto l1547;
  l1548:;	  G->pos= yypos1548; G->thunkpos= yythunkpos1548;
  }  if (!yy_Sp(G)) { goto l1546; }  if (!yy_Newline(G)) { goto l1546; }  goto l1544;
  l1546:;	  G->pos= yypos1546; G->thunkpos= yythunkpos1546;
  }  if (!yy_Inline(G)) { goto l1544; }
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxInline", G->buf+G->pos));
  return 1;
  l1544:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxInline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Inlines(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Inlines"));
  {  int yypos1552= G->pos, yythunkpos1552= G->thunkpos;
  {  int yypos1554= G->pos, yythunkpos1554= G->thunkpos;  if (!yy_Endline(G)) { goto l1554; }  goto l1553;
  l1554:;	  G->pos= yypos1554; G->thunkpos= yythunkpos1554;
  }  if (!yy_Inline(G)) { goto l1553; }  goto l1552;
  l1553:;	  G->pos= yypos1552; G->thunkpos= yythunkpos1552;  if (!yy_Endline(G)) { goto l1549; }
  {  int yypos1555= G->pos, yythunkpos1555= G->thunkpos;  if (!yy_Inline(G)) { goto l1549; }  G->pos= yypos1555; G->thunkpos= yythunkpos1555;
  }
  }
  l1552:;	
  l1550:;	
  {  int yypos1551= G->pos, yythunkpos1551= G->thunkpos;
  {  int yypos1556= G->pos, yythunkpos1556= G->thunkpos;
  {  int yypos1558= G->pos, yythunkpos1558= G->thunkpos;  if (!yy_Endline(G)) { goto l1558; }  goto l1557;
  l1558:;	  G->pos= yypos1558; G->thunkpos= yythunkpos1558;
  }  if (!yy_Inline(G)) { goto l1557; }  goto l1556;
  l1557:;	  G->pos= yypos1556; G->thunkpos= yythunkpos1556;  if (!yy_Endline(G)) { goto l1551; }
  {  int yypos1559= G->pos, yythunkpos1559= G->thunkpos;  if (!yy_Inline(G)) { goto l1551; }  G->pos= yypos1559; G->thunkpos= yythunkpos1559;
  }
  }
  l1556:;	  goto l1550;
  l1551:;	  G->pos= yypos1551; G->thunkpos= yythunkpos1551;
  }
  {  int yypos1560= G->pos, yythunkpos1560= G->thunkpos;  if (!yy_Endline(G)) { goto l1560; }  goto l1561;
  l1560:;	  G->pos= yypos1560; G->thunkpos= yythunkpos1560;
  }
  l1561:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Inlines", G->buf+G->pos));
  return 1;
  l1549:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Inlines", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NonindentSpace(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NonindentSpace"));
  {  int yypos1563= G->pos, yythunkpos1563= G->thunkpos;  if (!yymatchString(G, "   ")) goto l1564;  goto l1563;
  l1564:;	  G->pos= yypos1563; G->thunkpos= yythunkpos1563;  if (!yymatchString(G, "  ")) goto l1565;  goto l1563;
  l1565:;	  G->pos= yypos1563; G->thunkpos= yythunkpos1563;  if (!yymatchChar(G, ' ')) goto l1566;  goto l1563;
  l1566:;	  G->pos= yypos1563; G->thunkpos= yythunkpos1563;  if (!yymatchString(G, "")) goto l1562;
  }
  l1563:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "NonindentSpace", G->buf+G->pos));
  return 1;
  l1562:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NonindentSpace", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Plain(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Plain"));  if (!yy_Inlines(G)) { goto l1567; }
  yyprintf((stderr, "  ok   %s @ %s\n", "Plain", G->buf+G->pos));
  return 1;
  l1567:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Plain", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Para(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Para"));  if (!yy_NonindentSpace(G)) { goto l1568; }  if (!yy_Inlines(G)) { goto l1568; }  if (!yy_BlankLine(G)) { goto l1568; }
  l1569:;	
  {  int yypos1570= G->pos, yythunkpos1570= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1570; }  goto l1569;
  l1570:;	  G->pos= yypos1570; G->thunkpos= yythunkpos1570;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Para", G->buf+G->pos));
  return 1;
  l1568:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Para", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StyleBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1571;  if (!yy_LocMarker(G)) { goto l1571; }  yyDo(G, yySet, -1, 0);  if (!yy_InStyleTags(G)) { goto l1571; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1571;
  l1572:;	
  {  int yypos1573= G->pos, yythunkpos1573= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1573; }  goto l1572;
  l1573:;	  G->pos= yypos1573; G->thunkpos= yythunkpos1573;
  }  yyDo(G, yy_1_StyleBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1571:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1574;  if (!yy_LocMarker(G)) { goto l1574; }  yyDo(G, yySet, -1, 0);
  {  int yypos1575= G->pos, yythunkpos1575= G->thunkpos;  if (!yy_HtmlBlockInTags(G)) { goto l1576; }  goto l1575;
  l1576:;	  G->pos= yypos1575; G->thunkpos= yythunkpos1575;  if (!yy_HtmlComment(G)) { goto l1577; }  goto l1575;
  l1577:;	  G->pos= yypos1575; G->thunkpos= yythunkpos1575;  if (!yy_HtmlBlockSelfClosing(G)) { goto l1574; }
  }
  l1575:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1574;  if (!yy_BlankLine(G)) { goto l1574; }
  l1578:;	
  {  int yypos1579= G->pos, yythunkpos1579= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1579; }  goto l1578;
  l1579:;	  G->pos= yypos1579; G->thunkpos= yythunkpos1579;
  }  yyDo(G, yy_1_HtmlBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1574:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Table(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Table"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_TABLE) )) goto l1580;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1580;  if (!yy_LocMarker(G)) { goto l1580; }  yyDo(G, yySet, -1, 0);  if (!yy_TableHeader(G)) { goto l1580; }  if (!yy_TableDelimiter(G)) { goto l1580; }
  l1581:;	
  {  int yypos1582= G->pos, yythunkpos1582= G->thunkpos;  if (!yy_TableLine(G)) { goto l1582; }  goto l1581;
  l1582:;	  G->pos= yypos1582; G->thunkpos= yythunkpos1582;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1580;  yyDo(G, yy_1_Table, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Table", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1580:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Table", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BulletList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "BulletList"));
  {  int yypos1584= G->pos, yythunkpos1584= G->thunkpos;  if (!yy_Bullet(G)) { goto l1583; }  G->pos= yypos1584; G->thunkpos= yythunkpos1584;
  }
  {  int yypos1585= G->pos, yythunkpos1585= G->thunkpos;  if (!yy_ListTight(G)) { goto l1586; }  goto l1585;
  l1586:;	  G->pos= yypos1585; G->thunkpos= yythunkpos1585;  if (!yy_ListLoose(G)) { goto l1583; }
  }
  l1585:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "BulletList", G->buf+G->pos));
  return 1;
  l1583:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BulletList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OrderedList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OrderedList"));
  {  int yypos1588= G->pos, yythunkpos1588= G->thunkpos;  if (!yy_Enumerator(G)) { goto l1587; }  G->pos= yypos1588; G->thunkpos= yythunkpos1588;
  }
  {  int yypos1589= G->pos, yythunkpos1589= G->thunkpos;  if (!yy_ListTight(G)) { goto l1590; }  goto l1589;
  l1590:;	  G->pos= yypos1589; G->thunkpos= yythunkpos1589;  if (!yy_ListLoose(G)) { goto l1587; }
  }
  l1589:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "OrderedList", G->buf+G->pos));
  return 1;
  l1587:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OrderedList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Heading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Heading"));
  {  int yypos1592= G->pos, yythunkpos1592= G->thunkpos;  if (!yy_SetextHeading(G)) { goto l1593; }  goto l1592;
  l1593:;	  G->pos= yypos1592; G->thunkpos= yythunkpos1592;  if (!yy_AtxHeading(G)) { goto l1591; }
  }
  l1592:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Heading", G->buf+G->pos));
  return 1;
  l1591:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Heading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HorizontalRule(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HorizontalRule"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1594;  if (!yy_NonindentSpace(G)) { goto l1594; }
  {  int yypos1595= G->pos, yythunkpos1595= G->thunkpos;  if (!yymatchChar(G, '*')) goto l1596;  if (!yy_Sp(G)) { goto l1596; }  if (!yymatchChar(G, '*')) goto l1596;  if (!yy_Sp(G)) { goto l1596; }  if (!yymatchChar(G, '*')) goto l1596;
  l1597:;	
  {  int yypos1598= G->pos, yythunkpos1598= G->thunkpos;  if (!yy_Sp(G)) { goto l1598; }  if (!yymatchChar(G, '*')) goto l1598;  goto l1597;
  l1598:;	  G->pos= yypos1598; G->thunkpos= yythunkpos1598;
  }  goto l1595;
  l1596:;	  G->pos= yypos1595; G->thunkpos= yythunkpos1595;  if (!yymatchChar(G, '-')) goto l1599;  if (!yy_Sp(G)) { goto l1599; }  if (!yymatchChar(G, '-')) goto l1599;  if (!yy_Sp(G)) { goto l1599; }  if (!yymatchChar(G, '-')) goto l1599;
  l1600:;	
  {  int yypos1601= G->pos, yythunkpos1601= G->thunkpos;  if (!yy_Sp(G)) { goto l1601; }  if (!yymatchChar(G, '-')) goto l1601;  goto l1600;
  l1601:;	  G->pos= yypos1601; G->thunkpos= yythunkpos1601;
  }  goto l1595;
  l1599:;	  G->pos= yypos1595; G->thunkpos= yythunkpos1595;  if (!yymatchChar(G, '_')) goto l1594;  if (!yy_Sp(G)) { goto l1594; }  if (!yymatchChar(G, '_')) goto l1594;  if (!yy_Sp(G)) { goto l1594; }  if (!yymatchChar(G, '_')) goto l1594;
  l1602:;	
  {  int yypos1603= G->pos, yythunkpos1603= G->thunkpos;  if (!yy_Sp(G)) { goto l1603; }  if (!yymatchChar(G, '_')) goto l1603;  goto l1602;
  l1603:;	  G->pos= yypos1603; G->thunkpos= yythunkpos1603;
  }
  }
  l1595:;	  if (!yy_Sp(G)) { goto l1594; }  if (!yy_Newline(G)) { goto l1594; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1594;  yyDo(G, yy_1_HorizontalRule, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HorizontalRule", G->buf+G->pos));
  return 1;
  l1594:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HorizontalRule", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Reference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 3, 0);
  yyprintf((stderr, "%s\n", "Reference"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1604;  if (!yy_LocMarker(G)) { goto l1604; }  yyDo(G, yySet, -3, 0);  if (!yy_NonindentSpace(G)) { goto l1604; }
  {  int yypos1605= G->pos, yythunkpos1605= G->thunkpos;  if (!yymatchString(G, "[]")) goto l1605;  goto l1604;
  l1605:;	  G->pos= yypos1605; G->thunkpos= yythunkpos1605;
  }  if (!yy_Label(G)) { goto l1604; }  yyDo(G, yySet, -2, 0);  if (!yymatchChar(G, ':')) goto l1604;  if (!yy_Spnl(G)) { goto l1604; }  if (!yy_RefSrc(G)) { goto l1604; }  yyDo(G, yySet, -1, 0);  if (!yy_RefTitle(G)) { goto l1604; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1604;  if (!yy_BlankLine(G)) { goto l1604; }
  l1606:;	
  {  int yypos1607= G->pos, yythunkpos1607= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1607; }  goto l1606;
  l1607:;	  G->pos= yypos1607; G->thunkpos= yythunkpos1607;
  }  yyDo(G, yy_1_Reference, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Reference", G->buf+G->pos));  yyDo(G, yyPop, 3, 0);
  return 1;
  l1604:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Reference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Note(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Note"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l1608;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1608;  if (!yy_LocMarker(G)) { goto l1608; }  yyDo(G, yySet, -1, 0);  if (!yy_NonindentSpace(G)) { goto l1608; }  if (!yy_RawNoteReference(G)) { goto l1608; }  if (!yymatchChar(G, ':')) goto l1608;  if (!yy_Sp(G)) { goto l1608; }  if (!yy_RawNoteBlock(G)) { goto l1608; }
  l1609:;	
  {  int yypos1610= G->pos, yythunkpos1610= G->thunkpos;
  {  int yypos1611= G->pos, yythunkpos1611= G->thunkpos;  if (!yy_Indent(G)) { goto l1610; }  G->pos= yypos1611; G->thunkpos= yythunkpos1611;
  }  if (!yy_RawNoteBlock(G)) { goto l1610; }  goto l1609;
  l1610:;	  G->pos= yypos1610; G->thunkpos= yythunkpos1610;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1608;  yyDo(G, yy_1_Note, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Note", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1608:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Note", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormula(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormula"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH) )) goto l1612;
  {  int yypos1613= G->pos, yythunkpos1613= G->thunkpos;  if (!yy_DisplayFormulaDollar(G)) { goto l1614; }  goto l1613;
  l1614:;	  G->pos= yypos1613; G->thunkpos= yythunkpos1613;  if (!yy_DisplayFormulaRaw(G)) { goto l1612; }
  }
  l1613:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormula", G->buf+G->pos));
  return 1;
  l1612:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormula", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "FencedCodeBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1615;  if (!yy_LocMarker(G)) { goto l1615; }  yyDo(G, yySet, -1, 0);
  {  int yypos1616= G->pos, yythunkpos1616= G->thunkpos;  if (!yy_FencedCodeBlockTick(G)) { goto l1617; }  goto l1616;
  l1617:;	  G->pos= yypos1616; G->thunkpos= yythunkpos1616;  if (!yy_FencedCodeBlockTidle(G)) { goto l1615; }
  }
  l1616:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1615;  yyDo(G, yy_1_FencedCodeBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1615:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Verbatim(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Verbatim"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1618;  if (!yy_LocMarker(G)) { goto l1618; }  yyDo(G, yySet, -1, 0);  if (!yy_VerbatimChunk(G)) { goto l1618; }
  l1619:;	
  {  int yypos1620= G->pos, yythunkpos1620= G->thunkpos;  if (!yy_VerbatimChunk(G)) { goto l1620; }  goto l1619;
  l1620:;	  G->pos= yypos1620; G->thunkpos= yythunkpos1620;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1618;  yyDo(G, yy_1_Verbatim, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Verbatim", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1618:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Verbatim", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlockQuote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "BlockQuote"));  if (!yy_BlockQuoteRaw(G)) { goto l1621; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_BlockQuote, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "BlockQuote", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1621:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlockQuote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlankLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "BlankLine"));  if (!yy_Sp(G)) { goto l1622; }  if (!yy_Newline(G)) { goto l1622; }
  yyprintf((stderr, "  ok   %s @ %s\n", "BlankLine", G->buf+G->pos));
  return 1;
  l1622:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlankLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FrontMatterEndMark(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FrontMatterEndMark"));
  {  int yypos1624= G->pos, yythunkpos1624= G->thunkpos;  if (!yymatchString(G, "---")) goto l1625;  goto l1624;
  l1625:;	  G->pos= yypos1624; G->thunkpos= yythunkpos1624;  if (!yymatchString(G, "...")) goto l1623;
  }
  l1624:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "FrontMatterEndMark", G->buf+G->pos));
  return 1;
  l1623:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FrontMatterEndMark", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FrontMatterBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FrontMatterBlock"));
  {  int yypos1627= G->pos, yythunkpos1627= G->thunkpos;  if (!yy_FrontMatterEndMark(G)) { goto l1627; }  goto l1626;
  l1627:;	  G->pos= yypos1627; G->thunkpos= yythunkpos1627;
  }
  l1628:;	
  {  int yypos1629= G->pos, yythunkpos1629= G->thunkpos;
  {  int yypos1630= G->pos, yythunkpos1630= G->thunkpos;  if (!yy_Newline(G)) { goto l1630; }  goto l1629;
  l1630:;	  G->pos= yypos1630; G->thunkpos= yythunkpos1630;
  }  if (!yymatchDot(G)) goto l1629;  goto l1628;
  l1629:;	  G->pos= yypos1629; G->thunkpos= yythunkpos1629;
  }  if (!yy_Newline(G)) { goto l1626; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FrontMatterBlock", G->buf+G->pos));
  return 1;
  l1626:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FrontMatterBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Newline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Newline"));
  {  int yypos1632= G->pos, yythunkpos1632= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1633;  goto l1632;
  l1633:;	  G->pos= yypos1632; G->thunkpos= yythunkpos1632;  if (!yymatchChar(G, '\r')) goto l1631;
  {  int yypos1634= G->pos, yythunkpos1634= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1634;  goto l1635;
  l1634:;	  G->pos= yypos1634; G->thunkpos= yythunkpos1634;
  }
  l1635:;	
  }
  l1632:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Newline", G->buf+G->pos));
  return 1;
  l1631:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Newline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LocMarker(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LocMarker"));
  {  int yypos1637= G->pos, yythunkpos1637= G->thunkpos;  if (!yymatchDot(G)) goto l1636;  G->pos= yypos1637; G->thunkpos= yythunkpos1637;
  }  yyDo(G, yy_1_LocMarker, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "LocMarker", G->buf+G->pos));
  return 1;
  l1636:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LocMarker", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Block(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Block"));
  l1639:;	
  {  int yypos1640= G->pos, yythunkpos1640= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1640; }  goto l1639;
  l1640:;	  G->pos= yypos1640; G->thunkpos= yythunkpos1640;
  }
  {  int yypos1641= G->pos, yythunkpos1641= G->thunkpos;  if (!yy_BlockQuote(G)) { goto l1642; }  goto l1641;
  l1642:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Verbatim(G)) { goto l1643; }  goto l1641;
  l1643:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_FencedCodeBlock(G)) { goto l1644; }  goto l1641;
  l1644:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_DisplayFormula(G)) { goto l1645; }  goto l1641;
  l1645:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Note(G)) { goto l1646; }  goto l1641;
  l1646:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Reference(G)) { goto l1647; }  goto l1641;
  l1647:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_HorizontalRule(G)) { goto l1648; }  goto l1641;
  l1648:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Heading(G)) { goto l1649; }  goto l1641;
  l1649:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_OrderedList(G)) { goto l1650; }  goto l1641;
  l1650:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_BulletList(G)) { goto l1651; }  goto l1641;
  l1651:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Table(G)) { goto l1652; }  goto l1641;
  l1652:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_HtmlBlock(G)) { goto l1653; }  goto l1641;
  l1653:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_StyleBlock(G)) { goto l1654; }  goto l1641;
  l1654:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Para(G)) { goto l1655; }  goto l1641;
  l1655:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;  if (!yy_Plain(G)) { goto l1638; }
  }
  l1641:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Block", G->buf+G->pos));
  return 1;
  l1638:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Block", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FrontMatter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "FrontMatter"));  if (!yy_LocMarker(G)) { goto l1656; }  yyDo(G, yySet, -1, 0);
  {  int yypos1657= G->pos, yythunkpos1657= G->thunkpos;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_FRONTMATTER) )) goto l1657;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1657;  if (!yymatchString(G, "---")) goto l1657;  if (!yy_Newline(G)) { goto l1657; }
  l1659:;	
  {  int yypos1660= G->pos, yythunkpos1660= G->thunkpos;  if (!yy_FrontMatterBlock(G)) { goto l1660; }  goto l1659;
  l1660:;	  G->pos= yypos1660; G->thunkpos= yythunkpos1660;
  }  if (!yy_FrontMatterEndMark(G)) { goto l1657; }  if (!yy_Newline(G)) { goto l1657; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1657;  yyDo(G, yy_1_FrontMatter, G->begin, G->end);  goto l1658;
  l1657:;	  G->pos= yypos1657; G->thunkpos= yythunkpos1657;
  }
  l1658:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "FrontMatter", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1656:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FrontMatter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Doc(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Doc"));  if (!yy_FrontMatter(G)) { goto l1661; }
  l1662:;	
  {  int yypos1663= G->pos, yythunkpos1663= G->thunkpos;  if (!yy_Block(G)) { goto l1663; }  goto l1662;
  l1663:;	  G->pos= yypos1663; G->thunkpos= yythunkpos1663;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Doc", G->buf+G->pos));
  return 1;
  l1661:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Doc", G->buf+G->pos));
  return 0;
}

#ifndef YY_PART

typedef int (*yyrule)(GREG *G);

YY_PARSE(int) YY_NAME(parse_from)(GREG *G, yyrule yystart)
{
  int yyok;
  if (!G->buflen)
    {
      G->buflen= YY_BUFFER_START_SIZE;
      G->buf= (char *)YY_ALLOC(G->buflen, G->data);
      G->textlen= YY_BUFFER_START_SIZE;
      G->text= (char *)YY_ALLOC(G->textlen, G->data);
      G->thunkslen= YY_STACK_SIZE;
      G->thunks= (yythunk *)YY_ALLOC(sizeof(yythunk) * G->thunkslen, G->data);
      G->valslen= YY_STACK_SIZE;
      G->vals= (YYSTYPE*)YY_ALLOC(sizeof(YYSTYPE) * G->valslen, G->data);
      G->begin= G->end= G->pos= G->limit= G->thunkpos= 0;
    }
  G->pos = 0;
  G->begin= G->end= G->pos;
  G->thunkpos= 0;
  G->val= G->vals;
  yyok= yystart(G);
  if (yyok) yyDone(G);
  yyCommit(G);
  return yyok;
  (void)yyrefill(NULL);
  (void)yymatchDot(NULL);
  (void)yymatchChar(NULL, 0);
  (void)yymatchString(NULL, NULL);
  (void)yymatchClass(NULL, NULL);
  (void)yyDo(NULL, NULL, 0, 0);
  (void)yyText(NULL, 0, 0);
  (void)yyDone(NULL);
  (void)yyCommit(NULL);
  (void)yyAccept(NULL, 0);
  (void)yyPush(NULL, NULL, 0, NULL, NULL);
  (void)yyPop(NULL, NULL, 0, NULL, NULL);
  (void)yySet(NULL, NULL, 0, NULL, NULL);
}

YY_PARSE(int) YY_NAME(parse)(GREG *G)
{
  return YY_NAME(parse_from)(G, yy_Doc);
}

YY_PARSE(GREG *) YY_NAME(parse_new)(YY_XTYPE data)
{
  GREG *G = (GREG *)YY_CALLOC(1, sizeof(GREG), G->data);
  G->data = data;
  return G;
}

YY_PARSE(void) YY_NAME(parse_free)(GREG *G)
{
  YY_FREE(G->buf);
  YY_FREE(G->text);
  YY_FREE(G->thunks);
  YY_FREE(G->vals);
  YY_FREE(G);
}

#endif




/* PEG Markdown Highlight
 * Copyright 2011-2016 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_parser_foot.c
 * 
 * Code to be appended to the end of the parser code generated from the
 * PEG grammar.
 */


static void _parse(parser_data *p_data, yyrule start_rule)
{
    GREG *g = YY_NAME(parse_new)(p_data);
    if (start_rule == NULL)
        YY_NAME(parse)(g);
    else
        YY_NAME(parse_from)(g, start_rule);
    YY_NAME(parse_free)(g);
    
    pmh_PRINTF("\n\n");
}

static void parse_markdown(parser_data *p_data)
{
    pmh_PRINTF("\nPARSING DOCUMENT: ");
    
    _parse(p_data, NULL);
}

static void parse_references(parser_data *p_data)
{
    pmh_PRINTF("\nPARSING REFERENCES: ");
    
    p_data->parsing_only_references = true;
    _parse(p_data, yy_References);
    p_data->parsing_only_references = false;
    
    p_data->references = p_data->head_elems[pmh_REFERENCE];
    p_data->head_elems[pmh_REFERENCE] = NULL;
}

